<?xml version="1.0" encoding="UTF-8"?>

<!--
This stylesheet converts XML documents conforming to the 2.0
version of the ceres/proofdatabase DTD to the 4.0 DTD.
-->

<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:output method="xml" version="1.0" encoding="UTF-8" doctype-system="http://www.logic.at/ceres/xml/4.0/proofdatabase.dtd"/> 
  
  <xsl:template match="/proofdatabase">
    <xsl:copy>
      <xsl:apply-templates select="definitionlist"/>
      <xsl:element name="axiomset"/>
      <xsl:apply-templates select="proof[*]"/>
      <xsl:apply-templates select="variabledefinitions"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="/ceres">
    <xsl:element name="proofdatabase">
      <xsl:apply-templates select="definitionlist"/>
      <xsl:apply-templates select="axiomset"/>
      <xsl:apply-templates select="proof"/>
      <xsl:element name="variabledefinitions"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="variablelist">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="variabledefinitions">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="formulalistvariabledef">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="formulavariabledef">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="axiomset">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="definitionlist">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="formuladef">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:copy-of select="@id"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="termdef">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:copy-of select="@id"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="indirecttermdef">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:copy-of select="@id"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="proof">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:copy-of select="@calculus"/>
      <xsl:if test="rule">
        <xsl:apply-templates />
      </xsl:if>
      <xsl:if test="sequent">
        <xsl:call-template name="sequenttoaxiomrule">
          <xsl:with-param name="sequent" select="sequent"/>
        </xsl:call-template>
      </xsl:if>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="substitution">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="variabletermpair">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="constant">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="variable">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="function">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:copy-of select="@definition"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="atomformula">
    <xsl:element name="constantatomformula">
      <xsl:copy-of select="@symbol"/>
      <xsl:copy-of select="@definition"/>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="conjunctiveformula">
    <xsl:copy>
      <xsl:copy-of select="@type"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="quantifiedformula">
    <xsl:copy>
      <xsl:copy-of select="@type"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="formulavariable">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="formulalistvariable">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="rule">
    <xsl:copy>
      <xsl:copy-of select="@type"/>
      <xsl:copy-of select="@symbol"/>
      <xsl:copy-of select="@param"/>
      <xsl:apply-templates select="sequent[1]"/>
      <!-- this converts sequents to axiom rules if necessary.
           it only supports rules with arity <= 2
	   is it possible to do this with xsl:for-each?
	   problem: the first sequent element does NOT
	   receive this treatment because it is the conclusion
	   of the rule -->
      <xsl:if test="name(*[2]) = 'sequent'">
	<xsl:call-template name="sequenttoaxiomrule">
	  <xsl:with-param name="sequent" select="*[2]"/>
	</xsl:call-template>
      </xsl:if>
      <xsl:if test="name(*[2]) = 'rule'">
	<xsl:apply-templates select="*[2]"/>
      </xsl:if>
      <xsl:if test="name(*[3]) = 'sequent'">
	<xsl:call-template name="sequenttoaxiomrule">
	  <xsl:with-param name="sequent" select="*[3]"/>
	</xsl:call-template>
      </xsl:if>
      <xsl:if test="name(*[3]) = 'rule'">
	<xsl:apply-templates select="*[3]"/>
      </xsl:if>
      <xsl:apply-templates select="prooflink"/>
      <xsl:apply-templates select="substitution"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template name="sequenttoaxiomrule">
    <xsl:param name="sequent"/>
    <xsl:element name="rule">
      <xsl:attribute name="type">axiom</xsl:attribute>
      <xsl:apply-templates select="$sequent"/>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="sequent">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="prooflink">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="formulalist">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>
