<?xml version="1.0" encoding="UTF-8"?>
<!--
This stylesheet converts XML documents conforming to the 4.0
version of the proofdatabase DTD to the 2.0 DTD. As the
2.0 DTD does not support second order syntax, this syntax (if present)
is converted in such a way that the second order objects are
represented as first order objects. Of course the resulting
XML document does not contain a valid proof in this case, but it
can be viewed in the prooftool and this visual representation
retains the proof property.
-->

<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:template match="sequentlist">
    <xsl:element name="proof">
      <xsl:copy-of select="@symbol"/>
        <xsl:call-template name="sequentstorule">
          <xsl:with-param name="sequents" select="sequent[*]"/>
        </xsl:call-template>
    </xsl:element>
  </xsl:template>
  
  <xsl:template name="sequentstorule">
    <xsl:param name="sequents"/>
    <xsl:if test="count($sequents) = 1">
      <xsl:apply-templates select="$sequents"/>
    </xsl:if>
    <xsl:if test="count($sequents) > 1">
      <xsl:element name="rule">
        <xsl:attribute name="type">cllist</xsl:attribute>
        <xsl:apply-templates select="$sequents[1]"/>
        <xsl:if test="count($sequents) > 2">
          <xsl:call-template name="sequentstorule">
            <xsl:with-param name="sequents" select="$sequents[position() &gt; 1]"/>
          </xsl:call-template>
        </xsl:if>
        <xsl:if test="count($sequents) = 2">
          <xsl:apply-templates select="$sequents[2]"/>
        </xsl:if>
      </xsl:element>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="variablelist">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="variabledefinitions">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="formulalistvariabledef">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="formulavariabledef">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="axiomset">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="definitionlist">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="formuladef">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:copy-of select="@id"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="termdef">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:copy-of select="@id"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="indirecttermdef">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:copy-of select="@id"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="proof">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:copy-of select="@calculus"/>
      <xsl:apply-templates select="rule"/>
      <xsl:apply-templates select="sequent"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="substitution">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="variabletermpair">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="constant">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="variable">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="function">
    <xsl:copy>
      <xsl:copy-of select="@symbol"/>
      <xsl:copy-of select="@definition"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="constantatomformula">
    <xsl:element name="atomformula">
      <xsl:copy-of select="@symbol"/>
      <xsl:copy-of select="@definition"/>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="conjunctiveformula">
    <xsl:copy>
      <xsl:copy-of select="@type"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="quantifiedformula">
    <xsl:copy>
      <xsl:copy-of select="@type"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="formulavariable">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="formulalistvariable">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="rule[@type='foralll2']|rule[@type='existsr2']">
    <xsl:copy>
      <xsl:copy-of select="@type"/>
      <xsl:copy-of select="@param"/>
      <!-- convert the lambda term to its string
           representation and put it into
           the symbol attribute of a first-order
           quantifier rule -->
      <xsl:attribute name="symbol">
	<xsl:value-of select="@symbol"/>
	<xsl:call-template name="lambdatotext">
	  <xsl:with-param name="lambdasubst" select="lambdasubstitution"/>
        </xsl:call-template>
      </xsl:attribute>
      <xsl:apply-templates />
    </xsl:copy>
  </xsl:template>

  <!-- Convert axiom and initial rule elements
       to sequent elements
  <xsl:template match="rule[@type='axiom']">
    <xsl:apply-templates select="sequent"/>
  </xsl:template>
  
  <xsl:template match="rule[@type='init']">
    <xsl:apply-templates select="sequent"/>
  </xsl:template> -->
  
  <!-- Convert last item in cllist to
       sequent element -->
  <xsl:template match="rule[@type='cllist']">
    <xsl:if test="not(rule[1])">
      <xsl:apply-templates select="sequent"/>
    </xsl:if>
    <xsl:if test="rule[1]">
      <xsl:copy>
        <xsl:copy-of select="@type"/>
        <xsl:copy-of select="@symbol"/>
        <xsl:copy-of select="@param"/>
        <xsl:apply-templates/>
      </xsl:copy>
    </xsl:if>
  </xsl:template>
  
  <!-- Convert last item in axset to
       sequent element
  <xsl:template match="rule[@type='axset']">
    <xsl:if test="not(rule[1])">
      <xsl:apply-templates select="sequent"/>
    </xsl:if>
    <xsl:if test="rule[1]">
      <xsl:copy>
        <xsl:copy-of select="@type"/>
        <xsl:copy-of select="@symbol"/>
        <xsl:copy-of select="@param"/>
        <xsl:apply-templates/>
      </xsl:copy>
    </xsl:if>
  </xsl:template> -->
  
  <xsl:template match="rule">
    <xsl:if test="not(rule[1] or prooflink[1])">
      <xsl:apply-templates select="sequent"/>
    </xsl:if>
    <xsl:if test="rule[1] or prooflink[1]">
      <xsl:copy>
        <xsl:copy-of select="@type"/>
        <xsl:copy-of select="@symbol"/>
        <xsl:copy-of select="@param"/>
        <xsl:apply-templates/>
      </xsl:copy>
    </xsl:if>
  </xsl:template>

  <xsl:template match="sequent">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="prooflink">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="secondorderquantifiedformula[@type='all2']">
    <xsl:element name="quantifiedformula">
      <xsl:attribute name="type">all</xsl:attribute>
      <xsl:call-template name="secondordervariabletovariable">
	<xsl:with-param name="secondordervariable" select="secondordervariable"/>
      </xsl:call-template>
      <xsl:apply-templates />
    </xsl:element>
  </xsl:template>

  <xsl:template match="secondorderquantifiedformula[@type='exists2']">
    <xsl:element name="quantifiedformula">
      <xsl:attribute name="type">exists</xsl:attribute>
      <xsl:call-template name="secondordervariabletovariable">
	<xsl:with-param name="secondordervariable" select="secondordervariable"/>
      </xsl:call-template>
      <xsl:apply-templates />
    </xsl:element>
  </xsl:template>

  <xsl:template match="secondordervariable"/>

  <xsl:template name="secondordervariabletovariable">
    <xsl:param name="secondordervariable"/>
    <xsl:element name="variable">
      <xsl:copy-of select="$secondordervariable/@symbol"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="variableatomformula">
    <xsl:element name="atomformula">
      <xsl:copy-of select="secondordervariable/@symbol"/>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="formulalist">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="lambdasubstitution"/>

  <!-- convert a lambdasubstitution element to
       its string representation -->

  <xsl:template name="lambdatotext"><xsl:param name="lambdasubst"/> λ<xsl:call-template name="variablelisttotext"><xsl:with-param name="variablelist" select="$lambdasubst/variablelist"/></xsl:call-template>.<xsl:call-template name="formulatotext"><xsl:with-param name="element" select="$lambdasubst/*[2]"/></xsl:call-template></xsl:template>

  <xsl:template name="variablelisttotext"><xsl:param name="variablelist"/><xsl:for-each select="$variablelist/*"> <xsl:value-of select="@symbol"/></xsl:for-each></xsl:template>

  <xsl:template name="formulatotext">
    <xsl:param name="element"/>
    <xsl:if test="name($element) = 'conjunctiveformula'">
      <xsl:call-template name="conjunctiveformulatotext">
	<xsl:with-param name="element" select="$element"/>
      </xsl:call-template>
    </xsl:if>
    <xsl:if test="name($element) = 'constantatomformula' or name($element) = 'variableatomformula' or name($element) = 'formulavariable'">
      <xsl:call-template name="atomformulatotext">
	<xsl:with-param name="element" select="$element"/>
      </xsl:call-template>
    </xsl:if>
    <xsl:if test="name($element) = 'quantifiedformula'">
      <xsl:call-template name="quantifiedformulatotext">
	<xsl:with-param name="element" select="$element"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <xsl:template name="conjunctiveformulatotext">
    <xsl:param name="element"/>
    <xsl:if test="$element/@type = 'neg'"> \neg </xsl:if>
    <xsl:call-template name="formulatotext">
      <xsl:with-param name="element" select="$element/*[1]"/>
    </xsl:call-template>
    <xsl:if test="$element/@type = 'and'"> \and </xsl:if>
    <xsl:if test="$element/@type = 'or'"> \or </xsl:if>
    <xsl:if test="$element/@type = 'impl'"> \impl </xsl:if>
    <xsl:if test="$element/*[2]">
      <xsl:call-template name="formulatotext">
        <xsl:with-param name="element" select="$element/*[2]"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <xsl:template name="quantifiedformulatotext">
    <xsl:param name="element"/>
    <xsl:if test="$element/@type = 'all'">\Forall</xsl:if>
    <xsl:if test="$element/@type = 'exists'">\Exists</xsl:if>
    <xsl:call-template name="formulatotext">
      <xsl:with-param name="element" select="$element/*[1]"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="atomformulatotext">
    <xsl:param name="element"/>
    <xsl:if test="$element/@symbol">
      <xsl:value-of select="$element/@symbol"/>
    </xsl:if>
    <xsl:if test="$element/secondordervariable">
      <xsl:value-of select="$element/secondordervariable/@symbol"/>
    </xsl:if>(<xsl:call-template name="termlisttotext">
      <xsl:with-param name="element" select="$element"/>
    </xsl:call-template>)</xsl:template>

  <xsl:template name="termlisttotext">
    <xsl:param name="element"/>
    <xsl:for-each select="$element/*">
      <xsl:if test="name(current()) = 'variable'">
	<xsl:value-of select="@symbol"/>
      </xsl:if>
      <xsl:if test="name(current()) = 'constant'">
	<xsl:value-of select="@symbol"/>
      </xsl:if>
      <xsl:if test="name(current()) = 'function'">
	<xsl:call-template name="termlisttotext">
	  <xsl:with-param name="element" select="current()"/>
	</xsl:call-template>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>
</xsl:stylesheet>
