////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./command/commandprocessor.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _COMMANDPROCESSOR_H_
#define _COMMANDPROCESSOR_H_

#include "util/threads.h"

class DrawPoint;
class DrawItem;
class GraphicMenu;

class CommandCallArg
{
public:
  virtual ~CommandCallArg() {}
};

class CommandCall
{
public:
  CommandCall( bool show_in_console = false )
    { m_show_in_console = show_in_console; m_wait_for_result = false; }
  virtual ~CommandCall() {}

  void SetWaitForResult( bool yes ) { m_wait_for_result = yes; }
  bool HaveToWaitForResult() const { return m_wait_for_result; }
  bool HaveToShowResultInConsole() const { return m_show_in_console; }

  virtual CommandCall* Apply() const = 0;
  virtual CommandCall* Apply( CommandCallArg *arg ) const = 0;

  virtual CommandCallArg* NewArgument( DrawItem *item,
                                       DrawPoint *pt ) const = 0;

  virtual GraphicMenu* GetResult() const = 0;

private:
  bool                   m_wait_for_result;
  bool                   m_show_in_console;
};

class CommandProcessor : protected TMutex
{
public:
  virtual void Process( CommandCall *cmdcall ) = 0;
  virtual void Process( const char *cmd, bool showInConsole ) = 0;

  virtual void BeginUserInput() = 0;
  virtual void EndUserInput() = 0;
};  

class CommandConsole
{
public:
  virtual void ShowMessage( const char *msg ) = 0;
  virtual void ShowError( const char *msg ) = 0;
  virtual void ShowPrompt( const char *msg ) = 0;
  virtual void ClearPrompt() = 0;
};

#endif

