////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./command/guile/commandimpl.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _COMMANDIMPL_H_
#define _COMMANDIMPL_H_

#include <libguile.h>

#include "util/myassert.h"
#include "chars/chars.h"

#include "command/command.h"

class Exception;
class ExecContext;

////////////////////////////////////////////////////////////////////////////////

class CommandImpl : public Command
{
public:
//  CommandImpl( ExecContext *ctxt );

//  virtual SLObject* Execute( SLObject *args );

//  ExecContext* GetContext() const { return m_ctxt; }
  static SCM ToSCM( const USTRING& str );
  static USTRING ToUString( const char* str );

protected:
//  virtual SLObject* Execute_() = 0;

  typedef void (*t_CMD_set_func) (CommandImpl*,SCM,short);

  typedef struct
  {
    const char *name;
    t_CMD_set_func setfunc;
    int num;
  }
  t_CMD_arg_list;

  virtual const t_CMD_arg_list* GetCmdArgList() const = 0;

/*
  void SetStringParam( SLString **store, SLObject* obj );
  void SetIntegerParam( SLInteger **store, SLObject* obj );
  void SetObjectParam( SLObject **store, SLObject* obj );
  void SetSymbolParam( SLSymbol **store, SLObject* obj );
  void SetListParam( SLList **store, SLObject* obj );
*/

protected:
  void CheckAndSetParameter( SCM args );
  
  static void HandleException( Exception *e, const char *func );

private:
//  ExecContext           *m_ctxt;
};

#define CMD_COMMAND_NAME( classname, name ) \
public: \
  static const char* GetFuncName() { return name; } \
  virtual const char* GetCommandName() const { return GetFuncName(); } \
  static void Register() \
  { \
    scm_c_define_gsubr( name, 0, 0, 1, \
                        (scm_unused_struct*(*)()) classname::Execute_ ); \
  } \
  static SCM Execute_( SCM args ) \
  { \
    try \
    { \
      classname *cmd = new classname; \
      cmd->CheckAndSetParameter( args ); \
      return cmd->Execute(); \
    } \
    catch( Exception *e ) \
    { \
      HandleException( e, GetFuncName() ); \
    } \
    catch( ... ) \
    { \
      scm_error( scm_misc_error_key, name, \
                 "UNKNOWN EXCEPTION CAUGHT!", SCM_BOOL_F, SCM_BOOL_F ); \
    } \
    return SCM_UNSPECIFIED; \
  } \
protected: \
  virtual const t_CMD_arg_list* GetCmdArgList() const \
    { return m_CMD_arg_list; } \
  static t_CMD_arg_list m_CMD_arg_list[]; \
  typedef classname t_CMD_class_type; \
private:

////////////////////////////////////////////////////////////////////////////////

#define CMD_ARGUMENT_OBJ( name ) \
public: \
  SCM Get##name() const { return m_arg_##name; } \
protected: \
  static void Set##name( CommandImpl *cmd, SCM obj, short argn ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    mycmd->m_arg_##name = obj; \
  } \
private: \
  SCM m_arg_##name;

////////////////////////////////////////////////////////////////////////////////

//const char* Get##name() const { return m_arg_str_##name; } 


#define CMD_ARGUMENT_STR( name ) \
public: \
  USTRING Get##name() const { return ToUString(m_arg_str_##name); } \
  const char* GetCStr##name() const { return m_arg_str_##name; } \
protected: \
  static void Set##name( CommandImpl *cmd, SCM obj, short argn ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    SCM_ASSERT( SCM_STRINGP(obj), obj, argn, GetFuncName() ); \
    mycmd->m_arg_##name = obj; \
    mycmd->m_arg_str_##name = SCM_STRING_CHARS( obj ); \
  } \
private: \
  SCM m_arg_##name; \
  const char* m_arg_str_##name;

////////////////////////////////////////////////////////////////////////////////

#define CMD_ARGUMENT_SYM( name ) \
public: \
  const char* Get##name() const { return m_arg_str_##name; } \
protected: \
  static void Set##name( CommandImpl *cmd, SCM obj, short argn ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    SCM_ASSERT( SCM_SYMBOLP(obj), obj, argn, GetFuncName() ); \
    mycmd->m_arg_##name = obj; \
    mycmd->m_arg_str_##name = SCM_SYMBOL_CHARS( obj ); \
  } \
private: \
  SCM m_arg_##name; \
  const char *m_arg_str_##name; \

////////////////////////////////////////////////////////////////////////////////

#define CMD_ARGUMENT_INT( name ) \
public: \
  int Get##name() const { return m_arg_int_##name; } \
protected: \
  static void Set##name( CommandImpl *cmd, SCM obj, short argn ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    mycmd->m_arg_##name = obj; \
    mycmd->m_arg_int_##name = scm_num2int( obj, argn, GetFuncName() ); \
  } \
private: \
  SCM m_arg_##name; \
  int m_arg_int_##name; \

////////////////////////////////////////////////////////////////////////////////

#define CMD_ARGUMENT_DBL( name ) \
public: \
  double Get##name() const { return m_arg_dbl_##name; } \
protected: \
  static void Set##name( CommandImpl *cmd, SCM obj, short argn ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    SCM_ASSERT( SCM_NUMP(obj), obj, argn, GetFuncName() ); \
    mycmd->m_arg_##name = obj; \
    mycmd->m_arg_dbl_##name = SCM_REAL_VALUE( obj ); \
  } \
private: \
  SCM m_arg_##name; \
  double m_arg_dbl_##name; \

#if 0
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define CMD_COMMAND_NAME( classname, name ) \
public: \
  classname( ExecContext* ctxt ) : CommandImpl( ctxt ) {} \
  virtual const char* GetCommandName() const { return GetName(); } \
  static const char* GetName() { return name; } \
  static Command *NewCommand( ExecContext *ctxt ) \
    { return new classname( ctxt); } \
protected: \
  virtual const t_CMD_arg_list* GetCmdArgList() const \
    { return m_CMD_arg_list; } \
  static t_CMD_arg_list m_CMD_arg_list[]; \
  typedef classname t_CMD_class_type; \
private:

#if 0
private: \
  classname( const classname &o ) {} \

#endif

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define CMD_ARGUMENT_OBJ( name ) \
public: \
  SLObject* Get##name() const { return m_arg_##name; } \
protected: \
  static void Set##name( CommandImpl *cmd, SLObject *obj ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    mycmd->SetObjectParam( &mycmd->m_arg_##name, obj ); \
  } \
private: \
  SLObject *m_arg_##name; \

////////////////////////////////////////////////////////////////////////////////

#define CMD_ARGUMENT_LIST( name ) \
public: \
  SLList* Get##name() const { return m_arg_##name; } \
protected: \
  static void Set##name( CommandImpl *cmd, SLObject *obj ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    mycmd->SetListParam( &mycmd->m_arg_##name, obj ); \
  } \
private: \
  SLList *m_arg_##name; \

////////////////////////////////////////////////////////////////////////////////

#define CMD_ARGUMENT_SYM( name ) \
public: \
  SLSymbol* Get##name() const { return m_arg_##name; } \
protected: \
  static void Set##name( CommandImpl *cmd, SLObject *obj ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    mycmd->SetSymbolParam( &mycmd->m_arg_##name, obj ); \
  } \
private: \
  SLSymbol *m_arg_##name; \

////////////////////////////////////////////////////////////////////////////////

#define CMD_ARGUMENT_STR( name ) \
public: \
  SLString* Get##name() const { return m_arg_##name; } \
protected: \
  static void Set##name( CommandImpl *cmd, SLObject *obj ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    mycmd->SetStringParam( &mycmd->m_arg_##name, obj ); \
  } \
private: \
  SLString *m_arg_##name; \

////////////////////////////////////////////////////////////////////////////////

#define CMD_ARGUMENT_INT( name ) \
public: \
  SLInteger* Get##name() const { return m_arg_##name; } \
protected: \
  static void Set##name( CommandImpl *cmd, SLObject *obj ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    mycmd->SetIntegerParam( &mycmd->m_arg_##name, obj ); \
  } \
private: \
  SLInteger *m_arg_##name; \


#endif

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define CMD_DECLARE_ARG_LIST( classname ) \
CommandImpl::t_CMD_arg_list classname::m_CMD_arg_list[] = \
  {

////////////////////////////////////////////////////////////////////////////////

#define CMD_ARG_LIST_ENTRY( name, position ) \
    { #name, &Set##name, position }, \
  
////////////////////////////////////////////////////////////////////////////////

#define CMD_END_DECLARE_ARG_LIST \
    { NULL, NULL, 0 } \
  }; \
  
////////////////////////////////////////////////////////////////////////////////

#define CMD_DECLARE_EMPTY_ARG_LIST( classname ) \
CMD_DECLARE_ARG_LIST( classname ) \
CMD_END_DECLARE_ARG_LIST

////////////////////////////////////////////////////////////////////////////////

#endif
