////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./command/guile/guilecommandcall.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#include "graphic/proofsheet.h"

#include "guiletypes.h"
#include "guilecommandcall.h"
#include "menucommands.h"

/******************************************************************************/

#if 0
static char* PtrToStr( void* ptr )
{
  static char str[16];

  sprintf( str, "0x%lX", (long) ptr );

  return str;
}

static void* StrToPtr( const char *str )
{
  ASSERT( str != NULL );

  if( *(str++) != '0' || *(str++) != 'x' )
    return NULL;

  long l = strtol( str, NULL, 16 );

  return (void*) l;
}
#endif

/******************************************************************************/

GuileCommandCallArg::GuileCommandCallArg( DrawItem *item, DrawPoint *pt )
{
  m_draw_item = item;
}

SCM GuileCommandCallArg::MakeSCM() const
{
  SCM scm_cmd = SCM_EOL;

  if( m_draw_item == NULL )
    return scm_cmd;

  if( DrawProofLinkItem *p = dynamic_cast<DrawProofLinkItem *>( m_draw_item ) )
  {
    scm_cmd = ScmObjProofLink::Make( p );
    return scm_cmd;
  }

  if( DrawFromProofLinkItem *p =
        dynamic_cast<DrawFromProofLinkItem *>( m_draw_item ) )
  {
    scm_cmd = ScmObjProofLink::Make( p );
    return scm_cmd;
  }

  if( DrawSequentItem *p = dynamic_cast<DrawSequentItem *>( m_draw_item ) )
  {
    const ProofLib::NodePointer &np = p->GetHighlightPart();

    scm_cmd = ScmObjSequent::Make( p, np );

    return scm_cmd;
  }

  return scm_cmd;
}

////////////////////////////////////////////////////////////////////////////////

GuileCommandCall::GuileCommandCall( bool show_in_console )
  : CommandCall( show_in_console )
{
  m_cond = NULL;
  m_cmd = SCM_UNSPECIFIED;
  m_result = SCM_UNSPECIFIED;
}

GuileCommandCall::~GuileCommandCall()
{
  if( m_cmd != SCM_UNSPECIFIED )
    scm_gc_unprotect_object( m_cmd );
  if( m_result != SCM_UNSPECIFIED )
    scm_gc_unprotect_object( m_result );
}

void GuileCommandCall::SetResultSCM( SCM result )
{
  if( m_result != SCM_UNSPECIFIED )
    scm_gc_unprotect_object( m_result );

  m_result = scm_gc_protect_object(result);
}

void GuileCommandCall::SetSCM( SCM cmd )
{
  if( m_cmd != SCM_UNSPECIFIED )
    scm_gc_unprotect_object( m_cmd );

  m_cmd = scm_gc_protect_object(cmd);
}

SCM GuileCommandCall::GetCallSCM()
{
  while( !m_apply_stack.empty() )
  {
    if( m_apply_stack.top() == NULL )
      SetSCMApply( SCM_EOL );
    else
    {
      GuileCommandCallArg *garg = m_apply_stack.top();

      SetSCMApply( scm_cons( garg->MakeSCM(), SCM_EOL ) );

      delete garg;
    }

    m_apply_stack.pop();
  }

  return m_cmd;
}

CommandCall* GuileCommandCall::Apply() const
{
  GuileCommandCall *newcall = new GuileCommandCall;
  newcall->SetSCM( m_cmd );
  newcall->m_apply_stack.push( NULL );
  return newcall;
}

CommandCall* GuileCommandCall::Apply( CommandCallArg *arg ) const
{
  GuileCommandCallArg *garg = dynamic_cast<GuileCommandCallArg*>( arg );
  
  ASSERT( garg != NULL );

  GuileCommandCall *newcall = new GuileCommandCall;
  newcall->SetSCM( m_cmd );
  newcall->m_apply_stack.push( garg );
  return newcall;
}

CommandCallArg*
GuileCommandCall::NewArgument( DrawItem *item, DrawPoint *pt ) const
{
  return new GuileCommandCallArg( item, pt );
}

void GuileCommandCall::SetSCMApply( SCM args )
{
//  GuileCommandCall *newcall = new GuileCommandCall;

  SetSCM(
    scm_cons( scm_sym_apply, scm_list_2(
      m_cmd,
      scm_list_2( scm_sym_quote, args )
    ) ) );

//  return newcall;
}

GraphicMenu* GuileCommandCall::GetResult() const
{
  if( !SCM_MENU_P( m_result ) )
    return NULL;

  return ScmObjMenu::GetMenu( m_result );
}

