////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./command/guile/guiletypes.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#include "util/myassert.h"
//#include "util/exceptions.h"

//#include "guilecommandcall.h"

#include "graphic/proofsheet.h"

#include "commandimpl.h"
#include "guiletypes.h"

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

scm_t_bits ScmObjSequent::tag = 0;


SCM ScmObjSequent::Make( DrawSequentItem *sequent,
                         const ProofLib::NodePointer &np )
{
  t_data *data = new t_data;

  data->np = np;
  data->sequent = sequent;

  data->sequent = sequent;
  data->update_func = SCM_BOOL_F;

  SCM_RETURN_NEWSMOB( tag, data );
}

SCM ScmObjSequent::Mark( SCM data_smob )
{
  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  ////scm_gc_mark( data->sequent );

  return data->update_func;
}

size_t ScmObjSequent::Free( SCM data_smob )
{
  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  delete data;

  return sizeof( t_data );
}

int ScmObjSequent::Print( SCM data_smob, SCM port, scm_print_state *pstate )
{
//  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  scm_puts( "#<", port );
  scm_puts( OBJ_SNAME_SEQUENT, port );
#if 0
  scm_puts( " ", port );
  scm_puts( PtrToStr(data->sequent), port );
  scm_puts( ",", port );
  scm_display( SCM_MAKINUM(data->formula_no), port );
#endif
  scm_puts( ">", port );

  /* non-zero means success */
  return 1;
}

void ScmObjSequent::Init()
{
  ASSERT( tag == 0 );
  tag = scm_make_smob_type( OBJ_SNAME_SEQUENT, sizeof( t_data ) );
  scm_set_smob_mark( tag, Mark );
  scm_set_smob_free( tag, Free );
  scm_set_smob_print( tag, Print );

#if 0
  scm_c_define_gsubr( CMD_FNAME_SEQUENT_P,  1, 0, 0,
                      (SCM(*)()) IsASequent );
  scm_c_define_gsubr( CMD_FNAME_FORMULA_REF_P,  1, 0, 0,
                      (SCM(*)()) IsAFormula );
  scm_c_define_gsubr( CMD_FNAME_FORMULA_VAR_REF_P,  1, 0, 0,
                      (SCM(*)()) IsAFormulaVariable );
#endif
  scm_c_define_gsubr( CMD_FNAME_SEQUENT_GET_PARENT,  1, 0, 0,
                      (SCM(*)()) GetParent );

  scm_c_define_gsubr( CMD_FNAME_SEQUENT_P,  1, 0, 0,
                      (SCM(*)()) (SCM(*)(SCM)) IsAType<ProofLib::Sequent> );
  scm_c_define_gsubr( CMD_FNAME_SELEMENT_REF_P,  1, 0, 0,
                      (SCM(*)()) (SCM(*)(SCM)) IsAType<ProofLib::SElement> );
  scm_c_define_gsubr( CMD_FNAME_SFORMULALIST_VAR_REF_P,  1, 0, 0,
                      (SCM(*)()) (SCM(*)(SCM))
                                      IsAType<ProofLib::SFormulaListVariable> );
  scm_c_define_gsubr( CMD_FNAME_FORMULA_REF_P,  1, 0, 0,
                      (SCM(*)()) (SCM(*)(SCM)) IsAType<ProofLib::Formula> );
  scm_c_define_gsubr( CMD_FNAME_FORMULA_VAR_REF_P,  1, 0, 0,
                      (SCM(*)()) (SCM(*)(SCM))
                                      IsAType<ProofLib::FormulaVariable> );
}

SCM ScmObjSequent::GetParent( SCM obj )
{
  if( !SCM_SEQUENT_P( obj ) )
    return SCM_BOOL_F;

  t_data *data = (t_data *) SCM_SMOB_DATA( obj );

  if( data->np.IsEmpty() )
    return SCM_UNDEFINED;

  ProofLib::NodePointer np = data->np;
  np.Pop();

  return Make( data->sequent, np );
}

#if 0
SCM ScmObjSequent::IsASequent( SCM obj )
{
  if( !SCM_SEQUENT_P( obj ) )
    return SCM_BOOL_F;

  t_data *data = (t_data *) SCM_SMOB_DATA( obj );

  if( data->np.IsEmpty() )
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

SCM ScmObjSequent::IsAFormula( SCM obj )
{
  if( !SCM_SEQUENT_P( obj ) )
    return SCM_BOOL_F;

  t_data *data = (t_data *) SCM_SMOB_DATA( obj );

  if( dynamic_cast<ProofLib::Formula*>
      (data->np.GetDataNode( data->sequent->GetSequent() ))
      != NULL )
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}

SCM ScmObjSequent::IsAFormulaVariable( SCM obj )
{
  if( !SCM_SEQUENT_P( obj ) )
    return SCM_BOOL_F;

  t_data *data = (t_data *) SCM_SMOB_DATA( obj );

  if( dynamic_cast<ProofLib::FormulaVariable*>
      (data->np.GetDataNode( data->sequent->GetSequent() ))
      != NULL )
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}
#endif


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

scm_t_bits ScmObjProofLink::tag = 0;


SCM ScmObjProofLink::Make( DrawProofLinkItem *link )
{
  DrawToProofLinkItem *up = dynamic_cast<DrawToProofLinkItem*>( link );

  if( up != NULL )
    return Make( up );

  DrawFromProofLinkItem *down = dynamic_cast<DrawFromProofLinkItem*>( link );

  if( down != NULL )
    return Make( down );

  return SCM_EOL;
}

SCM ScmObjProofLink::Make( DrawToProofLinkItem *uplink )
{
  t_data *data = (t_data *) scm_must_malloc( sizeof( t_data ),
                                             OBJ_SNAME_PROOFLINK );

  data->uplink = uplink;
  data->downlink = NULL;
  data->update_func = SCM_BOOL_F;

  SCM_RETURN_NEWSMOB( tag, data );
}

SCM ScmObjProofLink::Make( DrawFromProofLinkItem *downlink )
{
  t_data *data = (t_data *) scm_must_malloc( sizeof( t_data ),
                                             OBJ_SNAME_PROOFLINK );

  data->uplink = NULL;
  data->downlink = downlink;
  data->update_func = SCM_BOOL_F;

  SCM_RETURN_NEWSMOB( tag, data );
}

SCM ScmObjProofLink::Mark( SCM data_smob )
{
  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  ////scm_gc_mark( data->sequent );

  return data->update_func;
}

size_t ScmObjProofLink::Free( SCM data_smob )
{
  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  free( (void *) data );

  return sizeof( t_data );
}

int ScmObjProofLink::Print( SCM data_smob, SCM port, scm_print_state *pstate )
{
//  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  scm_puts( "#<", port );
  scm_puts( OBJ_SNAME_PROOFLINK, port );
#if 0
  scm_puts( " ", port );
  scm_puts( PtrToStr(data->sequent), port );
  scm_puts( ",", port );
  scm_display( SCM_MAKINUM(data->formula_no), port );
#endif
  scm_puts( ">", port );

  /* non-zero means success */
  return 1;
}

void ScmObjProofLink::Init()
{
  ASSERT( tag == 0 );
  tag = scm_make_smob_type( OBJ_SNAME_PROOFLINK, sizeof( t_data ) );
  scm_set_smob_mark( tag, Mark );
  scm_set_smob_free( tag, Free );
  scm_set_smob_print( tag, Print );

  scm_c_define_gsubr( CMD_FNAME_PROOFLINK_P,  1, 0, 0,
                      (SCM(*)()) IsA );
  scm_c_define_gsubr( CMD_FNAME_PROOFLINK_GETPROOFNAME,  1, 0, 0,
                      (SCM(*)()) GetProofName );
}

SCM ScmObjProofLink::GetProofName( SCM data_smob )
{
  SCM_ASSERT( scm_list_p( data_smob ), data_smob, SCM_ARG1,
              CMD_FNAME_PROOFLINK_GETPROOFNAME );

  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  if( data->uplink != NULL )
    return CommandImpl::ToSCM( data->uplink->GetProofName() );

  ASSERT( data->downlink != NULL );

  return CommandImpl::ToSCM( data->downlink->GetProofName() );
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

