////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./command/guile/guiletypes.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _GUILETYPES_H_
#define _GUILETYPES_H_

#include "util/myassert.h"

#include "command/commandnames.h"

#include <libguile.h>

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define SCM_SEQUENT_P(x)  SCM_SMOB_PREDICATE( ScmObjSequent::tag, x )

////////////////////////////////////////////////////////////////////////////////

class ScmObjSequent
{
public:
  static scm_t_bits tag;

  typedef struct
  {
    DrawSequentItem *sequent;
    ProofLib::NodePointer np;
  
    /* A function to call when this image is
       modified, e.g., to update the screen,
       or SCM_BOOL_F if no action necessary */
    SCM update_func;
  }
  t_data;
  
  static SCM Make( DrawSequentItem *sequent, const ProofLib::NodePointer &np );

  static SCM Mark( SCM data_smob );
  static size_t Free( SCM data_smob );
  static int Print( SCM data_smob, SCM port, scm_print_state *pstate );
  static SCM IsA( SCM obj )
    { return ( SCM_SEQUENT_P( obj ) ? SCM_BOOL_T : SCM_BOOL_F ); }

  static SCM GetParent( SCM data_smob );

/*
  static SCM IsASequent( SCM obj );
  static SCM IsAFormula( SCM obj );
  static SCM IsAFormulaVariable( SCM obj );
*/

  template<class T> static SCM IsAType( SCM obj )
  {
    if( !SCM_SEQUENT_P( obj ) )
      return SCM_BOOL_F;

    t_data *data = (t_data *) SCM_SMOB_DATA( obj );

    if( dynamic_cast<T*> (data->np.GetDataNode( data->sequent->GetSequent() ))
        != NULL )
      return SCM_BOOL_T;

    return SCM_BOOL_F;
  }

  static void Init();
};

////////////////////////////////////////////////////////////////////////////////

#define CMD_ARGUMENT_SEQUENT( name ) \
public: \
  SCM Get##name##SCM() const { return m_arg_##name; } \
  DrawSequentItem* Get##name() const \
  { \
    ScmObjSequent::t_data *data = (ScmObjSequent::t_data *) SCM_SMOB_DATA( m_arg_##name ); \
    return data->sequent; \
  } \
  ProofLib::NodePointer Get##name##NodePointer() const \
  { \
    ScmObjSequent::t_data *data = (ScmObjSequent::t_data *) SCM_SMOB_DATA( m_arg_##name ); \
    return data->np; \
  } \
protected: \
  static void Set##name( CommandImpl *cmd, SCM obj, short argn ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    SCM_ASSERT( SCM_SEQUENT_P(obj), obj, argn, GetFuncName() ); \
    mycmd->m_arg_##name = obj; \
  } \
private: \
  SCM m_arg_##name;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define SCM_PROOFLINK_P(x)  SCM_SMOB_PREDICATE( ScmObjProofLink::tag, x )

////////////////////////////////////////////////////////////////////////////////

class ScmObjProofLink
{
public:
  static scm_t_bits tag;

  typedef struct
  {
    DrawToProofLinkItem *uplink;
    DrawFromProofLinkItem *downlink;
  
    /* A function to call when this image is
       modified, e.g., to update the screen,
       or SCM_BOOL_F if no action necessary */
    SCM update_func;
  }
  t_data;
  
  static SCM Make( DrawProofLinkItem *link );
  static SCM Make( DrawToProofLinkItem *uplink );
  static SCM Make( DrawFromProofLinkItem *downlink );

  static SCM Mark( SCM data_smob );
  static size_t Free( SCM data_smob );
  static int Print( SCM data_smob, SCM port, scm_print_state *pstate );
  static SCM IsA( SCM obj )
    { return ( SCM_PROOFLINK_P( obj ) ? SCM_BOOL_T : SCM_BOOL_F ); }
  static SCM GetProofName( SCM data_smob );

  static void Init();
};

////////////////////////////////////////////////////////////////////////////////

#define CMD_ARGUMENT_PROOFLINK( name ) \
public: \
  SCM Get##nameSCM() const { return m_arg_##name; } \
  bool Is##name##Up() const \
  { \
    ScmObjProofLink::t_data *data = (ScmObjProofLink::t_data *) SCM_SMOB_DATA( m_arg_##name ); \
    return (data->uplink != NULL); \
  } \
  DrawToProofLinkItem* Get##name##Up() const \
  { \
    ScmObjProofLink::t_data *data = (ScmObjProofLink::t_data *) SCM_SMOB_DATA( m_arg_##name ); \
    return data->uplink; \
  } \
  DrawFromProofLinkItem* Get##name##From() const \
  { \
    ScmObjProofLink::t_data *data = (ScmObjProofLink::t_data *) SCM_SMOB_DATA( m_arg_##name ); \
    return data->downlink; \
  } \
protected: \
  static void Set##name( CommandImpl *cmd, SCM obj, short argn ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    SCM_ASSERT( SCM_PROOFLINK_P(obj), obj, argn, GetFuncName() ); \
    mycmd->m_arg_##name = obj; \
  } \
private: \
  SCM m_arg_##name;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#if 0
#define SCM_FORMULA_REF_P(x)  SCM_SMOB_PREDICATE( ScmObjFormulaRef::tag, x )

////////////////////////////////////////////////////////////////////////////////

class ScmObjFormulaRef
{
public:
  static scm_t_bits tag;

  typedef struct
  {
    DrawSequentItem *sequent;
    ProofLib::NodePointer np;
  
    /* A function to call when this image is
       modified, e.g., to update the screen,
       or SCM_BOOL_F if no action necessary */
    SCM update_func;
  }
  t_data;
  
  static SCM Make( DrawSequentItem *sequent, const ProofLib::NodePointer &np );

  static SCM Mark( SCM data_smob );
  static size_t Free( SCM data_smob );
  static int Print( SCM data_smob, SCM port, scm_print_state *pstate );
  static SCM IsA( SCM obj )
    { return ( SCM_FORMULA_REF_P( obj ) ? SCM_BOOL_T : SCM_BOOL_F ); }

  static void Init();
};

////////////////////////////////////////////////////////////////////////////////

#define CMD_ARGUMENT_FORMULA_REF( name ) \
public: \
  SCM Get##name() const { return m_arg_##name; } \
  DrawSequentItem* Get##name##Seq() const \
  { \
    ScmObjFormulaRef::t_data *data = (ScmObjFormulaRef::t_data *) SCM_SMOB_DATA( m_arg_##name ); \
    return data->sequent; \
  } \
  ProofLib::NodePointer Get##name##NodePointer() const \
  { \
    ScmObjFormulaRef::t_data *data = (ScmObjFormulaRef::t_data *) SCM_SMOB_DATA( m_arg_##name ); \
    return data->np; \
  } \
protected: \
  static void Set##name( CommandImpl *cmd, SCM obj, short argn ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    SCM_ASSERT( SCM_FORMULA_REF_P(obj), obj, argn, GetFuncName() ); \
    mycmd->m_arg_##name = obj; \
  } \
private: \
  SCM m_arg_##name;

#endif

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#endif
