////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./command/guile/menucommands.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
#include <iostream>
#endif

#include "util/myassert.h"
#include "util/exceptions.h"

//#include "lispprocessor.h"
//#include "lispexceptions.h"
#include "commandimpl.h"
#include "guilecommandcall.h"

/**/
#include "graphic/graphiccontext.h"
//#include "import/latexproof.h"
/**/

//#include "globalcommandregister.h"
#include "command/execcontext.h"
#include "command/commandnames.h"
#include "menucommands.h"

////////////////////////////////////////////////////////////////////////////////

scm_t_bits ScmObjMenu::tag = 0;
scm_t_bits ScmObjMenuItem::tag = 0;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

GraphicMenu* ScmObjMenu::GetMenu( SCM data_smob )
{
  ASSERT( SCM_MENU_P( data_smob ) );

  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  SCM args = data->sub_menus;

  GraphicMenu *menu = GetContext()->GetGraphicSystem()->NewMenu();

  while( scm_null_p( args ) != SCM_BOOL_T )
  {
    ASSERT( scm_list_p( args ) == SCM_BOOL_T );

    SCM arg = SCM_CAR( args );

    ASSERT( SCM_MENU_ITEM_P( arg ) );

    GraphicMenuItem *mi = ScmObjMenuItem::GetMenuItem( arg );

    if( mi == NULL )
      menu->AppendSeparator();
    else
      menu->Append( mi );

    args = SCM_CDR( args );
  }

  return menu;
}

SCM ScmObjMenu::Make( SCM args )
{
  SCM_ASSERT( scm_list_p( args ), args, SCM_ARG1, CMD_FNAME_MAKE_MENU );

  SCM menu_list = args;

  size_t argn = 1;

  while( scm_null_p( args ) != SCM_BOOL_T )
  {
#if 0
    scm_write( args, scm_current_output_port() );
    scm_display( scm_makfrom0str("\n"), scm_current_output_port() );
#endif

    ASSERT( scm_list_p( args ) == SCM_BOOL_T );

    SCM arg = SCM_CAR( args );

#if 0
    scm_write( arg, scm_current_output_port() );
    scm_display( scm_makfrom0str("\n"), scm_current_output_port() );
#endif

    SCM_ASSERT( SCM_MENU_ITEM_P( arg ), arg, argn, CMD_FNAME_MAKE_MENU );

/*
    if( ScmObjMenuItem::GetMenuItem( arg ) == NULL )
      data->menu->AppendSeparator();
    else
      data->menu->Append( ScmObjMenuItem::GetMenuItem( arg ) );
*/

    args = SCM_CDR( args );
    argn++;
  }

  t_data *data = (t_data *) scm_must_malloc( sizeof( t_data ),
                                             OBJ_SNAME_MENU );

//  data->menu = GetContext()->GetGraphicSystem()->NewMenu();
  data->update_func = SCM_BOOL_F;
  data->sub_menus = menu_list;

  SCM_RETURN_NEWSMOB( tag, data );
}

#if 0
SCM Clear( SCM data_smob )
{
  SCM_ASSERT( SCM_SMOB_PREDICATE( tag, data_smob ), data_smob,
              SCM_ARG1, CMD_FNAME_CLEAR_MENU );

  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  if( data->update_func != SCM_BOOL_F )
    scm_apply( data->update_func, SCM_EOL, SCM_EOL);

  return SCM_UNSPECIFIED;
}
#endif

SCM ScmObjMenu::Mark( SCM data_smob )
{
  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  scm_gc_mark( data->sub_menus );

  return data->update_func;
}

size_t ScmObjMenu::Free( SCM data_smob )
{
#ifdef DEBUG
using namespace std;
std::cout << "FREE: menu";
//cout.flush();
//Print( data_smob, scm_current_output_port(), NULL );
cout << endl;
#endif

  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  ::free( (void *) data );

  return sizeof( t_data );
}

int ScmObjMenu::Print( SCM data_smob, SCM port, scm_print_state *pstate )
{
  //t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  scm_puts( "#<", port );
  scm_puts( OBJ_SNAME_MENU, port );
  //scm_puts( " ", port );
  //scm_display( SCM_MAKINUM( (unsigned long) data->menu ), port );
  scm_puts( ">", port );

  /* non-zero means success */
  return 1;
}

void ScmObjMenu::Init()
{
  ASSERT( tag == 0 );
  tag = scm_make_smob_type( OBJ_SNAME_MENU, sizeof( t_data ) );
  scm_set_smob_mark( tag, Mark );
  scm_set_smob_free( tag, Free );
  scm_set_smob_print( tag, Print );

#if 0
  scm_c_define_gsubr( CMD_FNAME_CLEAR_MENU, 1, 0, 0, 
                      (scm_unused_struct*(*)()) Clear );
#endif
  scm_c_define_gsubr( CMD_FNAME_MAKE_MENU,  0, 0, 1,
                      (scm_unused_struct*(*)()) Make );
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

GraphicMenuItem* ScmObjMenuItem::GetMenuItem( SCM data_smob )
{
  ASSERT( SCM_MENU_ITEM_P( data_smob ) );

  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  // a separator?
  if( data->name == SCM_EOL )
    return NULL;

  USTRING str;
  LatexConvertor.GetUnicode( SCM_STRING_CHARS(data->name), str );

  GraphicMenuItem* menuitem =
    GetContext()->GetGraphicSystem()->NewMenuItem( str );

  if( SCM_MENU_P( data->func ) ) // submenu
    menuitem->SetSubMenu( ScmObjMenu::GetMenu( data->func ) );
  else
  {
    GuileCommandCall *cmdcall = new GuileCommandCall;
    cmdcall->SetSCM( data->func );
    menuitem->SetCmd( cmdcall );

    if( SCM_INUMP( data->key ) )
      menuitem->SetAccel( SCM_INUM( data->modifier ), SCM_INUM( data->key ) );
  }

  return menuitem;
}

SCM ScmObjMenuItem::MakeSeparator()
{
  t_data *data = (t_data *) scm_must_malloc( sizeof( t_data ),
                                             OBJ_SNAME_MENU_ITEM );

  data->name = SCM_EOL;
  data->func = SCM_EOL;

  data->update_func = SCM_BOOL_F;

  SCM_RETURN_NEWSMOB( tag, data );
}

SCM ScmObjMenuItem::Make( SCM name, SCM func, SCM modifier, SCM key )
{
  SCM_ASSERT( SCM_STRINGP( name ), name, SCM_ARG1,
              CMD_FNAME_MAKE_MENU_ITEM );

  if( modifier != SCM_UNDEFINED || key != SCM_UNDEFINED )
  {
    SCM_ASSERT( SCM_INUMP( modifier ), modifier, SCM_ARG3,
                CMD_FNAME_MAKE_MENU_ITEM );

    SCM_ASSERT( SCM_INUMP( key ), key, SCM_ARG4,
                CMD_FNAME_MAKE_MENU_ITEM );
  }

  t_data *data = (t_data *) scm_must_malloc( sizeof( t_data ),
                                             OBJ_SNAME_MENU_ITEM );

  data->name = name;
  data->func = func;
  data->modifier = modifier;
  data->key = key;

  data->update_func = SCM_BOOL_F;
#if 0
  USTRING str;
  LatexConvertor.GetUnicode( SCM_STRING_CHARS(name), str );

  data->menuitem = GetContext()->GetGraphicSystem()->NewMenuItem( str );

  data->update_func = SCM_BOOL_F;

  if( SCM_MENU_P( func ) ) // submenu
    data->menuitem->SetSubMenu( ScmObjMenu::GetMenu( func ) );
  else
  {
    GuileCommandCall *cmdcall = new GuileCommandCall;
    cmdcall->SetSCM( func );
    data->menuitem->SetCmd( cmdcall );
  }
#endif

  SCM_RETURN_NEWSMOB( tag, data );
}

#if 0
static SCM Clear( SCM data_smob )
{
  SCM_ASSERT( SCM_SMOB_PREDICATE( tag, data_smob ), data_smob,
              SCM_ARG1, CMD_FNAME_CLEAR_MENU_ITEM );

  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  if( data->update_func != SCM_BOOL_F )
    scm_apply( data->update_func, SCM_EOL, SCM_EOL);

  return SCM_UNSPECIFIED;
}
#endif

SCM ScmObjMenuItem::Mark( SCM data_smob )
{
  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  scm_gc_mark( data->name );
  scm_gc_mark( data->func );

  return data->update_func;
}

size_t ScmObjMenuItem::Free( SCM data_smob )
{
#ifdef DEBUG
using namespace std;
cout << "FREE: menuitem";
//cout.flush();
//Print( data_smob, scm_current_output_port(), NULL );
cout << endl;
#endif

  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  ::free( (void *) data );

  return sizeof( t_data );
}

int ScmObjMenuItem::Print( SCM data_smob, SCM port, scm_print_state *pstate )
{
  t_data *data = (t_data *) SCM_SMOB_DATA( data_smob );

  scm_puts( "#<", port );
  scm_puts( OBJ_SNAME_MENU_ITEM, port );
  scm_puts( " ", port );
  scm_display( data->name, port );
  scm_puts( ">", port );

  /* non-zero means success */
  return 1;
}

void ScmObjMenuItem::Init()
{
  ASSERT( tag == 0 );
  tag = scm_make_smob_type( OBJ_SNAME_MENU_ITEM, sizeof( t_data ) );
  scm_set_smob_mark( tag, Mark );
  scm_set_smob_free( tag, Free );
  scm_set_smob_print( tag, Print );

#if 0
  scm_c_define_gsubr( CMD_FNAME_CLEAR_MENU_ITEM, 1, 0, 0, 
                      (scm_unused_struct*(*)()) Clear );
#endif
  scm_c_define_gsubr( CMD_FNAME_MAKE_MENU_ITEM, 2, 2, 0,
                      (scm_unused_struct*(*)()) Make );
  scm_c_define_gsubr( CMD_FNAME_MAKE_MENU_ITEM_SEP, 0, 0, 0,
                      (scm_unused_struct*(*)()) MakeSeparator );
}

////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////

class CmdPopupMenu : public CommandImpl
{
  CMD_COMMAND_NAME( CmdPopupMenu, CMD_FNAME_POPUPMENU )

  CMD_ARGUMENT_MENU( Menu )
  CMD_ARGUMENT_INT( Button )

private:
  SCM Execute()
  {
    GraphicMainInputDispatcher *mi =
      GetContext()->GetGraphicSystem()->GetMainInputDispatcher();

    mi->Lock();
#ifdef DEBUG
    std::cout << "BEFORE POPUP" << std::endl;
#endif
    mi->PopupMenu( GetMenu(), GetButton() );
#ifdef DEBUG
    std::cout << "AFTER POPUP" << std::endl;
#endif
    mi->Unlock();

    return SCM_BOOL_T;
  }
};

CMD_DECLARE_ARG_LIST( CmdPopupMenu )
CMD_ARG_LIST_ENTRY( Menu, 0 )
CMD_ARG_LIST_ENTRY( Button, 1 )
CMD_END_DECLARE_ARG_LIST

////////////////////////////////////////////////////////////////////////////////

class CmdAddMenu : public CommandImpl
{
  CMD_COMMAND_NAME( CmdAddMenu, CMD_FNAME_ADDMENU )

  CMD_ARGUMENT_STR( Title )
  CMD_ARGUMENT_OBJ( Cmd )

private:
  SCM Execute()
  {
    GraphicMainInputDispatcher *mi =
      GetContext()->GetGraphicSystem()->GetMainInputDispatcher();

    mi->Lock();
#ifdef DEBUG
    std::cout << "BEFORE ADD" << std::endl;
#endif
    GraphicTopMenuItem *m =
      GetContext()->GetGraphicSystem()->NewTopMenuItem( GetTitle() );

    if( m == NULL )
    {
      GenericException *e = new GenericException();
      e->Stream() << "Can not create menu.";
      throw e;
    }

    GuileCommandCall *cmdcall = new GuileCommandCall;
    cmdcall->SetSCM( scm_gc_protect_object( GetCmd() ) );
    m->SetCmd( cmdcall );
    //GetContext()->GetProofWindow()->AppendToMenuBar( GetMenuItem() );
    if( GetContext()->GetProofWindow() != NULL )
      GetContext()->GetGraphicSystem()->
        SetTopMenu( GetContext()->GetProofWindow() );
#ifdef DEBUG
    std::cout << "AFTER ADD" << std::endl;
#endif
    mi->Unlock();

    return SCM_BOOL_T;
  }
};

CMD_DECLARE_ARG_LIST( CmdAddMenu )
CMD_ARG_LIST_ENTRY( Title, 0 )
CMD_ARG_LIST_ENTRY( Cmd, 1 )
CMD_END_DECLARE_ARG_LIST

////////////////////////////////////////////////////////////////////////////////

class CmdClearMenu : public CommandImpl
{
  CMD_COMMAND_NAME( CmdClearMenu, CMD_FNAME_CLEARMENU )

private:
  SCM Execute()
  {
    GraphicMainInputDispatcher *mi =
      GetContext()->GetGraphicSystem()->GetMainInputDispatcher();

    mi->Lock();
#ifdef DEBUG
    std::cout << "BEFORE CLEAR" << std::endl;
#endif
    GetContext()->GetGraphicSystem()->ClearTopMenu();
    
    if( GetContext()->GetProofWindow() != NULL )
      GetContext()->GetGraphicSystem()->
        SetTopMenu( GetContext()->GetProofWindow() );
#ifdef DEBUG
    std::cout << "AFTER CLEAR" << std::endl;
#endif
    mi->Unlock();

    return SCM_BOOL_T;
  }
};

CMD_DECLARE_EMPTY_ARG_LIST( CmdClearMenu )

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void RegisterMenuCommands()
{
  ScmObjMenu::Init();
  ScmObjMenuItem::Init();

  CmdAddMenu::Register();
  CmdClearMenu::Register();
  CmdPopupMenu::Register();
}

