////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./command/guile/menucommands.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _MENUCOMMANDS_H_
#define _MENUCOMMANDS_H_

////////////////////////////////////////////////////////////////////////////////

void RegisterMenuCommands();

class ScmObjMenu;
class ScmObjMenuItem;

////////////////////////////////////////////////////////////////////////////////

#define SCM_MENU_P(x)  SCM_SMOB_PREDICATE( ScmObjMenu::GetTag(), x )
#define SCM_MENU_ITEM_P(x)  SCM_SMOB_PREDICATE( ScmObjMenuItem::GetTag(), x )

////////////////////////////////////////////////////////////////////////////////

class ScmObjMenuItem
{
private:
  static scm_t_bits tag;

public:
  static scm_t_bits GetTag() { return tag; }

  typedef struct
  {
    SCM name;
    SCM func;
    SCM modifier;
    SCM key;
  
    /* A function to call when this image is
       modified, e.g., to update the screen,
       or SCM_BOOL_F if no action necessary */
    SCM update_func;
  }
  t_data;
  
  static GraphicMenuItem* GetMenuItem( SCM menuitem );
  static SCM MakeSeparator();
  //static SCM Make( SCM name, SCM func );
  static SCM Make( SCM name, SCM func, SCM modifier, SCM key );
  static SCM Mark( SCM data_smob );
  static size_t Free( SCM data_smob );
  static int Print( SCM data_smob, SCM port, scm_print_state *pstate );
  static void Init();
};

#define CMD_ARGUMENT_MENU_ITEM( name ) \
public: \
  GraphicMenuItem* Get##name() const { return m_arg_menu_##name; } \
protected: \
  static void Set##name( CommandImpl *cmd, SCM obj, short argn ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    SCM_ASSERT( SCM_MENU_ITEM_P(obj), obj, argn, GetFuncName() ); \
    mycmd->m_arg_##name = obj; \
    mycmd->m_arg_menu_##name = ScmObjMenuItem::GetMenuItem( obj ); \
  } \
private: \
  SCM m_arg_##name; \
  GraphicMenuItem *m_arg_menu_##name;

////////////////////////////////////////////////////////////////////////////////

class ScmObjMenu
{
private:
  static scm_t_bits tag;

public:
  static scm_t_bits GetTag() { return tag; }

  typedef struct
  {
    SCM sub_menus;
  
    /* A function to call when this image is
       modified, e.g., to update the screen,
       or SCM_BOOL_F if no action necessary */
    SCM update_func;
  }
  t_data;
  
  static GraphicMenu* GetMenu( SCM data_smob );
  static SCM Make( SCM args );
#if 0
  static SCM Clear( SCM data_smob );
#endif
  static SCM Mark( SCM data_smob );
  static size_t Free( SCM data_smob );
  static int Print( SCM data_smob, SCM port, scm_print_state *pstate );
  static void Init();
};

////////////////////////////////////////////////////////////////////////////////

#define CMD_ARGUMENT_MENU( name ) \
public: \
  GraphicMenu* Get##name() const { return m_arg_menu_##name; } \
protected: \
  static void Set##name( CommandImpl *cmd, SCM obj, short argn ) \
  { \
    t_CMD_class_type *mycmd = dynamic_cast<t_CMD_class_type*>( cmd ); \
    ASSERT( mycmd != NULL ); \
    SCM_ASSERT( SCM_MENU_P(obj), obj, argn, GetFuncName() ); \
    mycmd->m_arg_##name = obj; \
    mycmd->m_arg_menu_##name = ScmObjMenu::GetMenu( obj ); \
  } \
private: \
  SCM m_arg_##name; \
  GraphicMenu *m_arg_menu_##name;

////////////////////////////////////////////////////////////////////////////////

#endif
