////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./graphic/event.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#include <iostream> // for user prompts

#include "util/exceptions.h"
#include "command/execcontext.h"
#include "command/commandprocessor.h"

#include "graphiccontext.h"
#include "event.h"
#include "proofsheet.h"

void ResizeEvent::Call( GraphicInputListener *lsnr )
{
  lsnr->OnEvent( this );
}

void RepaintEvent::Call( GraphicInputListener *lsnr )
{
  lsnr->OnEvent( this );
}

void KeyInputEvent::Call( GraphicInputListener *lsnr )
{
  lsnr->OnEvent( this );
}

void MouseMoveEvent::Call( GraphicInputListener *lsnr )
{
  lsnr->OnEvent( this );
}

void ButtonClickEvent::Call( GraphicInputListener *lsnr )
{
  lsnr->OnEvent( this );
}

void DragDropEvent::Call( GraphicInputListener *lsnr )
{
  lsnr->OnEvent( this );
}

////////////////////////////////////////////////////////////////////////////////

void GraphicInputDispatcher::Attach( GraphicInputListener *lsnt )
{
  m_listener.push_back( lsnt );
}

void GraphicInputDispatcher::Detach( GraphicInputListener *lsnt )
{
  // TODO
}

void GraphicInputDispatcher::Fire( Event *e )
{
  GraphicInputListenerQueue::iterator i;

  for( i = m_listener.begin(); i != m_listener.end(); i++ )
    e->Call( *i );
}

////////////////////////////////////////////////////////////////////////////////

GraphicMainInputDispatcher::GraphicMainInputDispatcher( GraphicSystem *sys )
{
  ASSERT( sys != NULL );

  m_sys = sys;
  m_wait_for_sevent = false;
  m_abort_select_item = false;
  m_selected_item = NULL;
}

void GraphicMainInputDispatcher::Fire( Event *e )
{
//  Lock();

#if 0
  if( m_wait_for_sevent )
  {
    m_sevent = e;
    Quit();
#if 0
    m_sevent_ready.Broadcast();
    m_sevent_done.WaitOnMutex( *this );
#endif

//    if( m_sevent == NULL ) // event was already handled
//    {
//      Unlock();
//      return;
//    }
  }

//  Unlock();
#endif

  GraphicInputDispatcher::Fire( e );
}

DrawSequentItem* GraphicMainInputDispatcher::SelectSequentItem()
{
  DrawSequentItem *item = NULL;

  while( item == NULL )
  {
    GraphicView *v_ = m_sys->GetActiveProofWindow()->GetView();
    ASSERT( v_ != NULL );

    GraphicProofView *v = dynamic_cast<GraphicProofView*>(v_);
    ASSERT( v != NULL );

    v->m_highlight_func =
      DrawSequentItem::IsSequentWithMarkedType<ProofLib::Sequent>;

    item = dynamic_cast<DrawSequentItem*>( SelectItem( "Select sequent" ) );

    v->m_highlight_func = NULL;
  }

  return item;
}

#if 0
DrawSequentItem* GraphicMainInputDispatcher::SelectSElement( size_t *index )
{
  ASSERT( index != NULL );

  DrawSequentItem *item = NULL;

  while( item == NULL )
  {
    DrawPoint pt;

    DrawItem *item_ = SelectItem( "Select sequent element", &pt );

    if( item_ != NULL )
    {
      item = dynamic_cast<DrawSequentItem*>( item_ );

      if( item != NULL )
        *index = item->GetSElementIndexAt( pt );

      if( *index == 0 ) // no formula selected
        item = NULL;
    }
  }

  return item;
}
#endif

DrawSequentItem* GraphicMainInputDispatcher::SelectSElement()
{
  DrawSequentItem *item = NULL;

  while( item == NULL )
  {
    GraphicView *v_ = m_sys->GetActiveProofWindow()->GetView();
    ASSERT( v_ != NULL );

    GraphicProofView *v = dynamic_cast<GraphicProofView*>(v_);
    ASSERT( v != NULL );

    v->m_highlight_func =
      DrawSequentItem::IsSequentWithMarkedType<ProofLib::SElement>;

    item = DrawSequentItem::SequentWithMarkedType<ProofLib::SElement>
             ( SelectItem( "Select s-element" ) );

    v->m_highlight_func = NULL;
  }

  return item;
}

DrawSequentItem* GraphicMainInputDispatcher::SelectSFormulaListVar()
{
  DrawSequentItem *item = NULL;

  while( item == NULL )
  {
    GraphicView *v_ = m_sys->GetActiveProofWindow()->GetView();
    ASSERT( v_ != NULL );

    GraphicProofView *v = dynamic_cast<GraphicProofView*>(v_);
    ASSERT( v != NULL );

    v->m_highlight_func =
      DrawSequentItem::IsSequentWithMarkedType<ProofLib::SFormulaListVariable>;

    item = DrawSequentItem::
      SequentWithMarkedType<ProofLib::SFormulaListVariable>(
        SelectItem( "Select s-formula list variable" ) );


    v->m_highlight_func = NULL;
  }

  return item;
}

DrawSequentItem* GraphicMainInputDispatcher::SelectFormula()
{
  DrawSequentItem *item = NULL;

  while( item == NULL )
  {
    GraphicView *v_ = m_sys->GetActiveProofWindow()->GetView();
    ASSERT( v_ != NULL );

    GraphicProofView *v = dynamic_cast<GraphicProofView*>(v_);
    ASSERT( v != NULL );

    v->m_highlight_func =
      DrawSequentItem::IsSequentWithMarkedType<ProofLib::Formula>;

    item = DrawSequentItem::SequentWithMarkedType<ProofLib::Formula>(
      SelectItem( "Select formula" ) );


    v->m_highlight_func = NULL;
  }

  return item;
}

DrawSequentItem* GraphicMainInputDispatcher::SelectFormulaVar()
{
  DrawSequentItem *item = NULL;

  while( item == NULL )
  {
    GraphicView *v_ = m_sys->GetActiveProofWindow()->GetView();
    ASSERT( v_ != NULL );

    GraphicProofView *v = dynamic_cast<GraphicProofView*>(v_);
    ASSERT( v != NULL );

    v->m_highlight_func =
      DrawSequentItem::IsSequentWithMarkedType<ProofLib::FormulaVariable>;

    item = DrawSequentItem::SequentWithMarkedType<ProofLib::FormulaVariable>(
      SelectItem( "Select formula variable" ) );


    v->m_highlight_func = NULL;
  }

  return item;
}

DrawProofLinkItem* GraphicMainInputDispatcher::SelectProofLinkItem()
{
  DrawProofLinkItem *item = NULL;

  while( item == NULL )
  {
    GraphicView *v_ = m_sys->GetActiveProofWindow()->GetView();
    ASSERT( v_ != NULL );

    GraphicProofView *v = dynamic_cast<GraphicProofView*>(v_);
    ASSERT( v != NULL );

    v->m_highlight_func = (GraphicProofView::THighlightCheckFunction)
                            DrawItem::IsA<DrawProofLinkItem>;

    item = dynamic_cast<DrawProofLinkItem*>(
      SelectItem( "Select proof link" ) );


    v->m_highlight_func = NULL;
  }

  return item;
}

void GraphicMainInputDispatcher::BeginSelectingItem()
{
  ASSERT( !m_wait_for_sevent );
  m_wait_for_sevent = true;
  m_selected_item = NULL;
  m_abort_select_item = false;
}

void GraphicMainInputDispatcher::EndSelectingItem()
{
  ASSERT( m_wait_for_sevent );
  m_wait_for_sevent = false;
  m_selected_item = NULL;
  m_abort_select_item = false;
}

DrawItem* GraphicMainInputDispatcher::SelectItem( const char *prompt,
                                                  DrawPoint *pt )
{
  BeginSelectingItem();

  GetContext()->GetConsole()->ShowPrompt( prompt );

  while( !HasSelectedItem() )
  {
    RunTillItemSelected();

    if( HaveToAbortSelectItem() )
    {
      EndSelectingItem();
      throw new InterruptException;
    }
  }

  DrawItem *item = GetSelectedItem();

  EndSelectingItem();

  GetContext()->GetConsole()->ClearPrompt();

  return item;
#if 0
  DrawItem *item = NULL;

  Lock();

  m_wait_for_sevent = true;

  while( item == NULL )
  {
    m_sevent_ready.WaitOnMutex( *this );

    // abort select item?
    if( !m_wait_for_sevent || m_sevent == NULL )
    {
      Unlock();
      throw new InterruptException;
    }

    ASSERT( m_sevent != NULL );
    ButtonClickEvent *e = dynamic_cast<ButtonClickEvent*>( m_sevent ); 

    if( e != NULL )
    {
      GraphicView *v_ = m_sys->GetActiveProofWindow()->GetView();
      ASSERT( v_ != NULL );

      GraphicProofView *v = dynamic_cast<GraphicProofView*>(v_);
      ASSERT( v != NULL );

      DrawTransMatrix m = v->GetTransMatrix()->Inverse();

      Event *n = e->GetEventTransformedBy( m );
      e = dynamic_cast<ButtonClickEvent *>( n );
      ASSERT( e != NULL );

      item = v->GetProofSheet()->GetItemAtPoint( e->m_pt );
      if( pt != NULL )
        *pt = e->m_pt;

      m_sevent = NULL;

      delete n;
    }

    m_sevent_done.Broadcast();
  }

  m_wait_for_sevent = false;

  Unlock();

  return item;
#endif
}

void GraphicMainInputDispatcher::AbortSelectItem()
{
  m_abort_select_item = true;
#if 0
  Lock();
  m_wait_for_sevent = false;
  m_sevent_done.Broadcast();
  Unlock();
  m_sevent_ready.Broadcast();
#endif
}

void GraphicMainInputDispatcher::CallCommand( CommandCall *cc )
{
  ASSERT( cc != NULL );

  GetContext()->GetCommandProcessor()->Process( cc );
}

void GraphicMainInputDispatcher::OnEvent( KeyInputEvent *e )
{
#ifdef DEBUG
cout << "KEY EVENT " << (char) e->m_char << "( " << e->m_char <<  " ) - " <<  e->m_mods << endl;
#endif

  CommandCall *cmd = GetAccelDb()->LookupCommand( e->m_char, e->m_mods );

  if( cmd != NULL )
  {
//    cout << cmd << endl;
    GetContext()->GetCommandProcessor()->Process( cmd->Apply() );
  }

}

void GraphicMainInputDispatcher::PopupMenu( GraphicMenu *popup, size_t button )
{
  popup->Popup( button );
}

////////////////////////////////////////////////////////////////////////////////

void GraphicInputListener::AttachToDispatcher( GraphicInputDispatcher *disp )
{
  disp->Attach( this );
}

void GraphicInputListener::DetachFromDispatcher( GraphicInputDispatcher *disp )
{
  disp->Detach( this );
}


