////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./graphic/fontmanager.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#include <cmath> // for lround on readhat

#include "graphic/fontmanager.h"

#define RND( a ) lround( a )

////////////////////////////////////////////////////////////////////////////////

FontBitmapCache::FontBitmapCache()
{
  m_lastupdate  = 0;
  m_regular     = NULL;
  m_italic      = NULL;
  m_cal         = NULL;
#if 0
  m_sup         = NULL;
  m_sub         = NULL;
#endif
}

FontBitmapCache::~FontBitmapCache()
{
#if 0
cout << "destructor" << endl;
  if( m_regular != NULL )
    delete m_regular;
  if( m_italic != NULL )
    delete m_italic;
#endif
}

////////////////////////////////////////////////////////////////////////////////

CharMetrics::CharMetrics()
{
  m_width       = 0.0;
  m_height      = 0.0;
  m_advance     = 0.0;
}

////////////////////////////////////////////////////////////////////////////////

FontCharMetrics::FontCharMetrics()
{
  m_regular = NULL;
  m_italic  = NULL;
  m_cal     = NULL;
}

FontCharMetrics::~FontCharMetrics()
{
  if( m_regular != NULL )
    delete m_regular;
  if( m_italic != NULL )
    delete m_italic;
  if( m_cal != NULL )
    delete m_cal;
}

CharMetrics* FontCharMetrics::GetRegular()
{
  if( m_regular == NULL )
    m_regular = new CharMetrics;

  return m_regular;
}

CharMetrics* FontCharMetrics::GetItalic()
{
  if( m_italic == NULL )
    m_italic = new CharMetrics;

  return m_italic;
}

CharMetrics* FontCharMetrics::GetCal()
{
  if( m_cal == NULL )
    m_cal = new CharMetrics;

  return m_cal;
}

////////////////////////////////////////////////////////////////////////////////

FontBitmap *FontBitmapDatabase::GetBitmapOf( const DataChar &c, SUNIT height ) const
{
  return GetBitmapOf( c, (FUNIT) RND( height ) );
}

FontBitmap *FontBitmapDatabase::GetBitmapOf( const DataChar &c, FUNIT height ) const
{
  FontBitmap **ptr;

#if 0
  ASSERT( !(c.m_sub && c.m_sup) );

  if( c.m_sub )
    ptr = &(m_cache[height][c.m_char].m_sub);
  else if( c.m_sup )
    ptr = &(m_cache[height][c.m_char].m_sup);
  else
#endif
  if( c.m_cal )
    ptr = &(m_cache[height][c.m_char].m_cal);
  else if( c.m_italic )
    ptr = &(m_cache[height][c.m_char].m_italic);
  else
    ptr = &(m_cache[height][c.m_char].m_regular);

  if( *ptr == NULL )
  {
#ifdef DEBUG
    cout << "Cache miss: ";
    c.Dump( cout );
    cout << "/" << height << endl;
#endif
    *ptr = m_fm->MakeBitmap( c, height );
  }

  return *ptr;
}

const CharMetrics& FontBitmapDatabase::GetMetricsOf( const DataChar &c ) const
{
  CharMetrics *metrics;

  if( c.m_cal )
    metrics = m_metrics_cache[c.m_char].GetCal();
  else if( c.m_italic )
    metrics = m_metrics_cache[c.m_char].GetItalic();
  else
    metrics = m_metrics_cache[c.m_char].GetRegular();

  ASSERT( metrics != NULL );

  if( metrics->m_advance == 0 )
  {
#ifdef DEBUG
    std::cout << "Metrics Cache miss: ";
    c.Dump( std::cout );
    std::cout << std::endl;
#endif

    m_fm->MakeMetrics( c, *metrics );
  }

  ASSERT( metrics->m_advance != 0 );

  return *metrics;
}

FUNIT FontBitmapDatabase::GetWidth( const DataString &str, SUNIT height ) const
{
  return GetWidth( str, (FUNIT) RND( height ) );
}

FUNIT FontManager::GetSuperscriptHeight( FUNIT height )
{
  return (FUNIT) RND( double(height) / 2 );
}

FUNIT FontBitmapDatabase::GetWidth( const DataString &str, FUNIT height ) const
{
  return FUNIT( ceil(GetWidthHelper( str, height )) );
}

double FontBitmapDatabase::GetWidthHelper( const DataString &str,
                                           FUNIT height ) const
{
  double width = 0;

  if( str.IsEmpty() )
    return 0;

  for( size_t i = 0; i < str.GetLength(); i++ )
  {
    const CharMetrics& metrics = GetMetricsOf( str[i] );

#if 0
    if( i == str.GetLength() - 1 ) // last char
      width += std::max( metrics.m_width, metrics.m_advance ) * height;
    else
#endif
    if( ( i < str.GetLength() - 1 ) &&
        ( metrics.m_width > metrics.m_advance ) &&
        ( !str[i].HasSameStyle( str[i+1] ) || str[i].HasSubStrings() )
      )
    {
      width += metrics.m_width * height;
    }
    else
    {
      width += metrics.m_advance * height;
    }


    double sup_width = 0;
    double sub_width = 0;

    if( str[i].m_sup != NULL )
    {
      sup_width = GetWidthHelper( *(str[i].m_sup),
                                  FontManager::GetSuperscriptHeight( height ) );
    }
    if( str[i].m_sub != NULL )
    {
      sub_width = GetWidthHelper( *(str[i].m_sub),
                                  FontManager::GetSuperscriptHeight( height ) );
    }
    
    width += std::max( sup_width, sub_width );
  }

  return width;
}
