////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./graphic/fontmanager.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _FONTMANAGER_H_
#define _FONTMANAGER_H_

#include "graphic/bitmap.h"
#include "libproof.h"

typedef int FUNIT;

#define STD_HEIGHT 128

class GraphicSystem;
class FontManager;

  
class FontBitmap : public DrawBitmap
{
public:
  double                 m_advance;
  double                 m_full_advance;
  GUNIT                  m_preadj;
  GUNIT                  m_horz_offset;

  virtual GraphicContext* GetGraphicContext() const { return NULL; }
};

class CharMetrics
{
public:
  CharMetrics();

  double                 m_width;
  double                 m_height;
  double                 m_advance;
};

class FontCharMetrics
{
private:
  CharMetrics           *m_regular;
  CharMetrics           *m_italic;
  CharMetrics           *m_cal;

public:
  FontCharMetrics();
  ~FontCharMetrics();

  CharMetrics* GetRegular();
  CharMetrics* GetItalic();
  CharMetrics* GetCal();
};

class FontCharMetricsCache : public stlhash::hash_map<UCHAR,FontCharMetrics>
{
};

class FontBitmapCache
{
public:
  FontBitmapCache();
  //FontBitmapCache( FontBitmapCache &o );
  ~FontBitmapCache();

  time_t                 m_lastupdate;
  FontBitmap            *m_regular;
  FontBitmap            *m_italic;
  FontBitmap            *m_cal;
};

class FontBitmapCacheChars : public stlhash::hash_map<UCHAR,FontBitmapCache>
{
};

class FontBitmapCacheHeight : public stlhash::hash_map<FUNIT,FontBitmapCacheChars>
{
};

class FontBitmapDatabase
{
private:
  mutable FontBitmapCacheHeight  m_cache;
  mutable FontCharMetricsCache   m_metrics_cache;
  FontManager                   *m_fm;

public:
  FontBitmapDatabase( FontManager *mgr ) { m_fm = mgr; };

  const CharMetrics& GetMetricsOf( const DataChar &c ) const;

  FontBitmap *GetBitmapOf( const DataChar &c, FUNIT height ) const;
  FontBitmap *GetBitmapOf( const DataChar &c, SUNIT height ) const;
  FUNIT GetWidth( const DataString &str, FUNIT height ) const;
  FUNIT GetWidth( const DataString &str, SUNIT height ) const;

protected:
  double GetWidthHelper( const DataString &str, FUNIT height ) const;
};

class FontManager
{
protected:
  FontBitmapDatabase     m_fontdb;
  GraphicSystem         *m_sys;

protected:
  FontManager( GraphicSystem *sys ) : m_fontdb( this ) { m_sys = sys; };
public:
  virtual ~FontManager() {};

  virtual FontBitmap* MakeBitmap( const DataChar &c, FUNIT height ) = 0;
  virtual void MakeMetrics( const DataChar &c, CharMetrics &metrics ) = 0;

  size_t GetHorDPI() const { return 72; }
  size_t GetVerDPI() const { return 72; }

  FontBitmapDatabase& GetBitmapDatabase() { return m_fontdb; };

  virtual long GetCharCode( const DataChar &c, int &font_num ) = 0;

public:
  static FUNIT GetSuperscriptHeight( FUNIT height );

public:
  static FontManager* NewFontManager( GraphicSystem *sys );
};

#endif
