////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./graphic/freetype/freetype.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _FREETYPE_H_
#define _FREETYPE_H_

#include <ft2build.h>
#include FT_FREETYPE_H

#include "libproof.h"
#include "graphic/fontmanager.h"

#define FACE_ROMAN       0
#define FACE_ITALIC      1
#define FACE_SYMBOL      2
#define FACE_CALIGRAPH   3
#define FACE_LAST        3

std::string GetPathForFont( const char* fone_filename );

class FtFontManager;

class FtFace
{
public:
  typedef stlhash::hash_map< UCHAR, FT_ULong > UnicodeMap;

private:
  FT_Face                m_face;

  UnicodeMap            *m_unicode_map;

  std::string            m_filename;

public:
  FtFace();
  ~FtFace();

  FT_Face GetFace() const { return m_face; }

  void LoadFace( FtFontManager *m, const char *filename );
  void LoadKern( const char *filename );

  FT_ULong GetCharCode( UCHAR c ) const;
};


class FtFontManager : public FontManager
{
private:
  FT_Library             m_library;
  FtFace                 m_faces[4];

protected:
  FtFontManager( GraphicSystem *sys );

  friend class FontManager;

  void LoadFace( FtFace *face, const char *filename );
  void DrawOnBitmap( FontBitmap *bm, FT_Bitmap* bitmap, GraphicPoint pt );

  virtual long GetCharCode( const DataChar &c, int &font_num );

public:
  virtual FontBitmap* MakeBitmap( const DataChar &c, FUNIT height );
  virtual void MakeMetrics( const DataChar &c, CharMetrics &metrics );

public:
  FT_Library GetLibrary() const { return m_library; }
  FT_ULong GetCharCode( const DataChar &c, FtFace **face );
};

#endif
