////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./graphic/graphictypes.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _GRAPHICTYPES_H_
#define _GRAPHICTYPES_H_

//#include "data/datatypes.h"
//#include "drawtypes.h"
//#include "event.h"

#ifdef DEBUG
#include <ostream>

using namespace std;
#endif

class DrawPoint;
class DrawRect;

typedef int GUNIT;

class GraphicPoint
{
public:
  GraphicPoint() { m_x = 0; m_y = 0; };
  GraphicPoint( GUNIT x, GUNIT y ) { m_x = x; m_y = y; };
  GraphicPoint( const DrawPoint &pt );

  GUNIT   m_x;
  GUNIT   m_y;

  inline bool operator==( const GraphicPoint &o ) const
  {
    return ( m_x == o.m_x && m_y == o.m_y );
  }
  inline GraphicPoint operator+( const GraphicPoint &o ) const
  {
    return GraphicPoint( m_x + o.m_x, m_y + o.m_y );
  }
  inline GraphicPoint operator-( const GraphicPoint &o ) const
  {
    return GraphicPoint( m_x - o.m_x, m_y - o.m_y );
  }

#ifdef DEBUG
  ostream& Dump( ostream &out ) const
  {
    out << "(" << m_x << "," << m_y << ")";
    return out;
  }
#endif
};

class GraphicSize : public GraphicPoint
{
public:
  GraphicSize() : GraphicPoint() {};
  GraphicSize( GUNIT x, GUNIT y ) : GraphicPoint( x, y ) {};
  GraphicSize( const GraphicPoint &p ) : GraphicPoint( p ) {};

  inline GUNIT GetWidth() const { return m_x; };
  inline GUNIT GetHeight() const { return m_y; };

  inline bool Overlap( const GraphicSize& other ) const
  {
    return ( m_x > other.m_x && m_y > other.m_y );
  }

#ifdef DEBUG
  ostream& Dump( ostream &out ) const
  {
    out << "(" << m_x << "x" << m_y << ")";
    return out;
  }
#endif
};

class GraphicRect
{
public:
  GraphicSize         m_size;
  GraphicPoint        m_origin;

  GraphicRect() {}
  GraphicRect( GUNIT x, GUNIT y, GUNIT w, GUNIT h )
    : m_size( w, h ), m_origin( x, y ) {}
  GraphicRect( const DrawRect &rect );

  inline GUNIT GetX() const { return m_origin.m_x; }
  inline GUNIT GetY() const { return m_origin.m_y; }
  inline GUNIT GetWidth() const { return m_size.m_x; }
  inline GUNIT GetHeight() const { return m_size.m_y; }

  bool Contains( const GraphicPoint &pt ) const
  {
    if( pt.m_x < GetX() )
      return false;
    if( pt.m_x > GetX() + GetWidth() )
      return false;
    if( pt.m_y < GetY() )
      return false;
    if( pt.m_y > GetY() + GetHeight() )
      return false;
  
    return true;
  }
  bool Contains( const GraphicRect &rect ) const
  {
    if( !Contains( rect.m_origin ) )
      return false;
    if( !Contains( rect.m_origin + rect.m_size ) )
      return false;

    return true;
  }

  bool IsNull() const { return GetWidth() == 0 || GetHeight() == 0; }

#ifdef DEBUG
  ostream& Dump( ostream &out ) const
  {
    m_origin.Dump( out );
    out << " ";
    m_size.Dump( out );

    return out;
  }
#endif
};

#endif
