////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: chars/chars.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#include "chars.h"
#include "util/myassert.h"

USTRING::USTRING() {}
USTRING::USTRING( const wchar_t *s )
{
  if( s != NULL )
    std::wstring::operator = (s);
}

std::string USTRING::utf8() const
{
  std::string utf8;

  bool ret = UTF8FromUString( *this, utf8 );

  ASSERT( ret );

  return utf8;
}

void USTRING::FromUtf8( const char *s )
{
  bool ret = UStringFromUTF8( s, *this );
  ASSERT( ret );
}

void USTRING::FromUtf8( const std::string &s )
{
  bool ret = UStringFromUTF8( s.c_str(), *this );
  ASSERT( ret );
}

std::ostream& operator << ( std::ostream& s, const USTRING &u )
{
  s << u.utf8();
  return s;
}

bool UStringFromUTF8( const char *utf8_, USTRING &dest )
{
  const unsigned char *utf8 = (const unsigned char *) utf8_;

  dest.clear();

  if( utf8 == NULL )
    return true;
  
  size_t to_read_tails = 0;
  UCHAR c = CHAR( 0x0U );
  
  for( ; *utf8 != '\0'; utf8++ )
  {
  #define HAS_START_BITS( bits ) ( ( *utf8 & ( bits | (bits >> 1) ) ) == bits )
  #define REST_BITS( fixedbits ) ( ( *utf8 & ~fixedbits ) )
    if( to_read_tails > 0 )
    {
      if( !HAS_START_BITS( 0x80 ) )
        return false;
  
      c = (c << 6) + REST_BITS( 0x80 );
  
      to_read_tails--;
  
      if( to_read_tails == 0 )
      {
        dest += c;
        c = CHAR( 0x0U );
      }
    }
    else if( *utf8 <= 0x7fU )
    {
      dest += CHAR( *utf8 );
    }
    else if( HAS_START_BITS( 0xfc ) )
    {
      if( !REST_BITS( 0xfc ) )
        return false;
  
      c = REST_BITS( 0xfc );
  
      to_read_tails = 5;        
    }
    else if( HAS_START_BITS( 0xf8 ) )
    {
      if( !REST_BITS( 0xf8 ) )
        return false;
  
      c = REST_BITS( 0xf8 );
  
      to_read_tails = 4;        
    }
    else if( HAS_START_BITS( 0xf0 ) )
    {
      if( !REST_BITS( 0xf0 ) )
        return false;
  
      c = REST_BITS( 0xf0 );
  
      to_read_tails = 3;        
    }
    else if( HAS_START_BITS( 0xe0 ) )
    {
      if( !REST_BITS( 0xe0 ) )
        return false;
  
      c = REST_BITS( 0xe0 );
  
      to_read_tails = 2;        
    }
    else if( HAS_START_BITS( 0xc0 ) )
    {
      if( !REST_BITS( 0xc0 ) )
        return false;
  
      c = REST_BITS( 0xc0 );
  
      to_read_tails = 1;        
    }
    else
      return false;
  #undef HAS_START_BITS
  #undef REST_BITS
  }
  
  if( to_read_tails > 0 )
    return false;
  
  return true;
}

bool UTF8FromUChar( const UCHAR c, std::string &utf8 )
{
  utf8.clear();

  if( c <= CHAR( 0x7fU ) )
  {
    utf8 += (char) c;
  }
  else if( c <= CHAR( 0x7ffU ) )
  {
    utf8 += (char) ( (c >> 1 * 6) + 0xc0 );
    utf8 += (char) (((c >> 0 * 6 ) & 0x3f) + 0x80 );
  }
  else if( c <= CHAR( 0xffffU ) )
  {
    utf8 += (char) ( (c >> 2 * 6) + 0xe0 );
    utf8 += (char) (((c >> 1 * 6 ) & 0x3f) + 0x80 );
    utf8 += (char) (((c >> 0 * 6 ) & 0x3f) + 0x80 );
  }
  else if( c <= CHAR( 0x1fffffU ) )
  {
    utf8 += (char) ( (c >> 3 * 6) + 0xf0 );
    utf8 += (char) (((c >> 2 * 6 ) & 0x3f) + 0x80 );
    utf8 += (char) (((c >> 1 * 6 ) & 0x3f) + 0x80 );
    utf8 += (char) (((c >> 0 * 6 ) & 0x3f) + 0x80 );
  }
  else if( c <= CHAR( 0x3ffffffU ) )
  {
    utf8 += (char) ( (c >> 4 * 6) + 0xf8 );
    utf8 += (char) (((c >> 3 * 6 ) & 0x3f) + 0x80 );
    utf8 += (char) (((c >> 2 * 6 ) & 0x3f) + 0x80 );
    utf8 += (char) (((c >> 1 * 6 ) & 0x3f) + 0x80 );
    utf8 += (char) (((c >> 0 * 6 ) & 0x3f) + 0x80 );
  }
  else if( c <= CHAR( 0x7fffffffU ) )
  {
    utf8 += (char) ( (c >> 5 * 6) + 0xfc );
    utf8 += (char) (((c >> 4 * 6 ) & 0x3f) + 0x80 );
    utf8 += (char) (((c >> 3 * 6 ) & 0x3f) + 0x80 );
    utf8 += (char) (((c >> 2 * 6 ) & 0x3f) + 0x80 );
    utf8 += (char) (((c >> 1 * 6 ) & 0x3f) + 0x80 );
    utf8 += (char) (((c >> 0 * 6 ) & 0x3f) + 0x80 );
  }
  else
    return false;

  return true;
}

bool UTF8FromUString( const USTRING &src, std::string &utf8 )
{
  utf8.clear();

  for( size_t i = 0; i < src.length(); i++ )
  {
    std::string ch;

    if( !UTF8FromUChar( src[i], ch ) )
      return false;

    utf8 += ch;

#if 0
    if( src[i] <= CHAR( 0x7fU ) )
    {
      utf8 += (char) src[i];
    }
    else if( src[i] <= CHAR( 0x7ffU ) )
    {
      utf8 += (char) ( (src[i] >> 1 * 6) + 0xc0 );
      utf8 += (char) (((src[i] >> 0 * 6 ) & 0x3f) + 0x80 );
    }
    else if( src[i] <= CHAR( 0xffffU ) )
    {
      utf8 += (char) ( (src[i] >> 2 * 6) + 0xe0 );
      utf8 += (char) (((src[i] >> 1 * 6 ) & 0x3f) + 0x80 );
      utf8 += (char) (((src[i] >> 0 * 6 ) & 0x3f) + 0x80 );
    }
    else if( src[i] <= CHAR( 0x1fffffU ) )
    {
      utf8 += (char) ( (src[i] >> 3 * 6) + 0xf0 );
      utf8 += (char) (((src[i] >> 2 * 6 ) & 0x3f) + 0x80 );
      utf8 += (char) (((src[i] >> 1 * 6 ) & 0x3f) + 0x80 );
      utf8 += (char) (((src[i] >> 0 * 6 ) & 0x3f) + 0x80 );
    }
    else if( src[i] <= CHAR( 0x3ffffffU ) )
    {
      utf8 += (char) ( (src[i] >> 4 * 6) + 0xf8 );
      utf8 += (char) (((src[i] >> 3 * 6 ) & 0x3f) + 0x80 );
      utf8 += (char) (((src[i] >> 2 * 6 ) & 0x3f) + 0x80 );
      utf8 += (char) (((src[i] >> 1 * 6 ) & 0x3f) + 0x80 );
      utf8 += (char) (((src[i] >> 0 * 6 ) & 0x3f) + 0x80 );
    }
    else if( src[i] <= CHAR( 0x7fffffffU ) )
    {
      utf8 += (char) ( (src[i] >> 5 * 6) + 0xfc );
      utf8 += (char) (((src[i] >> 4 * 6 ) & 0x3f) + 0x80 );
      utf8 += (char) (((src[i] >> 3 * 6 ) & 0x3f) + 0x80 );
      utf8 += (char) (((src[i] >> 2 * 6 ) & 0x3f) + 0x80 );
      utf8 += (char) (((src[i] >> 1 * 6 ) & 0x3f) + 0x80 );
      utf8 += (char) (((src[i] >> 0 * 6 ) & 0x3f) + 0x80 );
    }
    else
      return false;
#endif
  }

  return true;
}
