////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: chars/chars.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
#ifndef _CHARS_H
#define _CHARS_H

#include <string>
#include "util/hash.h"

typedef wchar_t          UCHAR;

//typedef std::wstring     USTRING;
class USTRING : public std::wstring
{
public:
  USTRING();
  USTRING( const wchar_t *s );

  std::string utf8() const;

  void FromUtf8( const char *s );
  void FromUtf8( const std::string &s );
};

namespace stlhash
{
  template<> struct hash<USTRING> {
    size_t operator()(const USTRING &x) const
    {
      hash<std::wstring> t;
      return t( x );
    }
  };
}

extern bool UStringFromUTF8( const char *utf8, USTRING &dest );
extern bool UTF8FromUChar( const UCHAR c, std::string &utf8 );
extern bool UTF8FromUString( const USTRING &src, std::string &utf8 );

extern std::ostream& operator << ( std::ostream& stream, const USTRING &string );

#define CHAR( unicode ) ((UCHAR) unicode)

#define CHAR_VOID                       CHAR(0xFFFFU)
#define CHAR_SPACE                      CHAR(0x0020U)
#define CHAR_THIN_SPACE                 CHAR(0x2009U)
#define CHAR_BACKSLASH                  CHAR(0x005CU)
#define CHAR_LINEFEED			CHAR(0x000AU)

// range for unicode spec math symbols
#define CHAR_SPEC_START                 CHAR(0x2200U)
#define CHAR_SPEC_END                   CHAR(0x22FFU)

#define CHAR_SEQUENT                    CHAR(0x22A2U)
#define CHAR_AND                        CHAR(0x22C0U)
#define CHAR_OR                         CHAR(0x22C1U)
#define CHAR_IMPL                       CHAR(0x2283U)
#define CHAR_ALL                        CHAR(0x2200U)
#define CHAR_EXISTS                     CHAR(0x2203U)

#define CHAR_NEG                        CHAR(0x00ACU)
#define CHAR_SQUOTE                     CHAR(0x0027U)

#define CHAR_DIV                        CHAR(0x00F7U)
#define CHAR_PART                       CHAR(0x2202U)
#define CHAR_EMPTY                      CHAR(0x2205U)
#define CHAR_NABLA                      CHAR(0x2207U)
#define CHAR_ISIN                       CHAR(0x2208U)
#define CHAR_NOTIN                      CHAR(0x2209U)
#define CHAR_NI                         CHAR(0x220BU)
#define CHAR_PROD                       CHAR(0x220FU)
#define CHAR_SUM                        CHAR(0x2211U)
#define CHAR_MINUS                      CHAR(0x2212U)
#define CHAR_LOWAST                     CHAR(0x2217U)
#define CHAR_RADIC                      CHAR(0x221AU)
#define CHAR_PROP                       CHAR(0x221DU)
#define CHAR_INFIN                      CHAR(0x221EU)
#define CHAR_ANG                        CHAR(0x2220U)
#define CHAR_CAP                        CHAR(0x2229U)
#define CHAR_CUP                        CHAR(0x222AU)
#define CHAR_INT                        CHAR(0x222BU)
#define CHAR_THERE4                     CHAR(0x2234U)
#define CHAR_SIM                        CHAR(0x223CU)
#define CHAR_CONG                       CHAR(0x2245U)
#define CHAR_ASYMP                      CHAR(0x2248U)
#define CHAR_NE                         CHAR(0x2260U)
#define CHAR_EQUIV                      CHAR(0x2261U)
#define CHAR_LE                         CHAR(0x2264U)
#define CHAR_GE                         CHAR(0x2265U)
#define CHAR_SUB                        CHAR(0x2282U)
#define CHAR_SUP                        CHAR(0x2283U)
#define CHAR_NSUB                       CHAR(0x2284U)
#define CHAR_SUBE                       CHAR(0x2286U)
#define CHAR_SUPE                       CHAR(0x2287U)
#define CHAR_OPLUS                      CHAR(0x2295U)
#define CHAR_OTIMES                     CHAR(0x2297U)
#define CHAR_PERP                       CHAR(0x22A5U)
#define CHAR_SDOT                       CHAR(0x22C5U)
#define CHAR_LOZ                        CHAR(0x25CAU)

// small greek letters
#define CHAR_ALPHA                      CHAR(0x03B1U)
#define CHAR_BETA                       CHAR(0x03B2U)
#define CHAR_GAMMA                      CHAR(0x03B3U)
#define CHAR_DELTA                      CHAR(0x03B4U)
#define CHAR_CURLYEPSILON               CHAR(0x03B5U)
#define CHAR_ZETA                       CHAR(0x03B6U)
#define CHAR_ETA                        CHAR(0x03B7U)
#define CHAR_THETA                      CHAR(0x03B8U)
#define CHAR_IOTA                       CHAR(0x03B9U)
#define CHAR_KAPPA                      CHAR(0x03BAU)
#define CHAR_LAMBDA                     CHAR(0x03BBU)
#define CHAR_MU                         CHAR(0x03BCU)
#define CHAR_NU                         CHAR(0x03BDU)
#define CHAR_XI                         CHAR(0x03BEU)
#define CHAR_OMICRON                    CHAR(0x03BFU)
#define CHAR_PI                         CHAR(0x03C0U)
#define CHAR_RHO                        CHAR(0x03C1U)
#define CHAR_FINALSIGMA                 CHAR(0x03C2U)
#define CHAR_SIGMA                      CHAR(0x03C3U)
#define CHAR_TAU                        CHAR(0x03C4U)
#define CHAR_UPSILON                    CHAR(0x03C5U)
#define CHAR_CURLYPHI                   CHAR(0x03C6U)
#define CHAR_CHI                        CHAR(0x03C7U)
#define CHAR_PSI                        CHAR(0x03C8U)
#define CHAR_OMEGA                      CHAR(0x03C9U)

#define CHAR_CURLYTHETA                 CHAR(0x03D1U)
#define CHAR_CURLYPI                    CHAR(0x03D6U)
#define CHAR_PHI                        CHAR(0x03D5U)
#define CHAR_CURLYRHO                   CHAR(0x03F1U)
#define CHAR_EPSILON                    CHAR(0x03F5U)

// capital greek letters
#define CHAR_CAPITALALPHA               CHAR(0x0391U)
#define CHAR_CAPITALBETA                CHAR(0x0392U)
#define CHAR_CAPITALGAMMA               CHAR(0x0393U)
#define CHAR_CAPITALDELTA               CHAR(0x0394U)
#define CHAR_CAPITALEPSILON             CHAR(0x0395U)
#define CHAR_CAPITALZETA                CHAR(0x0396U)
#define CHAR_CAPITALETA                 CHAR(0x0397U)
#define CHAR_CAPITALTHETA               CHAR(0x0398U)
#define CHAR_CAPITALIOTA                CHAR(0x0399U)
#define CHAR_CAPITALKAPPA               CHAR(0x039AU)
#define CHAR_CAPITALLAMBDA              CHAR(0x039BU)
#define CHAR_CAPITALMU                  CHAR(0x039CU)
#define CHAR_CAPITALNU                  CHAR(0x039DU)
#define CHAR_CAPITALXI                  CHAR(0x039EU)
#define CHAR_CAPITALOMICRON             CHAR(0x039FU)
#define CHAR_CAPITALPI                  CHAR(0x03A0U)
#define CHAR_CAPITALRHO                 CHAR(0x03A1U)
#define CHAR_CAPITALSIGMA               CHAR(0x03A3U)
#define CHAR_CAPITALTAU                 CHAR(0x03A4U)
#define CHAR_CAPITALUPSILON             CHAR(0x03A5U)
#define CHAR_CAPITALPHI                 CHAR(0x03A6U)
#define CHAR_CAPITALCHI                 CHAR(0x03A7U)
#define CHAR_CAPITALPSI                 CHAR(0x03A8U)
#define CHAR_CAPITALOMEGA               CHAR(0x03A9U)

#endif

