////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: chars/latexchars.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
#include <iostream>
using namespace std;
#endif

#include "latexchars.h"
#include "chars.h"

LatexChars LatexConvertor;

#define LATEXCHAR( l, u )  AddTranslation( l, u )

LatexChars::LatexChars()
{
  LATEXCHAR( "\\", CHAR_BACKSLASH );
  LATEXCHAR( "backslash", CHAR_BACKSLASH );

  LATEXCHAR( "seq", CHAR_SEQUENT );
  LATEXCHAR( "vdash", CHAR_SEQUENT );
  
  LATEXCHAR( "land", CHAR_AND );
  LATEXCHAR( "And", CHAR_AND );
  LATEXCHAR( "and", CHAR_AND );
  
  LATEXCHAR( "lor", CHAR_OR );
  LATEXCHAR( "Or", CHAR_OR );
  LATEXCHAR( "or", CHAR_OR );
  
  LATEXCHAR( "to", CHAR_IMPL );
  LATEXCHAR( "Impl", CHAR_IMPL );
  LATEXCHAR( "impl", CHAR_IMPL );
  
  LATEXCHAR( "forall", CHAR_ALL );
  LATEXCHAR( "all", CHAR_ALL );
  
  LATEXCHAR( "exists", CHAR_EXISTS );
  LATEXCHAR( "ex", CHAR_EXISTS );
  
  LATEXCHAR( "lnot", CHAR_NEG );
  LATEXCHAR( "Neg", CHAR_NEG );
  LATEXCHAR( "neg", CHAR_NEG );
  LATEXCHAR( "Not", CHAR_NEG );
  LATEXCHAR( "not", CHAR_NEG );

  LATEXCHAR( "Gamma", CHAR_CAPITALGAMMA );
  LATEXCHAR( "Delta", CHAR_CAPITALDELTA );
  LATEXCHAR( "Theta", CHAR_CAPITALTHETA );
  LATEXCHAR( "Lambda", CHAR_CAPITALLAMBDA );
  LATEXCHAR( "Xi", CHAR_CAPITALXI );
  LATEXCHAR( "Pi", CHAR_CAPITALPI );
  LATEXCHAR( "Sigma", CHAR_CAPITALSIGMA );
  LATEXCHAR( "Phi", CHAR_CAPITALPHI );
  LATEXCHAR( "Psi", CHAR_CAPITALPSI );
  LATEXCHAR( "Omega", CHAR_CAPITALOMEGA );
  LATEXCHAR( "alpha", CHAR_ALPHA );
  LATEXCHAR( "beta", CHAR_BETA );
  LATEXCHAR( "gamma", CHAR_GAMMA );
  LATEXCHAR( "delta", CHAR_DELTA );
  LATEXCHAR( "varepsilon", CHAR_CURLYEPSILON );
  LATEXCHAR( "epsilon", CHAR_EPSILON );
  LATEXCHAR( "zeta", CHAR_ZETA );
  LATEXCHAR( "eta", CHAR_ETA );
  LATEXCHAR( "theta", CHAR_THETA );
  LATEXCHAR( "vartheta", CHAR_CURLYTHETA );
  LATEXCHAR( "iota", CHAR_IOTA );
  LATEXCHAR( "kappa", CHAR_KAPPA );
  LATEXCHAR( "lambda", CHAR_LAMBDA );
  LATEXCHAR( "mu", CHAR_MU );
  LATEXCHAR( "nu", CHAR_NU );
  LATEXCHAR( "xi", CHAR_XI );
  LATEXCHAR( "pi", CHAR_PI );
  LATEXCHAR( "varpi", CHAR_CURLYPI );
  LATEXCHAR( "rho", CHAR_RHO );
  LATEXCHAR( "varrho", CHAR_CURLYRHO );
  LATEXCHAR( "varsigma", CHAR_FINALSIGMA );
  LATEXCHAR( "sigma", CHAR_SIGMA );
  LATEXCHAR( "tau", CHAR_TAU );
  LATEXCHAR( "upsilon", CHAR_UPSILON );
  LATEXCHAR( "varphi", CHAR_CURLYPHI );
  LATEXCHAR( "phi", CHAR_PHI );
  LATEXCHAR( "chi", CHAR_CHI );
  LATEXCHAR( "psi", CHAR_PSI );
  LATEXCHAR( "omega", CHAR_OMEGA );

  LATEXCHAR( "div", CHAR_DIV );
// #define CHAR_PART                       CHAR(0x2202U)
  LATEXCHAR( "emptyset", CHAR_EMPTY );
// #define CHAR_NABLA                      CHAR(0x2207U)
  LATEXCHAR( "in", CHAR_ISIN );
  LATEXCHAR( "notin", CHAR_NOTIN );
  LATEXCHAR( "ni", CHAR_NI );
// #define CHAR_PROD                       CHAR(0x220FU)
// #define CHAR_SUM                        CHAR(0x2211U)
// #define CHAR_MINUS                      CHAR(0x2212U)
// #define CHAR_LOWAST                     CHAR(0x2217U)
// #define CHAR_RADIC                      CHAR(0x221AU)
// #define CHAR_PROP                       CHAR(0x221DU)
// #define CHAR_INFIN                      CHAR(0x221EU)
// #define CHAR_ANG                        CHAR(0x2220U)
  LATEXCHAR( "cap", CHAR_CAP );
  LATEXCHAR( "cup", CHAR_CUP );
// #define CHAR_INT                        CHAR(0x222BU)
// #define CHAR_THERE4                     CHAR(0x2234U)
// #define CHAR_SIM                        CHAR(0x223CU)
// #define CHAR_CONG                       CHAR(0x2245U)
// #define CHAR_ASYMP                      CHAR(0x2248U)
  LATEXCHAR( "neq", CHAR_NE );
  LATEXCHAR( "equiv", CHAR_EQUIV );
  LATEXCHAR( "leq", CHAR_LE );
  LATEXCHAR( "geq", CHAR_GE );
  LATEXCHAR( "subset", CHAR_SUB );
  LATEXCHAR( "supset", CHAR_SUP );
// #define CHAR_NSUB                       CHAR(0x2284U)
  LATEXCHAR( "subseteq", CHAR_SUBE );
  LATEXCHAR( "supseteq", CHAR_SUPE );
  LATEXCHAR( "oplus", CHAR_OPLUS );
  LATEXCHAR( "otimes", CHAR_OTIMES );
// #define CHAR_PERP                       CHAR(0x22A5U)
  LATEXCHAR( "circle", CHAR_SDOT );
// #define CHAR_LOZ                        CHAR(0x25CAU)
}

void LatexChars::AddTranslation( const char *l, UCHAR u )
{
  m_ltou[l] = u;

  if( m_utol[u] == NULL )
    m_utol[u] = l;
}

UCHAR LatexChars::GetUnicode( const char* str )
{
  LatexToUnicode::const_iterator i = m_ltou.find( str );

  if( i == m_ltou.end() )
  {
#ifdef DEBUG
    cout << "UNKNOWN LATEX SYMBOL: \\" << str << endl;
#endif
    return '?';
  }

  return (*i).second;
}

const char* LatexChars::GetLatex( UCHAR code )
{
  if( m_utol.find( code ) == m_utol.end() )
    return NULL;

  return m_utol[code];
}


void LatexChars::GetUnicode( const char* src, USTRING &dest )
{
  dest.clear();

  if( src == NULL )
    return;

  std::string l;

  while( *src )
  {
    if( *src == '\\' )
    {
      src++;
      l.clear();
      while( *src && ( ( *src >= 'a' && *src <= 'z' ) ||
                       ( *src >= 'A' && *src <= 'Z' ) ) )
      {
        l += *(src++);
      }
        
      UCHAR c = LatexConvertor.GetUnicode( l.c_str() );

      if( c != 0 )
        dest += c;
      else
      {
        dest += '\\';
        for( size_t i = 0; i < l.length(); i++ )
          dest += UCHAR( l[i] );
      }
    }
    else
      dest += *(src++);
  }
}

USTRING& LatexChars::KnownLatexSymbolsToUnicode( USTRING &to_change )
{
  USTRING::iterator i = to_change.begin();

  while( i != to_change.end() )
  {
    if( *i == '\\' )
    {
      std::string l;

      USTRING::iterator j = i;
      j++;

      while( j != to_change.end() &&
             ( ( *j >= 'a' && *j <= 'z' ) ||
               ( *j >= 'A' && *j <= 'Z' ) ) )
      {
        l += *(j++);
      }

      j--;

      if( j > i )
      {
        UCHAR c = LatexConvertor.GetUnicode( l.c_str() );

        if( c != 0 )
        {
          *i = c;
          to_change.erase( i+1, j+1 );
        }
        else
          i = j;
      }
    }

    i++;
  }

  return to_change;
}

void LatexChars::GetLatex( const USTRING& src, std::string &dest )
{
  dest.clear();

  size_t i = 0;
  for( ; i < src.length(); i++ )
  {
/*
    if( src[i] <= 0xff )
    {
      dest += (char) src[i];
      continue;
    }
*/

    const char *dc = LatexConvertor.GetLatex( src[i] );

    if( dc == NULL )
      dest += (char) src[i];
    else
    {
      dest += '\\';
      dest += LatexConvertor.GetLatex( src[i] );
    }
  }
}

/*
void LatexChars::GetLatex( const UCHAR* src, std::string &dest )
{
  while( *src != NULL )
  {
    
  }
}
*/
