////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: export/exportflatproof.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#include "util/myassert.h"
#include "util/xmlwrite.h"
#include "util/proofdbdtd.h"
#include "libproof.h"

#include "exportflatproof.h"

using namespace std;
using namespace ProofLib;

////////////////////////////////////////////////////////////////////////////////

class ExportFlatProofXMLImpl : public ExportFlatProofXML
{
  XmlWriter             *m_xml_writer;
  int                    m_compression;
  bool                   m_bAsProofDatabase;

  const ProofDatabase   *m_pdb;

protected:
  ExportFlatProofXMLImpl( bool bAsProofDatabase )
  {
    m_xml_writer = NULL;
    m_compression = 0;
    m_pdb = NULL;
    m_bAsProofDatabase = bAsProofDatabase;
  }

  friend class ExportFlatProofXML;

public:
  XmlWriter* GetXmlWriter() const
  {
    return m_xml_writer;
  }

  const ProofDatabase* GetProofDatabase() const
  {
    return m_pdb;
  }

  virtual void Write( const ProofLib::ProofDatabase* proofdb )
  {
    Write( proofdb->GetDefaultProof(), proofdb );
  }

  virtual void Write( const ProofLib::Proof* proof,
                      const ProofLib::ProofDatabase* proofdb )
  {
    if( proof == NULL )
      return;

    ASSERT( proofdb != NULL );
    ASSERT( m_xml_writer != NULL );

    m_pdb = proofdb;

    m_xml_writer->StartDocument( PROOFDBDTD_VERSION, PROOFDBDTD_ENCODING );

    if( m_bAsProofDatabase )
    {
      m_xml_writer->WriteDTD( PROOFDBDTD_NODE_PROOFDATABASE,
                              PROOFDBDTD_DTD_PROOFDATABASE_PUBLIC,
                              PROOFDBDTD_DTD_PROOFDATABASE_SYSTEM,
                              PROOFDBDTD_DTD_PROOFDATABASE_SUBSET );

      GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_PROOFDATABASE );
      
      GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_DEFINITIONLIST );
      GetXmlWriter()->CloseElement();
      
      GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_AXIOMSET );
      GetXmlWriter()->CloseElement();
      
    }
    else
    {
      m_xml_writer->WriteDTD( PROOFDBDTD_NODE_PROOF,
                              PROOFDBDTD_DTD_PROOF_PUBLIC,
                              PROOFDBDTD_DTD_PROOF_SYSTEM,
                              PROOFDBDTD_DTD_PROOF_SUBSET );
    }

    ((ExportXML&) *this) << *proof;

    if( m_bAsProofDatabase )
    {
      GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_VARIABLE_DEFINITIONS );
      GetXmlWriter()->CloseElement();

      GetXmlWriter()->CloseElement(); // PROOFDBDTD_NODE_PROOFDATABASE
    }

    m_xml_writer->EndDocument();
  }

  virtual void Open( const char *sourcename )
  {
    ASSERT( m_xml_writer == NULL );

    m_xml_writer = XmlWriter::New();

    m_xml_writer->SetCompression( m_compression );
    m_xml_writer->Open( sourcename );
  }

  virtual void Close()
  {
    ASSERT( m_xml_writer != NULL );

    m_xml_writer->Close();
    delete m_xml_writer;
    m_xml_writer = NULL;
  }

  virtual void AddFormulaVar( const ProofLib::FormulaVariable *v ) {}
  virtual void AddFormulaListVar( const ProofLib::FormulaListVariable *v ) {}
  virtual void WriteVariables() {}

  virtual ExportXML& operator << ( const ProofLib::ProofLink& p );
  virtual ExportXML& operator << ( const ProofLib::FormulaVariable& f );
  virtual ExportXML& operator << ( const ProofLib::FormulaListVariable& l );
};

////////////////////////////////////////////////////////////////////////////////

ExportFlatProofXML::ExportFlatProofXML()
{
}

ExportFlatProofXML* ExportFlatProofXML::New( bool bAsProofDatabase )
{
  return new ExportFlatProofXMLImpl( bAsProofDatabase );
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportFlatProofXMLImpl::operator << ( const ProofLink& pl )
{
  Proof *p = pl.GetLinkedProof();
  ASSERT( p != NULL );
  ((ExportXML&) *this) << *(p->GetEndRule());

  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportFlatProofXMLImpl::operator << ( const FormulaVariable& f )
{
  ((ExportXML&) *this) << *(f.GetFormula());

  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportFlatProofXMLImpl::operator << ( const FormulaListVariable& l )
{
#if 0
  if( &l != NULL )
    for( size_t i = 0; i < l.GetChildCount(); i++ )
      ((ExportXML&) *this) << *(l.GetElement(i));
#endif
  ((ExportXML&) *this) << *(l.GetFormulaList());

  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

