////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: export/exportxml.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _EXPORTLATEX_H_
#define _EXPORTLATEX_H_

#include "export.h"

class XmlWriter;

namespace ProofLib
{
  class ProofLink;
  class Variable;
  class Constant;
  class Function;
  class Term;
  class StaticTermList;
  class StaticAbstractTermList;
  class AtomFormula;
  class QuantorFormula;
  class UnaryConFormula;
  class BinaryConFormula;
  class FormulaVariable;
  class Formula;
  class FormulaListVariable;
  class FormulaListElement;
  class SElement;
  class SElementList;
  class Sequent;
  class Rule;
  class RuleItem;
  class Proof;
  class ProofDatabase;
  class RecursiveObject;
}

////////////////////////////////////////////////////////////////////////////////

class ExportLaTeX : public ExportFile
{
protected:
  ExportLaTeX() {};

public:
  static ExportLaTeX* New( bool export_flat = false );

public:
  virtual void Write( const ProofLib::ProofDatabase* proofdb ) = 0;
  virtual void Write( const ProofLib::Proof* proof,
                      const ProofLib::ProofDatabase* proofdb ) = 0;
  virtual void Write( const ProofLib::SequentList* list,
                      const ProofLib::ProofDatabase* proofdb ) = 0;

  virtual bool IsFlatExport() const = 0;
  virtual bool IsCascadeExport() const = 0;
  virtual void AddCascade( ProofLib::Proof *p ) = 0;

  // helper for less parenthesis
  virtual bool IsFirstFormula() = 0;
  virtual void FirstFormula() = 0;
  virtual bool IsFirstTerm() = 0;
  virtual void FirstTerm() = 0;

protected:
#if 0
  virtual void AddFormulaVar( const ProofLib::FormulaVariable *v ) = 0;
  virtual void AddFormulaListVar( const ProofLib::FormulaListVariable *v ) = 0;
  virtual void WriteVariables() = 0;
#endif
  virtual ExportLaTeX& WriteRecursiveName( const USTRING& name,
                                           const ProofLib::RecursiveObject& o );

public:
  virtual ExportLaTeX& operator << ( const ProofLib::ProofLink& p );
  virtual ExportLaTeX& operator << ( const ProofLib::Variable& v );
  virtual ExportLaTeX& operator << ( const ProofLib::Constant& c );
  virtual ExportLaTeX& operator << ( const ProofLib::Function& f );
  virtual ExportLaTeX& operator << ( const ProofLib::AbstractTerm& t );
  virtual ExportLaTeX& operator << ( const ProofLib::Term& t );
  virtual ExportLaTeX& operator << ( const ProofLib::StaticTermList& l );
  virtual ExportLaTeX& operator << ( const ProofLib::StaticAbstractTermList& l );
  virtual ExportLaTeX& operator << ( const ProofLib::AtomFormula& f );
  virtual ExportLaTeX& operator << ( const ProofLib::QuantorFormula& f );
  virtual ExportLaTeX& operator << ( const ProofLib::UnaryConFormula& f );
  virtual ExportLaTeX& operator << ( const ProofLib::BinaryConFormula& f );
  virtual ExportLaTeX& operator << ( const ProofLib::FormulaVariable& f );
  virtual ExportLaTeX& operator << ( const ProofLib::Formula& f );
  virtual ExportLaTeX& operator << ( const ProofLib::FormulaList& l );
  virtual ExportLaTeX& operator << ( const ProofLib::FormulaListVariable& l );
  virtual ExportLaTeX& operator << ( const ProofLib::FormulaListElement& e );
  virtual ExportLaTeX& operator << ( const ProofLib::SElement& e );
  virtual ExportLaTeX& operator << ( const ProofLib::SElementList& l );
  ExportLaTeX& write ( const ProofLib::SElementList& l, bool formulas_by_line );
  virtual ExportLaTeX& operator << ( const ProofLib::Sequent& s );
  ExportLaTeX& write ( const ProofLib::Sequent& s, bool formulas_by_line );
  virtual ExportLaTeX& operator << ( const ProofLib::Rule& r );
  virtual ExportLaTeX& operator << ( const ProofLib::RuleItem& ruleitem );
  virtual ExportLaTeX& operator << ( const ProofLib::Proof& p );
  virtual ExportLaTeX& operator << ( const ProofLib::SequentList& l );
  virtual ExportLaTeX& operator << ( const ProofLib::ProofDatabase& pdb );
};

////////////////////////////////////////////////////////////////////////////////

#endif
