////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: export/exportxml.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#include "util/myassert.h"
#include "util/xmlwrite.h"
#include "libproof.h"
#include "recursive.h"

#include <typeinfo>
#include "exportxml.h"

#include <iostream>

//#include "../../printhlk.h"

using namespace std;
using namespace ProofLib;

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const Variable& v )
{
  //std::cerr << "XML@Variable --- start" << std::endl;
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_VARIABLE );

  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_SYMBOL,
                                    v.GetName() );

  GetXmlWriter()->CloseElement();
  //std::cerr << "XML@Variable --- end" << std::endl;
  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const SecondOrderVariable& v )
{
  //std::cerr << "XML@SecondOrderVariable --- start" << std::endl;
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_SECONDORDERVARIABLE );

  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_SYMBOL,
                                    v.GetName() );

  GetXmlWriter()->CloseElement();
  //std::cerr << "XML@SecondOrderVariable --- end" << std::endl;
  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const Constant& c )
{
  //std::cerr << "XML@Constant --- start" << std::endl;
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_CONSTANT );

  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_SYMBOL,
                                    c.GetName() );

  if( c.GetSymbol()->IsDefined() )
  {
    GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_DEFINITION,
                                    c.GetSymbol()->GetDefinitionId() );
  }

  GetXmlWriter()->CloseElement();

  //std::cerr << "XML@Constant --- end" << std::endl;
  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const Function& f )
{
  try
  {
    const RecursiveFunction &f_r = dynamic_cast<const RecursiveFunction&>( f );

    if( f_r.GetChildCount() < 1 )
      GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_CONSTANT );
    else
      GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_FUNCTION );

    WriteRecursiveName( f.GetName(), f_r );

    if( f.GetSymbol()->IsDefined( &f ) )
    {
      WriteRecursiveName( f.GetSymbol()->GetDefinitionId( &f ),
                          f_r, PROOFDBDTD_ATTR_DEFINITION );
    }
  }
  catch( bad_cast )
  {
    GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_FUNCTION );

    GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_SYMBOL,
                                    f.GetName() );

    if( f.GetSymbol()->IsDefined( &f ) )
    {
      GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_DEFINITION,
                                      f.GetSymbol()->GetDefinitionId( &f ) );
    }
  }

  (*this) << dynamic_cast<const StaticAbstractTermList&>( f );

  GetXmlWriter()->CloseElement();

  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::WriteRecursiveName( const USTRING& name,
                                          const ProofLib::RecursiveObject& o,
                                          const char* attr )
{
  std::wostringstream str;

  RecursiveExpr* e = o.GetRecursiveExpr();

  ASSERT( e->GetVariableName() == NULL );

  str << name.c_str() << "[" << e->GetNum() << "]";

  GetXmlWriter()->WriteAttribute( attr, str.str().c_str() );
  
  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const AbstractTerm& t )
{
  try
  {
    const Term &a = dynamic_cast<const Term&>( t );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}
  
  try
  {
    const SetTerm &a = dynamic_cast<const SetTerm&>( t );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}

  ASSERT( false );
  
  return (*this);
}

ExportXML& ExportXML::operator << ( const SetTerm& t )
{
  try
  {
    const SecondOrderVariable &a = dynamic_cast<const SecondOrderVariable&>( t );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}
  
  try
  {
    const LambdaTerm &a = dynamic_cast<const LambdaTerm&>( t );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}

  try
  {
    const DefinedSet &a = dynamic_cast<const DefinedSet&>( t );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}

  ASSERT( false );
  
  return (*this);
}
////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const Term& t )
{
  try
  {
    const Variable &a = dynamic_cast<const Variable&>( t );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}
  
  try
  {
    const Constant &a = dynamic_cast<const Constant&>( t );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}
  
  try
  {
    const Function &a = dynamic_cast<const Function&>( t );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}

  ASSERT( false );
  
  return (*this);
}

////////////////////////////////////////////////////////////////////////////////
#if 0
ExportXML& ExportXML::operator << ( const StaticTermList& l )
{
  //std::cerr << "XML@TermList --- start" << std::endl;
  for( size_t i = 0; i < l.GetChildCount(); i++ )
    (*this) << *(l.GetTerm(i));
  //std::cerr << "XML@TermList --- end" << std::endl;
  return (*this);
}
#endif
////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const StaticAbstractTermList& l )
{
  //std::cerr << "XML@TermList --- start" << std::endl;
  for( size_t i = 0; i < l.GetChildCount(); i++ )
    (*this) << *(l.GetTerm(i));
  //std::cerr << "XML@TermList --- end" << std::endl;
  return (*this);
}
////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const AtomFormula& f )
{
  //std::cerr << "XML@AtomFormula --- start" << std::endl;
  SecondOrderVariable* v = f.GetSecondOrderVariable();

  if ( v == NULL )
  {
    GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_CONSTANTATOMFORMULA );

    try
    {
      const RecursivePredicate &f_r =
	dynamic_cast<const RecursivePredicate&>( f );

      WriteRecursiveName( f.GetName(), f_r );

      if( f.GetSymbol()->IsDefined( &f ) )
      {
	WriteRecursiveName( f.GetSymbol()->GetDefinitionId( &f ),
			    f_r, PROOFDBDTD_ATTR_DEFINITION );
      }
    }
    catch( bad_cast )
    {
      GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_SYMBOL,
				      f.GetName() );

      if( f.GetSymbol()->IsDefined( &f ) )
      {
	GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_DEFINITION,
					f.GetSymbol()->GetDefinitionId( &f ) );
      }
    }
  }
  else
  {
    GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_VARIABLEATOMFORMULA );
    
    (*this) << *v;
  }

  (*this) << dynamic_cast<const StaticAbstractTermList&>( f );

  GetXmlWriter()->CloseElement();
  //std::cerr << "XML@AtomFormula --- end" << std::endl;
  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const DefinedSetFormula& f )
{
  //std::cerr << "XML@DefinedSetFormula --- start" << std::endl;
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_DEFINEDSETFORMULA );
  (*this) << *(f.getDefinedSet());
  (*this) <<  dynamic_cast<const StaticTermList&>( f );
  GetXmlWriter()->CloseElement();
  //std::cerr << "XML@AtomFormula --- end" << std::endl;
  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const DefinedSet& d )
{
  //std::cerr << "XML@DefinedSetFormula --- start" << std::endl;
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_DEFINEDSET );
  // FIXME: write definition!
  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_DEFINITION,
					"" );
  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_SYMBOL,
				      d.GetName() );
  (*this) << dynamic_cast<const StaticAbstractTermList&>( d );
  GetXmlWriter()->CloseElement();
  //std::cerr << "XML@AtomFormula --- end" << std::endl;
  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const QuantorFormula& f )
{
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_QUANTIFIEDFORMULA );

  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_TYPE,
                                  f.GetOperatorSymbol()->GetName() );

  (*this) << *(f.GetVariable());

  (*this) << *(f.GetFormula());

  GetXmlWriter()->CloseElement();

  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const SecondOrderQuantorFormula& f )
{
  //std::cerr << "XML@SecondOrderQuantifiedFormula --- start" << std::endl;
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_SECONDORDERQUANTIFIEDFORMULA );

  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_TYPE,
                                  f.GetOperatorSymbol()->GetName() );

  (*this) << *(f.GetSecondOrderVariable());

  (*this) << *(f.GetFormula());

  GetXmlWriter()->CloseElement();
  //std::cerr << "XML@SecondOrderQuantifiedFormula --- end" << std::endl;
  return (*this);
}
////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const UnaryConFormula& f )
{
  //std::cerr << "XML@UnaryFormula --- start" << std::endl;
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_CONJUNCTIVEFORMULA );

  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_TYPE,
                                  f.GetOperatorSymbol()->GetName() );

  (*this) << *(f.GetFormula());

  GetXmlWriter()->CloseElement();
  //std::cerr << "XML@UnaryFormula --- end" << std::endl;
  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const BinaryConFormula& f )
{
  //std::cerr << "XML@BinaryFormula --- start" << std::endl;
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_CONJUNCTIVEFORMULA );

  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_TYPE,
                                  f.GetOperatorSymbol()->GetName() );

  (*this) << *(f.GetLeft());

  (*this) << *(f.GetRight());

  GetXmlWriter()->CloseElement();
  //std::cerr << "XML@BinaryFormula --- end" << std::endl;
  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const FormulaVariable& f )
{
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_FORMULAVAR );

  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_SYMBOL,
                                    f.GetName() );

  AddFormulaVar( &f );

  GetXmlWriter()->CloseElement();
  
  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const Formula& f )
{
  try
  {
    const AtomFormula &a = dynamic_cast<const AtomFormula&>( f );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}

  try
  {
    const QuantorFormula &a = dynamic_cast<const QuantorFormula&>( f );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}

  try
  {
    const SecondOrderQuantorFormula &a = dynamic_cast<const SecondOrderQuantorFormula&>( f );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}

  try
  {
    const UnaryConFormula &a = dynamic_cast<const UnaryConFormula&>( f );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}

  try
  {
    const BinaryConFormula &a = dynamic_cast<const BinaryConFormula&>( f );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}

  try
  {
    const FormulaVariable &a = dynamic_cast<const FormulaVariable&>( f );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}

  try
  {
    const DefinedSetFormula &a = dynamic_cast<const DefinedSetFormula&>( f );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}


  ASSERT( false );

  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const FormulaList& l )
{
//  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_FORMULALIST );

  if( &l != NULL )
    for( size_t i = 0; i < l.GetChildCount(); i++ )
      (*this) << *(l.GetElement(i));

//  GetXmlWriter()->CloseElement();

  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const FormulaListVariable& l )
{
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_FORMULALISTVAR );

  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_SYMBOL,
                                    l.GetName() );

  AddFormulaListVar( &l );


  GetXmlWriter()->CloseElement();

  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const FormulaListElement& e )
{
  try
  {
    const Formula &a = dynamic_cast<const Formula&>( e );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}

  try
  {
    const FormulaListVariable &a =
      dynamic_cast<const FormulaListVariable&>( e );
    (*this) << a;
    return (*this);
  }
  catch( bad_cast ) {}

  ASSERT( false );
  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const SElement& e )
{
  try
  {
    const SFormula &a = dynamic_cast<const SFormula&>( e );
    (*this) << *(a.GetFormula());
    return (*this);
  }
  catch( bad_cast ) {}

  try
  {
    const SFormulaListVariable &a =
      dynamic_cast<const SFormulaListVariable&>( e );
    (*this) << *(a.GetFormulaListVariable());
    return (*this);
  }
  catch( bad_cast ) {}

  ASSERT( false );
  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const SElementList& l )
{
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_FORMULALIST );

  if( &l != NULL )
    for( size_t i = 0; i < l.GetChildCount(); i++ )
      (*this) << *(l.GetSElement(i));

  GetXmlWriter()->CloseElement();

  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const Sequent& s )
{
  //std::cerr << "XML@Sequent --- start" << std::endl;
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_SEQUENT );

  (*this) << *(s.GetLeft());

  (*this) << *(s.GetRight());

  GetXmlWriter()->CloseElement();

  //std::cerr << "XML@Sequent --- end" << std::endl;

  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const LambdaTerm& t )
{
  //std::cerr << "XML@LambdaTerm --- start" << std::endl;
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_LAMBDASUBSTITUTION );

  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_VARIABLELIST );

  for ( size_t i = 0; i < t.GetChildCount() - 1; i++ )
  {
    (*this) << *(dynamic_cast<Variable*>( t.GetChild(i) ));
  }

  GetXmlWriter()->CloseElement();

  (*this) << *(t.getFormula());

  GetXmlWriter()->CloseElement();
  //std::cerr << "XML@LambdaTerm --- end" << std::endl;
  return (*this);
}
////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const Rule& r )
{
  //std::cerr << "XML@Rule --- start" << std::endl;
  if( r.GetRuleItemCount() == 0 )
  {
    // AxiomRule
    GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_RULE );

    GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_TYPE,
                                    "axiom" );

    (*this) << *(r.GetConclusion());

    GetXmlWriter()->CloseElement();
    //std::cerr << "XML@Rule --- end" << std::endl;
    return (*this);
  }

  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_RULE );

  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_SYMBOL,
                                    r.GetName() );
  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_TYPE,
                                    r.GetType() );
  //std::cerr << "XML@Rule --- type: " << r.GetType() << std::endl;

  if( !r.GetParam().empty() )
    GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_PARAM,
                                      r.GetParam() );

  (*this) << *(r.GetConclusion());

  for( size_t i = 0; i < r.GetRuleItemCount(); i++ )
  {
    (*this) << *(r.GetRuleItem(i));
  }

  const LambdaTerm *t = r.GetLambdaTerm();

  if ( t != NULL )
  {
    (*this) << *t;
  }

  GetXmlWriter()->CloseElement();
  //std::cerr << "XML@Rule --- end" << std::endl;
  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const ProofLink& p )
{
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_PROOFLINK );

  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_SYMBOL,
                                    p.GetLinkedProofName() );

  GetXmlWriter()->CloseElement();

  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const RuleItem& ruleitem )
{
  try
  {
    const Sequent &s = dynamic_cast<const Sequent&>( ruleitem );
    // AxiomRule
    GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_RULE );

    GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_TYPE,
                                    "axiom" );

    (*this) << s;

    GetXmlWriter()->CloseElement();
    
    return (*this);
  }
  catch( bad_cast ) { }

  try
  {
    const Rule &r = dynamic_cast<const Rule&>( ruleitem );
    (*this) << r;
    return (*this);
  }
  catch( bad_cast ) { }

  try
  {
    const ProofLink &pl = dynamic_cast<const ProofLink&>( ruleitem );
    (*this) << pl;
    return (*this);
  }
  catch( bad_cast ) { }

  ASSERT( false ); // unknow type (*this)f ruleitem

  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const Proof& p )
{
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_PROOF );

  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_SYMBOL,
                                    p.GetName() );
  GetXmlWriter()->WriteAttribute( PROOFDBDTD_ATTR_CALCULUS,
                                    PROOFDBDTD_DEFAULT_CALCULUS );

  (*this) << *(p.GetEndRule());

  GetXmlWriter()->CloseElement();

  return (*this);
}

////////////////////////////////////////////////////////////////////////////////

ExportXML& ExportXML::operator << ( const ProofDatabase& pdb )
{
  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_PROOFDATABASE );

  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_DEFINITIONLIST );
  // TODO
  GetXmlWriter()->CloseElement();

  GetXmlWriter()->OpenElement( PROOFDBDTD_NODE_AXIOMSET );
  // TODO
  GetXmlWriter()->CloseElement();

#if 0
  ProofDatabase::const_iterator i;

  for( i = pdb.begin(); i != pdb.end(); i++ )
  {
    Proof *p = pdb.GetProof( (*i).second->GetName() );

    (*this) <<  *p;
  }
#endif
  for( size_t i = 0; i < pdb.GetCount(); i++ )
  {
    Proof *p = pdb.GetProof( i );

    (*this) <<  *p;
  }

  WriteVariables();

  GetXmlWriter()->CloseElement();

  return (*this);
}

////////////////////////////////////////////////////////////////////////////////
