////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: export/exportxml.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _EXPORTXML_H_
#define _EXPORTXML_H_

#include "export.h"
#include "util/proofdbdtd.h"

class XmlWriter;

namespace ProofLib
{
  class ProofLink;
  class Variable;
  class Constant;
  class Function;
  class Term;
  class StaticTermList;
  class AtomFormula;
  class QuantorFormula;
  class UnaryConFormula;
  class BinaryConFormula;
  class FormulaVariable;
  class Formula;
  class FormulaList;
  class FormulaListVariable;
  class FormulaListElement;
  class SElement;
  class SElementList;
  class Sequent;
  class Rule;
  class RuleItem;
  class Proof;
  class ProofDatabase;
  class RecursiveObject;
  class DefinedSetFormula;
  class DefinedSet;
}

////////////////////////////////////////////////////////////////////////////////

class ExportXML : public ExportModul
{
protected:
  ExportXML() {};

public:
  virtual XmlWriter* GetXmlWriter() const = 0;

protected:
  virtual void AddFormulaVar( const ProofLib::FormulaVariable *v ) = 0;
  virtual void AddFormulaListVar( const ProofLib::FormulaListVariable *v ) = 0;
  virtual void WriteVariables() = 0;

  virtual ExportXML& WriteRecursiveName(
    const USTRING& name,
    const ProofLib::RecursiveObject& o,
    const char* attr = PROOFDBDTD_ATTR_SYMBOL );

public:
  virtual ExportXML& operator << ( const ProofLib::ProofLink& p );
  virtual ExportXML& operator << ( const ProofLib::Variable& v );
  virtual ExportXML& operator << ( const ProofLib::SecondOrderVariable& v );
  virtual ExportXML& operator << ( const ProofLib::Constant& c );
  virtual ExportXML& operator << ( const ProofLib::Function& f );
  virtual ExportXML& operator << ( const ProofLib::AbstractTerm& t );
  virtual ExportXML& operator << ( const ProofLib::Term& t );
  virtual ExportXML& operator << ( const ProofLib::LambdaTerm& t );
  virtual ExportXML& operator << ( const ProofLib::StaticAbstractTermList& l );
  virtual ExportXML& operator << ( const ProofLib::AtomFormula& f );
  virtual ExportXML& operator << ( const ProofLib::QuantorFormula& f );
  virtual ExportXML& operator << ( const ProofLib::SecondOrderQuantorFormula& f );
  virtual ExportXML& operator << ( const ProofLib::UnaryConFormula& f );
  virtual ExportXML& operator << ( const ProofLib::BinaryConFormula& f );
  virtual ExportXML& operator << ( const ProofLib::FormulaVariable& f );
  virtual ExportXML& operator << ( const ProofLib::Formula& f );
  virtual ExportXML& operator << ( const ProofLib::FormulaList& l );
  virtual ExportXML& operator << ( const ProofLib::FormulaListVariable& l );
  virtual ExportXML& operator << ( const ProofLib::FormulaListElement& e );
  virtual ExportXML& operator << ( const ProofLib::SElement& e );
  virtual ExportXML& operator << ( const ProofLib::SElementList& l );
  virtual ExportXML& operator << ( const ProofLib::Sequent& s );
  virtual ExportXML& operator << ( const ProofLib::Rule& r );
  virtual ExportXML& operator << ( const ProofLib::RuleItem& ruleitem );
  virtual ExportXML& operator << ( const ProofLib::Proof& p );
  virtual ExportXML& operator << ( const ProofLib::ProofDatabase& pdb );
  virtual ExportXML& operator << ( const ProofLib::DefinedSetFormula& f );
  virtual ExportXML& operator << ( const ProofLib::DefinedSet& f );
  virtual ExportXML& operator << ( const ProofLib::SetTerm& t );

};

////////////////////////////////////////////////////////////////////////////////

#endif
