////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: term_operations.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _EXPRESSION_OPERATIONS_H_
#define _EXPRESSION_OPERATIONS_H_

#include <vector>

#include "util/myassert.h"
#include "libproof.h"
#include "operations.h"

namespace HLK
{
    class SymbolTable;
}

// FIXME: change all getNormalVariable to use instead a central fresh/normal variable
// facility

namespace ProofLib
{

#if 0
// checks for equality modulo alpha-equality w.r.t. lambda-bound variables
// in a _deep_ way, i.e. not with pointer comparisons
class OperationCheckEquality : public ExpressionCompareOperation
{
public:
  virtual void Push( size_t index ) { np.Push(index); }
  virtual size_t Pop() { return np.Pop(); }

private:
  typedef std::pair< Variable*, Variable* > VarMap;
  typedef std::vector< VarMap > VarMapping;

  // record variables for alpha-equality check
  VarMapping m_alpha_map;

  NodePointer np;
public:
  OperationCheckEquality() {}

  DECLARE_APPLY_FUNCTION_FOR_COMPARE( Variable );
  DECLARE_APPLY_FUNCTION_FOR_COMPARE( Constant );

  DECLARE_APPLY_FUNCTION_FOR_COMPARE( LambdaTerm );
  DECLARE_APPLY_FUNCTION_FOR_COMPARE( SecondOrderVariable );
};
#endif

class OperationGetEtaNf : public ExpressionOperation
{
protected:
  NodeDatabase *m_ndb;
  HLK::SymbolTable *m_symtbl;
  std::vector<Variable*> normal_variables;

  Variable* getNormalVariable( size_t index, USTRING type_name );
public:
  NodeDatabase* GetNodeDatabase() { return m_ndb; }
  ProofDatabase* GetProofDatabase() { ASSERT(false); return NULL; }

public:
  OperationGetEtaNf( NodeDatabase *ndb, HLK::SymbolTable* symtbl ) :
    m_ndb( ndb ), m_symtbl( symtbl ) {}
  
//  DECLARE_APPLY_FUNCTION( AtomFormula, Formula );
//  DECLARE_APPLY_FUNCTION( DefinedSet, SetTerm );
  DECLARE_APPLY_FUNCTION( SecondOrderVariable, SetTerm );
};

////////////////////////////////////////////////////////////////////////////////
class OperationReplaceTerm : public ExpressionOperation
{
protected:
  NodeDatabase *m_ndb;
  
  Term &replacement;
  const NodePointer &position;
  NodePointer np;

public:
  NodeDatabase* GetNodeDatabase() { return m_ndb; }
  ProofDatabase* GetProofDatabase() { ASSERT(false); return NULL; }

  virtual void Push( size_t index ) { np.Push(index); }
  virtual size_t Pop() { return np.Pop(); }

public:
  OperationReplaceTerm( NodeDatabase *ndb, Term &replacement, const NodePointer &position, const NodePointer &start = NodePointer() ) :
    m_ndb( ndb ), replacement( replacement ), position( position ), np( start ) {}
  
  DECLARE_APPLY_FUNCTION( AtomFormula, Formula );
  DECLARE_APPLY_FUNCTION( DefinedSetFormula, Formula );
  DECLARE_APPLY_FUNCTION( Variable, Term );
  DECLARE_APPLY_FUNCTION( Constant, Term );
  DECLARE_APPLY_FUNCTION( Function, Term );
};

////////////////////////////////////////////////////////////////////////////////

class OperationGetTerms :
  public ExpressionOperation
{
protected:
  TermList &list;
  NodePointers &nps;
  NodePointer np;
public:
  NodeDatabase* GetNodeDatabase() { ASSERT(false); return NULL; }
  ProofDatabase* GetProofDatabase() { ASSERT(false); return NULL; }

  virtual void Push( size_t index ) { np.Push(index); }
  virtual size_t Pop() { return np.Pop(); }

public:
  OperationGetTerms( TermList &list, NodePointers &nps ) :
    list( list ), nps( nps ) {}

  DECLARE_APPLY_FUNCTION( AtomFormula, Formula );
  DECLARE_APPLY_FUNCTION( DefinedSetFormula, Formula );
  DECLARE_APPLY_FUNCTION( DefinedSet, SetTerm );
};

////////////////////////////////////////////////////////////////////////////////

class LambdaSubstitution
{
public:
  virtual ~LambdaSubstitution() {}

  virtual size_t GetCount() const = 0;

  virtual const SecondOrderVariable* GetVar( size_t index ) const = 0;
  virtual const LambdaTerm* GetSubst( size_t index ) const = 0;

  const LambdaTerm* GetSubstFor( const SecondOrderVariable *v ) const;
};

////////////////////////////////////////////////////////////////////////////////

class OperationLambdaSubstitution :
  public ExpressionOperation,
  public LambdaSubstitution
{
  ProofDatabase *m_pdb;
  NodeDatabase  *m_ndb;

  typedef std::pair< const SecondOrderVariable*, const LambdaTerm* > Substitution;

  std::vector< Substitution > substitutions;

  std::vector<SecondOrderVariable*> bounded;

  NodePointer m_np;
  typedef std::vector<NodePointers> PositionList;
  PositionList m_positions;

public:
  OperationLambdaSubstitution( ProofDatabase *pdb,
                                 const SecondOrderVariable *v, const LambdaTerm *subst );
  OperationLambdaSubstitution( NodeDatabase *ndb,
                                 const SecondOrderVariable *v, const LambdaTerm *subst );
  OperationLambdaSubstitution( NodeDatabase *ndb );
  virtual ~OperationLambdaSubstitution() {}

  void AddSubstitution( const SecondOrderVariable *v, const LambdaTerm *subst );

  NodeDatabase* GetNodeDatabase() { return m_ndb; }
  ProofDatabase* GetProofDatabase() { return m_pdb; }

  virtual void Push( size_t index ) { m_np.Push( index ); }
  virtual size_t Pop() { return m_np.Pop(); }

  virtual size_t GetCount() const { return substitutions.size(); }

  virtual const SecondOrderVariable* GetVar( size_t index ) const;
  virtual const LambdaTerm* GetSubst( size_t index ) const;

  const NodePointers& GetPositions( size_t index ) const;
  const PositionList& GetPositions() const { return m_positions; }

  virtual std::ostream& ToString( std::ostream &out ) const;

  DECLARE_APPLY_FUNCTION( AtomFormula, Formula );
  DECLARE_APPLY_FUNCTION( SecondOrderVariable, SetTerm );
  DECLARE_APPLY_FUNCTION( SecondOrderQuantorFormula, Formula );
};

////////////////////////////////////////////////////////////////////////////////

class OperationGetSecondOrderVariableSubstitutions :
  public ExpressionCompareOperation,
  public LambdaSubstitution
{
public:
  typedef struct {
    NodePointer          var;
    NodePointers         substs;
  } Positions;

protected:
  std::vector<SecondOrderVariable*> vars;
  std::vector<LambdaTerm*> reps;
  std::vector<SecondOrderVariable*> bounded;
  std::vector<Positions> pos;
  std::vector<SecondOrderVariable*> equals;

  ProofDatabase *m_pdb;
  NodeDatabase *m_ndb;
  HLK::SymbolTable *m_symtbl;

  NodePointer np;

  std::vector<Variable*> normal_variables;

  Variable* getNormalVariable( size_t index, USTRING type_name );

public:
  virtual void Push( size_t index ) { np.Push(index); }
  virtual size_t Pop() { return np.Pop(); }

  ProofDatabase* GetProofDatabase() { return m_pdb; }
  NodeDatabase* GetNodeDatabase() { return m_ndb; }

public:
  OperationGetSecondOrderVariableSubstitutions( ProofDatabase *pdb, NodeDatabase *ndb, HLK::SymbolTable *symtbl ) :
    m_pdb( pdb ), m_ndb( ndb ), m_symtbl( symtbl ) {}
  virtual ~OperationGetSecondOrderVariableSubstitutions() {}

  void Clear();

  bool CheckForSubstitutions( DataNode *general, DataNode *instance );

  virtual size_t GetCount() const { return vars.size(); }

  virtual const SecondOrderVariable* GetVar( size_t index ) const;
  virtual const LambdaTerm* GetSubst( size_t index ) const;

  virtual const NodePointer& GetVarPos( size_t index ) const;
  virtual const NodePointers& GetSubstPos( size_t index ) const;

  virtual std::ostream& ToString( std::ostream &out ) const;

  DECLARE_APPLY_FUNCTION_FOR_COMPARE( AtomFormula );
  DECLARE_APPLY_FUNCTION_FOR_COMPARE( SecondOrderQuantorFormula );
  DECLARE_APPLY_FUNCTION_FOR_COMPARE( SecondOrderVariable );
};

////////////////////////////////////////////////////////////////////////////////

// Generalized variant of OperationGetSecondOrderVariableSubstitutions
// OperationGetSecondOrderVariableSubstitutions finds second order
// variable renamings, while this operation finds a set of lambda
// terms L for two formulas F, G s.t. F[X<-l]=G iff l \in L (modulo
// alpha conversion) 

class OperationGetLambdaSubstitutions :
  public ExpressionCompareOperation,
  public LambdaSubstitution
{
public:
  typedef struct {
    NodePointer          var;
    NodePointers         substs;
  } Positions;

protected:
  std::vector<SecondOrderVariable*> vars;
  std::vector< std::set<LambdaTerm*> > reps;    // sets of possible
                                                // lambdaterms
  std::vector<SecondOrderVariable*> bounded;
  std::vector<Positions> pos;
  std::vector<Formula*> permutations;
  TermList blacklist;   // these variables may not appear
                        // in the computed lambdaterms
                        // to satisfy the condition of the
                        // weak quantor rule 

  std::vector<Variable*> normal_variables;

  ProofDatabase *m_pdb;
  NodeDatabase *m_ndb;
  HLK::SymbolTable *m_symtbl;

  NodePointer np;
  
  Variable* getNormalVariable( size_t index, USTRING type_name );
  
  void createPermutedFormulas( NodePointers &term_positions,
                               //NodePointer &current,
                               TermList &terms,
                               TermList &variables,
                               TermList &arguments,
                               size_t pos,
                               Formula *f );

  NodeDatabase* GetNodeDatabase() { return m_ndb; }

public:
  virtual void Push( size_t index ) { np.Push(index); }
  virtual size_t Pop() { return np.Pop(); }

public:
  OperationGetLambdaSubstitutions( ProofDatabase *pdb, NodeDatabase *ndb, HLK::SymbolTable *symtbl, TermList &blacklist ) :
    blacklist( blacklist ), m_pdb( pdb ), m_ndb( ndb ), m_symtbl( symtbl ) {}
  virtual ~OperationGetLambdaSubstitutions() {}

  void Clear();

  bool CheckForSubstitutions( DataNode *general, DataNode *instance );

  virtual size_t GetCount() const { return vars.size(); }

  virtual const SecondOrderVariable* GetVar( size_t index ) const;
  virtual const std::set<LambdaTerm*> GetSubstSet( size_t index ) const;
  virtual const LambdaTerm* GetSubst( size_t index ) const;

  virtual const NodePointer& GetVarPos( size_t index ) const;
  virtual const NodePointers& GetSubstPos( size_t index ) const;

  DECLARE_APPLY_FUNCTION_FOR_COMPARE( AtomFormula );
  DECLARE_APPLY_FUNCTION_FOR_COMPARE( SecondOrderQuantorFormula );
};

////////////////////////////////////////////////////////////////////////////////

class OperationReplaceFormula :
  public ExpressionOperation
{
protected:
  Formula *search;
  Formula *replacement;

  NodeDatabase *ndb;

protected:
  NodeDatabase* GetNodeDatabase() { return ndb; }
  ProofDatabase* GetProofDatabase();

public:
  OperationReplaceFormula( NodeDatabase *ndb,
                           Formula *to_search, Formula *its_replacement );

  DECLARE_APPLY_FUNCTION( AtomFormula, Formula );
  DECLARE_APPLY_FUNCTION( UnaryConFormula, Formula );
  DECLARE_APPLY_FUNCTION( QuantorFormula, Formula );
  DECLARE_APPLY_FUNCTION( SecondOrderQuantorFormula, Formula );
  DECLARE_APPLY_FUNCTION( BinaryConFormula, Formula );
  DECLARE_APPLY_FUNCTION( FormulaVariable, Formula );
};

////////////////////////////////////////////////////////////////////////////////

class OperationReplaceFormulaSequence :
  public ExpressionOperation
{
protected:
  const FormulaList *search;
  FormulaListElement *replacement;

  NodeDatabase *ndb;

protected:
  NodeDatabase* GetNodeDatabase() { return ndb; }
  ProofDatabase* GetProofDatabase();

public:
  OperationReplaceFormulaSequence( NodeDatabase *ndb,
                                   const FormulaList *to_search,
                                   FormulaListElement *its_replacement );

  DECLARE_APPLY_FUNCTION( FormulaList, FormulaList );
  DECLARE_APPLY_FUNCTION( SElementList, SElementList );
};

////////////////////////////////////////////////////////////////////////////////

class OperationReplaceFormulaListElementBySequence :
  public ExpressionOperation
{
protected:
  FormulaListElement *search;
  const FormulaList *replacement;

  NodeDatabase *ndb;

protected:
  NodeDatabase* GetNodeDatabase() { return ndb; }
  ProofDatabase* GetProofDatabase();

public:
  OperationReplaceFormulaListElementBySequence( NodeDatabase *ndb,
                                   FormulaListElement *to_search,
                                   const FormulaList *its_replacement );

  DECLARE_APPLY_FUNCTION( SElementList, SElementList );
};




////////////////////////////////////////////////////////////////////////////////

class OperationGetVariables :
  public ExpressionOperation
{
protected:
  TermList &list;

public:
  NodeDatabase* GetNodeDatabase() { ASSERT(false); return NULL; }
  ProofDatabase* GetProofDatabase() { ASSERT(false); return NULL; }

  virtual void Push( size_t index ) {}
  virtual size_t Pop() { return 0; }

public:
  OperationGetVariables( TermList &list ) : list( list ) {}

  DECLARE_APPLY_FUNCTION( Variable, Term );
  DECLARE_APPLY_FUNCTION( QuantorFormula, Formula );
};

////////////////////////////////////////////////////////////////////////////////

class OperationGetAllVariables :
  public ExpressionOperation
{
protected:
  AbstractTermList &list;

public:
  NodeDatabase* GetNodeDatabase() { ASSERT(false); return NULL; }
  ProofDatabase* GetProofDatabase() { ASSERT(false); return NULL; }

  virtual void Push( size_t index ) {}
  virtual size_t Pop() { return 0; }

public:
  OperationGetAllVariables( AbstractTermList &list ) : list( list ) {}

  DECLARE_APPLY_FUNCTION( Variable, Term );
  DECLARE_APPLY_FUNCTION( QuantorFormula, Formula );
  DECLARE_APPLY_FUNCTION( SecondOrderVariable, SetTerm );
  DECLARE_APPLY_FUNCTION( SecondOrderQuantorFormula, Formula );
};


////////////////////////////////////////////////////////////////////////////////

/* Gets the direct subterms of a term and nodpointers to them.
 * If the term is a constant or a variable, the empty list is returned.
 * If the term is a function, the list contains the arguments of the function
 * and pointers to them.
 */

class OperationGetSubterms : public ExpressionOperation
{
protected:
  TermList &list;
  NodePointers &nps;
  NodePointer np;
public:
  NodeDatabase* GetNodeDatabase() { ASSERT(false); return NULL; }
  ProofDatabase* GetProofDatabase() { ASSERT(false); return NULL; }

  virtual void Push( size_t index ) { np.Push(index); }
  virtual size_t Pop() { return np.Pop(); }

public:
  OperationGetSubterms( TermList &list, NodePointers &nps, NodePointer start ) :
    list( list ), nps( nps ), np( start ) {}

  DECLARE_APPLY_FUNCTION( Function, Term );
};

////////////////////////////////////////////////////////////////////////////////

#if 0
class OperationGetFirstAndSecondOrderVariables :
  public TermOperation
{
protected:
  DefinitionArgumentList &list;

public:
  NodeDatabase* GetNodeDatabase() { ASSERT(false); return NULL; }
  ProofDatabase* GetProofDatabase() { ASSERT(false); return NULL; }

  virtual void Push( size_t index ) {}
  virtual size_t Pop() { return 0; }

public:
  OperationGetFirstAndSecondOrderVariables( DefinitionArgumentList &list ) : list( list ) {}

  DECLARE_APPLY_FUNCTION( Variable, Term );
  DECLARE_APPLY_FUNCTION( QuantorFormula, Formula );
  DECLARE_APPLY_FUNCTION( SecondOrderQuantorFormula, Formula );
  DECLARE_APPLY_FUNCTION( AtomFormula, Formula );
};
#endif 

////////////////////////////////////////////////////////////////////////////////

class OperationCountFreeVariables :
  public ExpressionOperation
{
protected:
  NodePointer np;
  TermList bounded;
  int varcount;

public:
  NodeDatabase* GetNodeDatabase() { ASSERT(false); return NULL; }
  ProofDatabase* GetProofDatabase() { ASSERT(false); return NULL; }

  virtual void Push( size_t index ) { np.Push(index); }
  virtual size_t Pop() { return np.Pop(); }

  int GetCount() const { return varcount; }

public:
  OperationCountFreeVariables( TermList &list ) : varcount( 0 ) {}

  DECLARE_APPLY_FUNCTION( LambdaTerm, SetTerm );
  DECLARE_APPLY_FUNCTION( Variable, Term );
  DECLARE_APPLY_FUNCTION( QuantorFormula, Formula );
};

////////////////////////////////////////////////////////////////////////////////

class OperationCountBoundVariables :
  public ExpressionOperation
{
protected:
  NodePointer np;
  TermList bounded;
  int varcount;

public:
  NodeDatabase* GetNodeDatabase() { ASSERT(false); return NULL; }
  ProofDatabase* GetProofDatabase() { ASSERT(false); return NULL; }

  virtual void Push( size_t index ) { np.Push(index); }
  virtual size_t Pop() { return np.Pop(); }

  int GetCount() const { return varcount; }

public:
  OperationCountBoundVariables( TermList &list ) : varcount( 0 ) {}

  DECLARE_APPLY_FUNCTION( LambdaTerm, SetTerm );
  DECLARE_APPLY_FUNCTION( Variable, Term );
  DECLARE_APPLY_FUNCTION( QuantorFormula, Formula );
};

////////////////////////////////////////////////////////////////////////////////

class OperationGetFreeVariablesAndPositions :
  public ExpressionOperation
{
public:
  typedef std::vector<NodePointers> PositionList;

protected:
  TermList &list;
  PositionList pos;
  NodePointer np;

  TermList bounded;

public:
  NodeDatabase* GetNodeDatabase() { ASSERT(false); return NULL; }
  ProofDatabase* GetProofDatabase() { ASSERT(false); return NULL; }

  virtual void Push( size_t index ) { np.Push(index); }
  virtual size_t Pop() { return np.Pop(); }

  const NodePointers& GetPositions( size_t index ) const;

public:
  OperationGetFreeVariablesAndPositions( TermList &list ) : list( list ) {}

  DECLARE_APPLY_FUNCTION( Variable, Term );
  DECLARE_APPLY_FUNCTION( QuantorFormula, Formula );
};

////////////////////////////////////////////////////////////////////////////////

class OperationGetAllFreeVariablesAndPositions :
  public ExpressionOperation
{
public:
  typedef std::vector<NodePointers> PositionList;

protected:
  AbstractTermList &list;
  PositionList pos;
  NodePointer np;

  AbstractTermList bounded;

public:
  NodeDatabase* GetNodeDatabase() { ASSERT(false); return NULL; }
  ProofDatabase* GetProofDatabase() { ASSERT(false); return NULL; }

  virtual void Push( size_t index ) { np.Push(index); }
  virtual size_t Pop() { return np.Pop(); }

  const NodePointers& GetPositions( size_t index ) const;

public:
  OperationGetAllFreeVariablesAndPositions( AbstractTermList &list ) : list( list ) {}

  DECLARE_APPLY_FUNCTION( Variable, Term );
  DECLARE_APPLY_FUNCTION( QuantorFormula, Formula );
  DECLARE_APPLY_FUNCTION( SecondOrderVariable, SetTerm );
  DECLARE_APPLY_FUNCTION( SecondOrderQuantorFormula, Formula );
};


////////////////////////////////////////////////////////////////////////////////

class OperationGetFreeVariables :
  public OperationGetVariables
{
public:
  OperationGetFreeVariables( TermList &list )
    : OperationGetVariables( list ) {}

  DECLARE_APPLY_FUNCTION( QuantorFormula, Formula );
};

////////////////////////////////////////////////////////////////////////////////

class OperationGetAllFreeVariables :
  public OperationGetAllVariables
{
public:
  OperationGetAllFreeVariables( AbstractTermList &list )
    : OperationGetAllVariables( list ) {}

  DECLARE_APPLY_FUNCTION( QuantorFormula, Formula );
  DECLARE_APPLY_FUNCTION( SecondOrderQuantorFormula, Formula );
  DECLARE_APPLY_FUNCTION( LambdaTerm, SetTerm );
};


////////////////////////////////////////////////////////////////////////////////
#if 0
class OperationGetFreeFirstAndSecondOrderVariables :
  public OperationGetFirstAndSecondOrderVariables
{
public:
  OperationGetFreeFirstAndSecondOrderVariables( DefinitionArgumentList &list )
    : OperationGetFirstAndSecondOrderVariables( list ) {}

  DECLARE_APPLY_FUNCTION( QuantorFormula, Formula );
  DECLARE_APPLY_FUNCTION( SecondOrderQuantorFormula, Formula );
};
#endif
////////////////////////////////////////////////////////////////////////////////

class OperationGetBoundedVariables :
  public ExpressionOperation
{
protected:
  TermList &list;

public:
  NodeDatabase* GetNodeDatabase() { ASSERT(false); return NULL; }
  ProofDatabase* GetProofDatabase() { ASSERT(false); return NULL; }

  virtual void Push( size_t index ) {}
  virtual size_t Pop() { return 0; }

public:
  OperationGetBoundedVariables( TermList &list ) : list( list ) {}

  DECLARE_APPLY_FUNCTION( QuantorFormula, Formula );
};

////////////////////////////////////////////////////////////////////////////////
class AllVariableSubstitution
{
public:
  virtual ~AllVariableSubstitution() {}

  virtual size_t GetCount() const = 0;

  virtual const AbstractTerm* GetVar( size_t index ) const = 0;
  virtual const AbstractTerm* GetSubst( size_t index ) const = 0;

  const AbstractTerm* GetSubstFor( const AbstractTerm *v ) const;
};

////////////////////////////////////////////////////////////////////////////////
class VariableSubstitution
{
public:
  virtual ~VariableSubstitution() {}

  virtual size_t GetCount() const = 0;

  virtual const Variable* GetVar( size_t index ) const = 0;
  virtual const Term* GetSubst( size_t index ) const = 0;

  const Term* GetSubstFor( const Variable *v ) const;
};
////////////////////////////////////////////////////////////////////////////////

class OperationGetAllVariableSubstitutions :
  public ExpressionCompareOperation,
  public AllVariableSubstitution
{
public:
  typedef struct {
    NodePointer          var;
    NodePointers         substs;
  } Positions;

protected:
  AbstractTermList vars;
  std::vector< std::set<AbstractTerm*> > reps;
  AbstractTermList bounded;
  AbstractTermList equals;
  std::vector<Positions> pos;
  bool equals_too;
  std::vector<Variable*> normal_variables;
  ProofDatabase *m_pdb;
  NodeDatabase *m_ndb;
  HLK::SymbolTable *m_symtbl;
  NodePointer np;
  std::vector<Formula*> permutations;
  const TermList blacklist;   // these variables may not appear
                        // in the computed lambdaterms
                        // e.g. to satisfy the condition of the
                        // weak quantor rule 



  Variable* getNormalVariable( size_t index, USTRING type_name );
  
  void createPermutedFormulas( NodePointers &term_positions,
                               //NodePointer &current,
                               TermList &terms,
                               TermList &variables,
                               TermList &arguments,
                               size_t pos,
                               Formula *f );


public:
  virtual void Push( size_t index ) { np.Push(index); }
  virtual size_t Pop() { return np.Pop(); }

  virtual std::ostream& ToString( std::ostream &out ) const;

public:
  OperationGetAllVariableSubstitutions( ProofDatabase *pdb, NodeDatabase *ndb, HLK::SymbolTable *symtbl, bool equals_too = false, const TermList &blacklist = TermList()) :
    m_pdb( pdb ), m_ndb( ndb ), m_symtbl( symtbl ), blacklist( blacklist )
    { this->equals_too = equals_too; }
  virtual ~OperationGetAllVariableSubstitutions() {}

  void Clear();

  bool CheckForSubstitutions( DataNode *general, DataNode *instance );

  virtual size_t GetCount() const { return vars.GetChildCount(); }

  const std::set<AbstractTerm*> GetSubstSet( size_t index ) const;
  void SetSubstSet( size_t index, std::set<AbstractTerm*> &s );

  virtual const AbstractTerm* GetVar( size_t index ) const;
  virtual const AbstractTerm* GetSubst( size_t index ) const;

  virtual const NodePointer& GetVarPos( size_t index ) const;
  virtual const NodePointers& GetSubstPos( size_t index ) const;

  DECLARE_APPLY_FUNCTION_FOR_COMPARE( Variable );
  DECLARE_APPLY_FUNCTION_FOR_COMPARE( QuantorFormula );
  DECLARE_APPLY_FUNCTION_FOR_COMPARE( SecondOrderQuantorFormula );
  DECLARE_APPLY_FUNCTION_FOR_COMPARE( AtomFormula );
};

////////////////////////////////////////////////////////////////////////////////


class OperationGetVariableSubstitutions :
  public ExpressionCompareOperation,
  public VariableSubstitution
{
public:
  typedef struct {
    NodePointer          var;
    NodePointers         substs;
  } Positions;

private:
  NodeDatabase *m_ndb;

protected:
  TermList vars;
  TermList reps;
  TermList bounded;
  TermList equals;
  std::vector<Positions> pos;
  bool equals_too;

  NodePointer np;
public:
  virtual void Push( size_t index ) { np.Push(index); }
  virtual size_t Pop() { return np.Pop(); }

public:
  OperationGetVariableSubstitutions( NodeDatabase *ndb, bool equals_too = false ) : m_ndb( ndb )
    { this->equals_too = equals_too; }
  virtual ~OperationGetVariableSubstitutions() {}

  NodeDatabase* GetNodeDatabase() { return m_ndb; }

  void Clear();

  bool CheckForSubstitutions( DataNode *general, DataNode *instance );

  virtual size_t GetCount() const { return vars.GetChildCount(); }

  virtual const Variable* GetVar( size_t index ) const;
  virtual const Term* GetSubst( size_t index ) const;

  virtual const NodePointer& GetVarPos( size_t index ) const;
  virtual const NodePointers& GetSubstPos( size_t index ) const;

  DECLARE_APPLY_FUNCTION_FOR_COMPARE( Variable );
  DECLARE_APPLY_FUNCTION_FOR_COMPARE( QuantorFormula );
};

////////////////////////////////////////////////////////////////////////////////

class OperationAllVariableSubstitution :
  public ExpressionOperation,
  public AllVariableSubstitution
{
  ProofDatabase *m_pdb;
  NodeDatabase  *m_ndb;

  typedef std::pair< const AbstractTerm*, const AbstractTerm* > Substitution;

  std::vector< Substitution > substitutions;

  AbstractTermList bounded;

  NodePointer m_np;
  typedef std::vector<NodePointers> PositionList;
  PositionList m_positions;

public:
  OperationAllVariableSubstitution( ProofDatabase *pdb,
                                 const AbstractTerm *v, const AbstractTerm *subst );
  OperationAllVariableSubstitution( NodeDatabase *ndb,
                                 const AbstractTerm *v, const AbstractTerm *subst );
  OperationAllVariableSubstitution( NodeDatabase *ndb,
                                 const AllVariableSubstitution &subst );
  OperationAllVariableSubstitution( NodeDatabase *ndb );
  OperationAllVariableSubstitution( ProofDatabase *pdb );
  virtual ~OperationAllVariableSubstitution() {}

  void AddSubstitution( const AbstractTerm *v, const AbstractTerm *subst );

  NodeDatabase* GetNodeDatabase() { return m_ndb; }
  ProofDatabase* GetProofDatabase() { return m_pdb; }

  virtual void Push( size_t index ) { m_np.Push( index ); }
  virtual size_t Pop() { return m_np.Pop(); }

  virtual size_t GetCount() const { return substitutions.size(); }

  virtual const AbstractTerm* GetVar( size_t index ) const;
  virtual const AbstractTerm* GetSubst( size_t index ) const;

  const NodePointers& GetPositions( size_t index ) const;
  const PositionList& GetPositions() const { return m_positions; }

  virtual std::ostream& ToString( std::ostream &out ) const;

  DECLARE_APPLY_FUNCTION( Variable, Term );
  DECLARE_APPLY_FUNCTION( QuantorFormula, Formula );
  DECLARE_APPLY_FUNCTION( SecondOrderQuantorFormula, Formula );
  DECLARE_APPLY_FUNCTION( AtomFormula, Formula );
  DECLARE_APPLY_FUNCTION( SecondOrderVariable, SetTerm );
};

////////////////////////////////////////////////////////////////////////////////


class OperationVariableSubstitution :
  public ExpressionOperation,
  public VariableSubstitution
{
  ProofDatabase *m_pdb;
  NodeDatabase  *m_ndb;

  typedef std::pair< const Variable*, const Term* > Substitution;

  std::vector< Substitution > substitutions;

  TermList bounded;

  NodePointer m_np;
  typedef std::vector<NodePointers> PositionList;
  PositionList m_positions;

public:
  OperationVariableSubstitution( ProofDatabase *pdb,
                                 const Variable *v, const Term *subst );
  OperationVariableSubstitution( NodeDatabase *ndb,
                                 const Variable *v, const Term *subst );
  OperationVariableSubstitution( NodeDatabase *ndb,
                                 const VariableSubstitution &subst );
  OperationVariableSubstitution( NodeDatabase *ndb );
  OperationVariableSubstitution( ProofDatabase *pdb );
  virtual ~OperationVariableSubstitution() {}

  void AddSubstitution( const Variable *v, const Term *subst );

  NodeDatabase* GetNodeDatabase() { return m_ndb; }
  ProofDatabase* GetProofDatabase() { return m_pdb; }

  virtual void Push( size_t index ) { m_np.Push( index ); }
  virtual size_t Pop() { return m_np.Pop(); }

  virtual size_t GetCount() const { return substitutions.size(); }

  virtual const Variable* GetVar( size_t index ) const;
  virtual const Term* GetSubst( size_t index ) const;

  const NodePointers& GetPositions( size_t index ) const;
  const PositionList& GetPositions() const { return m_positions; }

  virtual std::ostream& ToString( std::ostream &out ) const;

  DECLARE_APPLY_FUNCTION( Variable, Term );
  DECLARE_APPLY_FUNCTION( QuantorFormula, Formula );
};

////////////////////////////////////////////////////////////////////////////////

class OperationTermSubstitution :
  public ExpressionOperation
{
private:
//  ProofDatabase *m_pdb;
  NodeDatabase  *m_ndb;

  const Term *t;
  const Term *subst;

  NodePointers m_positions;

  const NodePointers *m_positions_allowed;

  NodePointer m_np;

public:
//  OperationTermSubstitution( ProofDatabase *pdb,
  OperationTermSubstitution( NodeDatabase *ndb,
                             const Term *t, const Term *subst,
                             const NodePointers* positions_allowed = NULL );
  virtual ~OperationTermSubstitution() {}

  NodeDatabase* GetNodeDatabase() { return m_ndb; }
  ProofDatabase* GetProofDatabase() { ASSERT(false); return NULL; }

  virtual void Push( size_t index );
  virtual size_t Pop();

  const NodePointers& GetPositions() const { return m_positions; }

  DECLARE_APPLY_FUNCTION( Variable, Term );
  DECLARE_APPLY_FUNCTION( Constant, Term );
  DECLARE_APPLY_FUNCTION( Function, Term );
};

////////////////////////////////////////////////////////////////////////////////

class OperationMGUSimple :
  public ExpressionCompareOperation,
  public VariableSubstitution
{
  typedef std::pair< const Variable*, const Term* > Unification;
  typedef std::vector< Unification > UnificationTable;
  typedef enum { Left, Right } Side;

  UnificationTable m_table;

  NodeDatabase *m_ndb;

public:
  OperationMGUSimple( NodeDatabase *ndb ) { m_ndb = ndb; }

  NodeDatabase* GetNodeDatabase() { return m_ndb; }

  bool Unify( const DataNode *left, const DataNode *right );

  virtual void Push( size_t index ) {}
  virtual size_t Pop() { return 0; }

  virtual std::ostream& ToString( std::ostream &out ) const;

  virtual size_t GetCount() const { return m_table.size(); }

  virtual const Variable* GetVar( size_t index ) const;
  virtual const Term* GetSubst( size_t index ) const;

protected:
  bool AddSubstition( Side s, const Variable *v, const Term *t );

  DECLARE_APPLY_FUNCTION_FOR_COMPARE( Variable );
  DECLARE_APPLY_FUNCTION_FOR_COMPARE( Constant );
  DECLARE_APPLY_FUNCTION_FOR_COMPARE( Function );
};

////////////////////////////////////////////////////////////////////////////////
class FreshVariableGenerator
{
public:
  virtual ~FreshVariableGenerator() {}

  virtual Variable* GetVariableNum( size_t ) = 0;
  virtual NodeDatabase* GetNodeDatabase() = 0;
};

////////////////////////////////////////////////////////////////////////////////

class FreshVarMap : public VariableSubstitution
{
  VarMapping m_map;

  FreshVariableGenerator *m_fg;
  size_t m_next_var_num;

public:
  FreshVarMap( FreshVariableGenerator *fg, const DataNode *node,
               size_t start = 0 );
  FreshVarMap( const FreshVarMap &o ) { (*this) = o; }
  virtual ~FreshVarMap();

  void operator=( const FreshVarMap &o );

  void AddVarsFromNode( const DataNode *node );

  size_t GetNextVarNum() const { return m_next_var_num; }

  virtual size_t GetCount() const { return m_map.size(); }
  virtual const Variable* GetVar( size_t index ) const;
  virtual const Variable* GetSubst( size_t index ) const;
};

////////////////////////////////////////////////////////////////////////////////

class OperationMGU :
  public ExpressionCompareOperation
{
public:
  typedef enum { Left, Right } Side;
private:
  typedef std::pair< Variable*, Term* > Unification;

  class UnificationTable : private std::vector< Unification >,
                           public VariableSubstitution
  {
  public:
    UnificationTable() {}
    UnificationTable( const UnificationTable &o ) { (*this) = o; }
    ~UnificationTable();
    void operator=( const UnificationTable &o );

    void AddSubstitution( const Variable* v, const Term* t );
    void ApplyOperation( ExpressionOperation &op );
    virtual size_t GetCount() const { return size(); }
    virtual const Variable* GetVar( size_t index ) const;
    virtual const Term* GetSubst( size_t index ) const;
    virtual std::ostream& ToString( std::ostream &out ) const;
  };

  UnificationTable m_table;

#if 0
  typedef std::pair< const Variable*, const Variable* > VarMap;
  typedef std::vector< VarMap > VarMapping;

  VarMapping m_map_l;
  VarMapping m_map_r;
#endif
  const FreshVarMap *m_map_l;
  const FreshVarMap *m_map_r;

  size_t m_new_var_count;

  FreshVariableGenerator *m_fg;

public:
  OperationMGU( FreshVariableGenerator *fg,
                const FreshVarMap *m1, const FreshVarMap *m2 );
/*
  OperationMGU( const OperationMGU &o ) { (*this) = o; }
  void operator=( const OperationMGU &o );
*/

  NodeDatabase* GetNodeDatabase() { return m_fg->GetNodeDatabase(); }

  bool Unify( const DataNode *left, const DataNode *right );

  virtual void Push( size_t index ) {}
  virtual size_t Pop() { return 0; }

  virtual std::ostream& ToString( std::ostream &out ) const;

  void GetSubstitution( Side s, OperationVariableSubstitution &op ) const;

  virtual void Finalize();

protected:
#if 0
  virtual size_t GetCount() const { return m_table.GetCount(); }

  virtual const Variable* GetVar( size_t index ) const;
  virtual const Term* GetSubst( size_t index ) const;
#endif

  const Variable* MapVar( Side s, const Variable *v );
  bool AddSubstition( Side s, const Variable *v, const Term *t );
  bool AddSubstition( const Variable *v, const Term *t );

  DECLARE_APPLY_FUNCTION_FOR_COMPARE( Variable );
  DECLARE_APPLY_FUNCTION_FOR_COMPARE( Constant );
  DECLARE_APPLY_FUNCTION_FOR_COMPARE( Function );
};

////////////////////////////////////////////////////////////////////////////////

class Substitution
{
  Variable *var;
  Term     *by;

public:
  Substitution();
  Substitution( const Variable *v, const Term *r );
  Substitution( const Substitution &o );
  ~Substitution();

  void operator=( const Substitution &o );

  const Variable* GetVar() const { return var; }
  const Term* GetSubst() const { return by; }

  std::ostream& ToString( std::ostream &out ) const;
};

////////////////////////////////////////////////////////////////////////////////

class Substitutions : public std::vector<Substitution>
{
public:
  const Term* GetReplacementFor( const Variable *var ) const;

  bool CheckSubstitution( const Variable *var, const Term *rep );

  /// subset of s?
  bool operator < ( const Substitutions &s ) const;

  std::ostream& ToString( std::ostream &out ) const;
};

////////////////////////////////////////////////////////////////////////////////

class SubstPossibilities : public std::vector<Substitutions>
{
public:
  void Add( const Substitutions &s );
  bool Merge( const SubstPossibilities &o );

  std::ostream& ToString( std::ostream &out ) const;
};

////////////////////////////////////////////////////////////////////////////////
}

#endif
