////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: import/import.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#include <string.h>
#include <fstream>

#include "util/myassert.h"
#include "util/fileexceptions.h"

#include "libproof.h"
#include "import.h"

namespace ProofLib
{

////////////////////////////////////////////////////////////////////////////////

ImportModul::ImportModul( ProofLib::ProofDatabase* pdb )
{
  m_pdb = pdb;
}

NodeDatabase* ImportModul::GetNodeDatabase() const
{
  ASSERT( m_pdb != NULL );
  return m_pdb->GetNodeDatabase();
}

SymbolTable* ImportModul::GetSymbolTable() const
{
  ASSERT( m_pdb != NULL );
  return m_pdb->GetSymbolTable();
}

////////////////////////////////////////////////////////////////////////////////

ImportFile::ImportFile( ProofLib::ProofDatabase* pdb )
  : ImportModul( pdb )
{
  m_sourcename  = NULL;
  m_file        = NULL;
  m_isopen      = false;
}

void ImportFile::Open( FILE *file )
{
  ASSERT( m_isopen == false );
  ASSERT( file != NULL );

  m_file = file;
}

void ImportFile::Open( const char *sourcename )
{
  ASSERT( m_isopen == false );
  ASSERT( sourcename != NULL );

  m_sourcename = strdup( sourcename );

  m_file = fopen( m_sourcename, "r" );

  if( m_file == NULL )
    throw new FileOpenFailException( m_sourcename, errno );
}

void ImportFile::Close()
{
  if( m_file != NULL )
    fclose( m_file );

  m_file = NULL;

  if( m_sourcename != NULL )
    free( m_sourcename );

  m_sourcename = NULL;

  m_isopen = false;
}

////////////////////////////////////////////////////////////////////////////////

ImportStream::ImportStream( ProofLib::ProofDatabase* pdb )
  : ImportModul( pdb )
{
  m_sourcename  = NULL;
  m_in          = NULL;
  m_isopen      = false;
}

void ImportStream::Open( std::istream *in )
{
  ASSERT( m_isopen == false );
  ASSERT( in != NULL );

  m_in = in;
}

void ImportStream::Open( const char *sourcename )
{
  ASSERT( m_isopen == false );
  ASSERT( sourcename != NULL );

  m_sourcename = strdup( sourcename );

  m_in = new std::ifstream( m_sourcename );

  if( !(*m_in) )
    throw new FileOpenFailException( m_sourcename, errno );
}

void ImportStream::Close()
{
  if( m_in != NULL )
  {
    delete m_in;
  }

  m_in = NULL;

  if( m_sourcename != NULL )
    free( m_sourcename );

  m_sourcename = NULL;

  m_isopen = false;
}

////////////////////////////////////////////////////////////////////////////////

}
