////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: import/import.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _IMPORT_H_
#define _IMPORT_H_

#include <stdio.h>
#include <istream>

namespace ProofLib
{

class ProofDatabase;
class NodeDatabase;
class SymbolTable;
class Proof;

class ImportModul
{
public:
  ImportModul( ProofDatabase* pdb );
  virtual ~ImportModul() {};

  virtual Proof* Read() = 0;  
  virtual void Open( const char *sourcename ) = 0;  
  virtual void Close() = 0;  

  ProofDatabase* GetProofDatabase() const { return m_pdb; }
  NodeDatabase* GetNodeDatabase() const;
  SymbolTable* GetSymbolTable() const;

protected:
  ProofDatabase* m_pdb;
};

class ImportFile : public ImportModul
{
public:
  ImportFile( ProofDatabase* pdb );

  virtual void Open( FILE *file );  
  virtual void Open( const char *sourcename );  
  virtual void Close();  

protected:
  char          *m_sourcename;
  FILE          *m_file;
  bool           m_isopen;
};

class ImportStream : public ImportModul
{
public:
  ImportStream( ProofDatabase* pdb );

  virtual void Open( std::istream *in );  
  virtual void Open( const char *sourcename );  
  virtual void Close();  

  std::istream& is() const { return *m_in; }

protected:
  char          *m_sourcename;
  std::istream  *m_in;
  bool           m_isopen;
};

}

#endif
