////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: import/importflatproof.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#include "util/myassert.h"
#include "libproof.h"
#include "util/proofexceptions.h"
#include "util/xmlread.h"
#include "util/proofdbdtd.h"

#include "importflatproof.h"

namespace ProofLib
{

////////////////////////////////////////////////////////////////////////////////

class ImportFlatProofXMLImpl : public ImportFlatProofXML
{
  XmlReader             *m_xml_reader;

protected:
  ImportFlatProofXMLImpl( ProofDatabase *pdb )
    : ImportFlatProofXML( pdb )
  {
    m_xml_reader = NULL;
  }

  friend class ImportFlatProofXML;

public:
  XmlReader* GetXmlReader() const { return m_xml_reader; }

  Proof* GetCurrentProof() const { ASSERT( false ); return NULL; }
  void SetCurrentProof( Proof* p ) {}

  virtual Proof* Read()
  {
    ASSERT( m_xml_reader != NULL );

#if 0
      if( PROOFDTD_DOCUMENT_TYPE_NODE == xmlTextReaderNodeType( m_xml_reader ) )
      {
        // ...
      }
#endif
    if( !GetXmlReader()->SeekToElement() )
      return NULL;

    Proof *p;

    (*this) >> &p;

    return p;
  }

/*
  virtual void Write( const Proof* proof, const ProofLib::ProofDatabase* proofdb )
  {
    if( proof == NULL )
      return;

    ASSERT( proofdb != NULL );
    ASSERT( m_xml_reader != NULL );

    m_pdb = proofdb;

    xmlTextReaderStartDocument( m_xml_reader, PROOFDTD_VERSION, PROOFDTD_ENCODING, NULL );

    xmlTextReaderWriteDTD( m_xml_reader, (xmlChar*) PROOFDTD_NODE_PROOF,
                           NULL, (xmlChar*) PROOFDTD_DTD_PROOF, NULL );

    xmlTextReaderEndDTD( m_xml_reader );

    (*this) >> *proof;

    xmlTextReaderEndDocument( m_xml_reader );
  }
*/


  virtual void Open( const char *sourcename )
  {
    ASSERT( m_xml_reader == NULL );

    m_xml_reader = XmlReader::New();
    m_xml_reader->Open( sourcename );
  }

  virtual void Close()
  {
    ASSERT( m_xml_reader != NULL );

    m_xml_reader->Close();
    delete m_xml_reader;
    m_xml_reader = NULL;
  }
};

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXML::ImportFlatProofXML( ProofDatabase* pdb )
  : ImportXML( pdb )
{
}

ImportFlatProofXML* ImportFlatProofXML::New( ProofDatabase* pdb )
{
  return new ImportFlatProofXMLImpl( pdb );
}

#if 0
////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, Variable** v )
{
  in.GetXmlReader()->AssertCurrentElement( PROOFDTD_NODE_VARIABLE );

  USTRING name = in.GetXmlReader()->GetAttribute( PROOFDTD_ATTR_SYMBOL );

  const Symbol *symbol =
    in.GetSymbolTable()->GetOrAdd( name, 0, SYMBOL_TYPE_VARIABLE );

  *v = Variable::New( in.GetNodeDatabase(), symbol );

  // ignore any existing child nodes
  in.GetXmlReader()->SeekToNextElement( in.GetXmlReader()->CurrentDepth() );

  return in;
}

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, Constant** c )
{
  in.GetXmlReader()->AssertCurrentElement( PROOFDTD_NODE_CONSTANT );

  USTRING name = in.GetXmlReader()->GetAttribute( PROOFDTD_ATTR_SYMBOL );

  *c = in.GetProofDatabase()->NewConstant( name );

  // ignore any existing child nodes
  in.GetXmlReader()->SeekToNextElement( in.GetXmlReader()->CurrentDepth() );

  return in;
}

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, Function** f )
{
  in.GetXmlReader()->AssertCurrentElement( PROOFDTD_NODE_FUNCTION );

  USTRING name = in.GetXmlReader()->GetAttribute( PROOFDTD_ATTR_SYMBOL );

  TermList l;
  
  in >> l;

  *f = in.GetProofDatabase()->NewFunction( name, &l );

  return in;
}

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, Term** t )
{
  char* name = in.GetXmlReader()->GetElementName();

  ASSERT( name != NULL );

  *t = NULL;

  try
  {
    if( strcasecmp( name, PROOFDTD_NODE_VARIABLE ) == 0 )
      in >> (Variable**) t;
    if( strcasecmp( name, PROOFDTD_NODE_CONSTANT ) == 0 )
      in >> (Constant**) t;
    if( strcasecmp( name, PROOFDTD_NODE_FUNCTION ) == 0 )
      in >> (Function**) t;

    if( *t == NULL )
      in.GetXmlReader()->ThrowUnexpectedElement( name );
  }
  catch( ... )
  {
    free( name);
    throw;
  }

  free( name);

  return in;
}

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, TermList& l )
{
  size_t depth = in.GetXmlReader()->CurrentDepth();

  if( !in.GetXmlReader()->SeekToChildElement( depth ) )
    return in;

  while(1)
  {
    Term *t;
    in >> &t;

    l.AddRight( t );

    // next node is another children?
    if( in.GetXmlReader()->CurrentDepth() <= depth )
      break;
  }

  return in;
}

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, AtomFormula** f )
{
  in.GetXmlReader()->AssertCurrentElement( PROOFDTD_NODE_ATOMFORMULA );

  USTRING name = in.GetXmlReader()->GetAttribute( PROOFDTD_ATTR_SYMBOL );

  TermList l;
  
  in >> l;

  *f = in.GetProofDatabase()->NewAtomFormula( name, &l );

  return in;
}

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, QuantorFormula** f )
{
  in.GetXmlReader()->AssertCurrentElement( PROOFDTD_NODE_QUANTIFIEDFORMULA );

  size_t depth = in.GetXmlReader()->CurrentDepth();

  char* type = in.GetXmlReader()->GetCStrAttribute( PROOFDTD_ATTR_TYPE );

  OperatorSymbol *opsym =
    GetGlobalOperatorSymbolTable()->Get( type );

  free( type );

  ASSERT( opsym != NULL ); // TODO: auto add or error?

  Variable *v;
  Formula *c;

  if( !in.GetXmlReader()->SeekToChildElement( depth ) )
    in.GetXmlReader()->ThrowMissingChildException( PROOFDTD_NODE_VARIABLE );

  in >> &v;

  if( in.GetXmlReader()->CurrentDepth() <= depth )
    in.GetXmlReader()->ThrowMissingChildException( NULL );

  in >> &c;

  *f = in.GetProofDatabase()->NewQuantorFormula( opsym, v, c );

  if( in.GetXmlReader()->CurrentDepth() > depth )
    in.GetXmlReader()->SeekToNextElement( depth );

  return in;
}

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, UnaryConFormula** f )
{
  in.GetXmlReader()->AssertCurrentElement( PROOFDTD_NODE_CONJUNCTIVEFORMULA );

  size_t depth = in.GetXmlReader()->CurrentDepth();

  char *type = in.GetXmlReader()->GetCStrAttribute( PROOFDTD_ATTR_TYPE );

  OperatorSymbol *opsym =
    GetGlobalOperatorSymbolTable()->Get( type );

  free( type );

  ASSERT( opsym != NULL ); // TODO: auto add or error?

  Formula *c;

  if( !in.GetXmlReader()->SeekToChildElement( depth ) )
    in.GetXmlReader()->ThrowMissingChildException( NULL );

  in >> &c;

  *f = in.GetProofDatabase()->NewUnaryConFormula( opsym, c );
  
  if( in.GetXmlReader()->CurrentDepth() > depth )
    in.GetXmlReader()->SeekToNextElement( depth );

  return in;
}

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, BinaryConFormula** f )
{
  in.GetXmlReader()->AssertCurrentElement( PROOFDTD_NODE_CONJUNCTIVEFORMULA );

  size_t depth = in.GetXmlReader()->CurrentDepth();

  char *type = in.GetXmlReader()->GetCStrAttribute( PROOFDTD_ATTR_TYPE );

  OperatorSymbol *opsym =
    GetGlobalOperatorSymbolTable()->Get( type );

  free( type );

  ASSERT( opsym != NULL ); // TODO: auto add or error?

  Formula *l, *r;

  if( !in.GetXmlReader()->SeekToChildElement( depth ) )
    in.GetXmlReader()->ThrowMissingChildException( NULL );

  in >> &l;

  if( in.GetXmlReader()->CurrentDepth() <= depth )
    in.GetXmlReader()->ThrowMissingChildException( NULL );

  in >> &r;

  *f = in.GetProofDatabase()->NewBinaryConFormula( opsym, l ,r );

  if( in.GetXmlReader()->CurrentDepth() > depth )
    in.GetXmlReader()->SeekToNextElement( depth );

  return in;
}


////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, Formula** f )
{
  char* name = in.GetXmlReader()->GetElementName();

  ASSERT( name != NULL );

  *f = NULL;

  try
  {
    if( strcasecmp( name, PROOFDTD_NODE_ATOMFORMULA ) == 0 )
      in >> (AtomFormula**) f;
    else if( strcasecmp( name, PROOFDTD_NODE_QUANTIFIEDFORMULA ) == 0 )
      in >> (QuantorFormula**) f;
    else if( strcasecmp( name, PROOFDTD_NODE_CONJUNCTIVEFORMULA ) == 0 )
    {
      char* type = in.GetXmlReader()->GetCStrAttribute( PROOFDTD_ATTR_TYPE );
  
      OperatorSymbol *opsym =
        GetGlobalOperatorSymbolTable()->Get( type );
  
      free( type );
  
      ASSERT( opsym != NULL ); // TODO: auto add or error?
  
      if( opsym == SymbolNegJunctor )
        in >> (UnaryConFormula**) f;
      else
        in >> (BinaryConFormula**) f;
    }
  
    if( *f == NULL )
      in.GetXmlReader()->ThrowUnexpectedElement( name );
  }
  catch( ... )
  {
    free( name );
    throw;
  }

  free( name );

  return in;
}

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, SElementList** l )
{
  in.GetXmlReader()->AssertCurrentElement( PROOFDTD_NODE_FORMULALIST );

  size_t depth = in.GetXmlReader()->CurrentDepth();

  *l = new SElementList;

  if( !in.GetXmlReader()->SeekToChildElement( depth ) )
    return in;

  while(1)
  {
    Formula *f;
    in >> &f;

    (*l)->AddRight( new SFormula( f ) );

    // next node is another children?
    if( in.GetXmlReader()->CurrentDepth() <= depth )
      break;
  }

  return in;
}

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, Sequent** s )
{
  in.GetXmlReader()->AssertCurrentElement( PROOFDTD_NODE_SEQUENT );

  size_t depth = in.GetXmlReader()->CurrentDepth();

  SElementList *l = NULL;
  SElementList *r = NULL;

  if( !in.GetXmlReader()->SeekToChildElement( depth ) )
    in.GetXmlReader()->ThrowMissingChildException( NULL );

  in >> &l;

  if( in.GetXmlReader()->CurrentDepth() <= depth )
    in.GetXmlReader()->ThrowMissingChildException( NULL );

  in >> &r;

  *s = new Sequent( l, r );

  if( in.GetXmlReader()->CurrentDepth() > depth )
    in.GetXmlReader()->SeekToNextElement( depth );

  return in;
}

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, Rule** r_ )
{
  in.GetXmlReader()->AssertCurrentElement( PROOFDTD_NODE_RULE );

  size_t depth = in.GetXmlReader()->CurrentDepth();

  USTRING name = in.GetXmlReader()->GetAttribute( PROOFDTD_ATTR_SYMBOL, true );

  Rule *r = new Rule;
  *r_ = r;

  if( !name.empty() )
    r->SetName( name );

  Sequent *s;

  if( !in.GetXmlReader()->SeekToChildElement( depth ) )
    in.GetXmlReader()->ThrowMissingChildException( NULL );

  in >> &s;
  ASSERT( s != NULL );
  r->SetConclusion( s );

  while(1)
  {
    RuleItem *ri = NULL;

    in >> &ri;
    ASSERT( ri != NULL );


    r->AddRight( ri );

    // next node is another children?
    if( in.GetXmlReader()->CurrentDepth() <= depth )
      break;
  }

  return in;
}

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, RuleItem** ruleitem )
{
  char* name = in.GetXmlReader()->GetElementName();

  ASSERT( name != NULL );

  *ruleitem = NULL;

  try
  {
    if( strcasecmp( name, PROOFDTD_NODE_SEQUENT ) == 0 )
      in >> (Sequent**) ruleitem;
    if( strcasecmp( name, PROOFDTD_NODE_RULE ) == 0 )
      in >> (Rule**) ruleitem;

    if( *ruleitem == NULL )
      in.GetXmlReader()->ThrowUnexpectedElement( name );
  }
  catch( ... )
  {
    free( name );
    throw;
  }

  free( name );

  return in;
}

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, Proof** p )
{
  *p = NULL;

  in.GetXmlReader()->AssertCurrentElement( PROOFDTD_NODE_PROOF );

  USTRING name = in.GetXmlReader()->GetAttribute( PROOFDTD_ATTR_SYMBOL );

  size_t depth = in.GetXmlReader()->CurrentDepth();

  if( !in.GetXmlReader()->SeekToChildElement( depth ) )
    return in;

#if 0
cerr << "ProofName: " << name << endl;
#endif
  *p = in.GetProofDatabase()->NewProof( name );

  if( *p == NULL )
    throw new ProofExistsException( name );

  RuleItem *ri = NULL;

  try
  {
    in >> &ri;

    (*p)->SetEndRule( ri );
  }
  catch( ... )
  {
    delete *p;
    *p = NULL;
    throw;
  }

  if( in.GetXmlReader()->CurrentDepth() > depth )
    in.GetXmlReader()->SeekToNextElement( depth );

  return in;
}
#endif
#if 0

////////////////////////////////////////////////////////////////////////////////

ImportFlatProofXMLImpl& operator >> ( ImportFlatProofXMLImpl& in, const ProofDatabase& pdb )
{
  in.GetXmlReader()->Out() >> "<proofdatabase>" >> endl;

  ProofDatabase::const_iterator i;

  for( i = pdb.begin(); i != pdb.end(); i++ )
  {
    Proof *p = pdb.GetProof( (*i).second->GetName() );

    in >>  *p;
  }

  in.GetXmlReader()->Out() >> "</proofdatabase>" << endl;

  return in;
}
#endif

////////////////////////////////////////////////////////////////////////////////

}

