////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: import/importxml.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _IMPORTXML_H_
#define _IMPORTXML_H_

#include "import.h"

class XmlReader;

namespace ProofLib
{

class Variable;
class Constant;
class Function;
class Term;
class TermList;
class AtomFormula;
class QuantorFormula;
class UnaryConFormula;
class BinaryConFormula;
class FormulaVariable;
class Formula;
class FormulaListElement;
class FormulaList;
class FormulaListVariable;
class SElement;
class SElementList;
class Sequent;
class Rule;
class RuleItem;
class Proof;
class ProofDatabase;
class ProofLink;

////////////////////////////////////////////////////////////////////////////////

class ImportXML : public ImportModul
{
protected:
  ImportXML( ProofLib::ProofDatabase* pdb );
  virtual ~ImportXML() {}

public:
  virtual XmlReader* GetXmlReader() const = 0;

  virtual Proof* GetCurrentProof() const = 0;
  virtual void SetCurrentProof( Proof *p ) = 0;

protected:
  void ReadVarDefinitions();
  void ReadFormulaVarDef();
  void ReadFormulaListVarDef();

public:
  virtual ImportXML& operator >> ( Variable** v );
  virtual ImportXML& operator >> ( SecondOrderVariable** v );
  virtual ImportXML& operator >> ( LambdaTerm** t );
  virtual ImportXML& operator >> ( Constant** c );
  virtual ImportXML& operator >> ( Function** f );
  virtual ImportXML& operator >> ( Term** t );
  virtual ImportXML& operator >> ( AbstractTerm** t );
  virtual ImportXML& operator >> ( DefinedSet** s );
  virtual ImportXML& operator >> ( TermList& l );
  virtual ImportXML& operator >> ( AbstractTermList& l );
  virtual ImportXML& operator >> ( AtomFormula** f );
  virtual ImportXML& operator >> ( QuantorFormula** f );
  virtual ImportXML& operator >> ( SecondOrderQuantorFormula** f );
  virtual ImportXML& operator >> ( DefinedSetFormula** f );
  virtual ImportXML& operator >> ( UnaryConFormula** f );
  virtual ImportXML& operator >> ( BinaryConFormula** f );
  virtual ImportXML& operator >> ( FormulaVariable** f );
  virtual ImportXML& operator >> ( Formula** f );
  virtual ImportXML& operator >> ( FormulaListElement** e );
  virtual ImportXML& operator >> ( FormulaList& l );
  virtual ImportXML& operator >> ( FormulaListVariable** f );
  virtual ImportXML& operator >> ( SElement** e );
  virtual ImportXML& operator >> ( SElementList** l );
  virtual ImportXML& operator >> ( Sequent** s );
  virtual ImportXML& operator >> ( Rule** r_ );
  virtual ImportXML& operator >> ( RuleItem** ruleitem );
  virtual ImportXML& operator >> ( Proof** p );
  virtual ImportXML& operator >> ( ProofDatabase& pdb );
  virtual ImportXML& operator >> ( ProofLink** pl );
  virtual ImportXML& operator >> ( SequentList** l );
};

}

#endif
