////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: import/opendbxml.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#include "util/myassert.h"
#include "libproof.h"
#include "util/proofexceptions.h"
#include "util/xmlread.h"
#include "util/proofdbdtd.h"
#include "symboltable.h"

#include "opendbxml.h"

namespace ProofLib
{

////////////////////////////////////////////////////////////////////////////////

class OpenDbXMLImpl : public OpenDbXML
{
  XmlReader             *m_xml_reader;

  Proof                 *m_first_proof;
  Proof                 *m_proof;

protected:
  OpenDbXMLImpl( ProofDatabase *pdb )
    : OpenDbXML( pdb )
  {
    m_xml_reader = NULL;
    m_first_proof = NULL;
    m_proof = NULL;
  }

  friend class OpenDbXML;

public:
  XmlReader* GetXmlReader() const { return m_xml_reader; }

  Proof* GetCurrentProof() const { return m_proof; }

  void SetCurrentProof( Proof *p )
  {
    ASSERT( p != NULL );
    m_proof = p;
  
    if( m_first_proof == NULL )
      m_first_proof = p;
  }

  virtual Proof* Read()
  {
    ASSERT( m_xml_reader != NULL );

    m_first_proof = NULL;
    m_proof = NULL;
#if 0
      if( PROOFDBDTD_DOCUMENT_TYPE_NODE == xmlTextReaderNodeType( m_xml_reader ) )
      {
        // ...
      }
#endif
    if( !GetXmlReader()->SeekToElement() )
      return NULL;

    (*this) >> *(GetProofDatabase());

    return m_first_proof;
  }

  virtual void Open( const char *sourcename )
  {
    ASSERT( m_xml_reader == NULL );

    m_xml_reader = XmlReader::New();
    m_xml_reader->Open( sourcename );
  }

  virtual void Close()
  {
    ASSERT( m_xml_reader != NULL );

    m_xml_reader->Close();
    delete m_xml_reader;
    m_xml_reader = NULL;
  }
};

////////////////////////////////////////////////////////////////////////////////

OpenDbXML::OpenDbXML( ProofDatabase* pdb )
  : ImportXML( pdb )
{
}

OpenDbXML* OpenDbXML::New( ProofDatabase* pdb )
{
  return new OpenDbXMLImpl( pdb );
}

////////////////////////////////////////////////////////////////////////////////

}
