////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: node.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// nodes
////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
#include <iostream>
#include <set>
#include <stack>
#include "util/hash.h"
#endif

#include "libproof.h"

namespace ProofLib
{

////////////////////////////////////////////////////////////////////////////////

SymbolNode::SymbolNode( const Symbol *e )
{
  m_symbolentry = e;
}

std::ostream& SymbolNode::ToString( std::ostream &out ) const
{
  out << GetName();
  return out;
}

////////////////////////////////////////////////////////////////////////////////

#ifdef DEBUG
size_t DataNode::m_dbg_new_count = 0;

std::set<DataNode*> DEBUG_DataNode_Table;
#endif

DataNode::DataNode()
{
#ifdef DEBUG
#if 0
  cout << "DATANODE CONST at " << (void *) this << endl;
#endif

  DEBUG_DataNode_Table.insert( this );
#endif
}

#ifdef DEBUG
DataNode::~DataNode()
{
#if 0
  cout << "DATANODE DEST at " << (void *) this << endl;
#endif

  DEBUG_DataNode_Table.erase( this );
}
#endif

#ifdef DEBUG
#ifndef LEAK_TEST
void* DataNode::operator new ( size_t size )
{
  void *ptr = malloc( size );
#if 0
  cout << "DATANODE NEW at " << ptr << " size " << size 
       << " #" << ++m_dbg_new_count << endl;
#endif
  return ptr;
}

void DataNode::operator delete ( void *ptr )
{
#if 0
  size_t size = 0;
  cout << "DATANODE DELETE at " << ptr << " size " << size
       << " #" << --m_dbg_new_count << endl;
#endif
  free( ptr );
}
#endif
#endif

bool DataNode::ContainsPointer( const DataNode *o ) const
{
  if( o == this )
    return true;

  const TreeInternalNode *t = dynamic_cast<const TreeInternalNode*>( this );

  if( t != NULL ) for( size_t i = 0; i < t->GetChildCount(); i++ )
  {
    if( t->GetChild( i )->ContainsPointer( o ) )
      return true;
  }

  return false;
}

////////////////////////////////////////////////////////////////////////////////

} // namespace ProofLib

