////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: nodedatabase.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Datastructures for Proofs
////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG
#include <iostream>
#include <set>
#include <stack>
#include "util/hash.h"
#endif

#include "util/myassert.h"
#include "libproof.h"

namespace ProofLib
{

////////////////////////////////////////////////////////////////////////////////

DbSymbolNode::DbSymbolNode( const Symbol *e )
 : SymbolNode( e )
{
}

bool DbSymbolNode::DbEqual( const DbNode *n, VarMapping var_map ) const
{
  ASSERT( n != NULL );

  const SymbolNode *s = dynamic_cast<const SymbolNode*>(n);

  if( s == NULL )
    return false;

  return ( GetSymbol() == (s->GetSymbol()) );
}

////////////////////////////////////////////////////////////////////////////////

void DbNode::Detach()
{
  if( m_ref_count > 0 )
    m_ref_count--;

  if( m_ref_count == 0 )
  {
    if( m_node_db != NULL )
      m_node_db->Remove( this );

    delete this;
  }
}

////////////////////////////////////////////////////////////////////////////////

void NodeDatabase::Remove( DbNode *n )
{
  ASSERT( n->m_ref_count == 0 );
#if 0
  std::cout << "remove " << (void*) n << ": ";
  dynamic_cast<DataNode*>(n)->ToString(std::cout);
  std::cout << std::endl;
#endif
  erase( n );

  TreeInternalNode *tn = dynamic_cast<TreeInternalNode*>( n );

  if( tn != NULL )
  {
    for( size_t i = 0; i < tn->GetChildCount(); i++ )
    {
      DbNode *c = dynamic_cast<DbNode*>( tn->GetChild(i) );
  
      if( c != NULL )
        c->Detach();
    }
  }
}

/*
Variable* NodeDatabase::NewVariable( const SymbolTableEntry *e )
{
  return GetOrAdd( new Variable( e ) );
}

Constant* NodeDatabase::NewConstant( const SymbolTableEntry *e )
{
  return GetOrAdd( new Constant( e ) );
}

Function* NodeDatabase::NewFunction( const SymbolTableEntry *e,
                                     TermList *terms )
{
  ASSERT( terms != NULL );

  Function *f = new Function( e, terms->GetChildCount() );

  f->SetTerms( terms );

  return GetOrAdd( f );
}

AtomFormula*
NodeDatabase::NewAtomFormula( const SymbolTableEntry *e, TermList *terms )
{
  AtomFormula *f;

  if( terms == NULL )
    f = new AtomFormula( e, 0 );
  else
  {
    f = new AtomFormula( e, terms->GetChildCount() );

    f->SetTerms( terms );
  }

  return GetOrAdd( f );
}

UnaryConFormula*
NodeDatabase::NewUnaryConFormula( const OperatorSymbol *sym, Formula *f )
{
  return GetOrAdd( new UnaryConFormula( sym, f ) );
}

QuantorFormula*
NodeDatabase::NewQuantorFormula( const OperatorSymbol *sym, Variable *v, Formula *f )
{
  return GetOrAdd( new QuantorFormula( sym, v, f ) );
}

BinaryConFormula*
NodeDatabase::NewBinaryConFormula( const OperatorSymbol *sym,
                                    Formula *fl, Formula *fr )
{
  return GetOrAdd( new BinaryConFormula( sym, fl, fr ) );
}
*/
////////////////////////////////////////////////////////////////////////////////

} // namespace ProofLib

