////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: operations_def.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _OPERATIONS_DEF_H_
#define _OPERATIONS_DEF_H_

namespace ProofLib
{
  class Operation;
  class CompareOperation;
}

////////////////////////////////////////////////////////////////////////////////

/// use this definition to delcare Apply in DataNode and other abstract
/// subclasses easily.
#define DECLARE_APPLY_ABSTRACT \
  virtual DataNode* Apply( Operation &op ) = 0; \
  virtual bool ApplyCompare( CompareOperation &op, const DataNode *other ) \
    const = 0;

/// use this definition to delcare Apply in DataNode subclasses easily.
#define DECLARE_APPLY \
  virtual DataNode* Apply( Operation &op ); \
  virtual bool ApplyCompare( CompareOperation &op, const DataNode *other ) const;

/// use this definition to implement Apply in DataNode subclasses easily.
#define IMPLEMENT_APPLY( CLASS, OPERATION_CLASS_TYPE ) \
  DataNode* CLASS::Apply( Operation &op_ ) \
  { \
      try \
      { \
	ExpressionOperation &op = \
	  dynamic_cast<ExpressionOperation&>( op_ ); \
	\
	return op.ApplyTo##CLASS( this ); \
      } \
      catch( std::bad_cast ) \
      { \
	ASSERT( false ); \
      } \
    return this; \
  } \
  bool CLASS::ApplyCompare( CompareOperation &op_, const DataNode *other ) \
    const \
  { \
    try \
    { \
      ExpressionCompareOperation &op = \
        dynamic_cast<ExpressionCompareOperation&>( op_ ); \
      \
      return op.ApplyTo##CLASS( this, other ); \
    } \
    catch( std::bad_cast ) \
    { \
      ASSERT( false );\
    } \
    return false; \
  }
////////////////////////////////////////////////////////////////////////////////

#endif
