////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: recursive.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _RECURSIVE_H_
#define _RECURSIVE_H_

#include "libproof.h"

namespace ProofLib
{

////////////////////////////////////////////////////////////////////////////////

class RecursiveExpr :
  public DbNode
{
  USTRING m_var;
  char m_op;
  unsigned int m_num;
  
protected:
  RecursiveExpr( const wchar_t *var, char op, unsigned int num );
  virtual ~RecursiveExpr() {};

  friend class NodeDatabase;

public:
  bool HasVariable() const { return !m_var.empty(); }
  const wchar_t* GetVariableName() const
    { return (m_var.empty() ? NULL : m_var.c_str()); }
  char GetOp() const { return m_op; }
  unsigned int GetNum() const { return m_num; }

  unsigned int GetValueAtLevel( unsigned int lvl ) const;

public:
  virtual bool DbEqual( const DbNode *n, VarMapping var_map ) const;
  virtual size_t Hash() const;

public:
  static RecursiveExpr* New( NodeDatabase *ndb, const wchar_t *var,
                             char op, unsigned int num );
};

////////////////////////////////////////////////////////////////////////////////

class RecursiveObject
{
  RecursiveExpr      *m_expr;

protected:
  RecursiveObject( RecursiveExpr *expr );
  virtual ~RecursiveObject() {};

public:
  virtual RecursiveExpr* GetRecursiveExpr() const { return m_expr; }
};

////////////////////////////////////////////////////////////////////////////////

class RecursiveVariable :
  public Variable,
  public RecursiveObject
{
protected:
  RecursiveVariable( const Symbol *e,
                     RecursiveExpr *expr );
  virtual ~RecursiveVariable() {};

  friend class NodeDatabase;

public:
  virtual bool DbEqual( const DbNode *n, VarMapping var_map ) const;
  virtual size_t Hash() const;

public:
  static RecursiveVariable* New( NodeDatabase *ndb, const Symbol *e,
                                 RecursiveExpr *expr );
};

////////////////////////////////////////////////////////////////////////////////

class RecursiveConstant :
  public Constant,
  public RecursiveObject
{
protected:
  RecursiveConstant( const Symbol *e,
                     RecursiveExpr *expr );
  virtual ~RecursiveConstant() {};

  friend class NodeDatabase;

public:
  virtual bool DbEqual( const DbNode *n, VarMapping var_map ) const;
  virtual size_t Hash() const;

public:
  static RecursiveConstant* New( NodeDatabase *ndb, const Symbol *e,
                                 RecursiveExpr *expr );
};

////////////////////////////////////////////////////////////////////////////////

class RecursiveFunction :
  public Function,
  public RecursiveObject
{
protected:
  RecursiveFunction( const Symbol *e,
                     int arity, RecursiveExpr *expr );
  virtual ~RecursiveFunction() {};

  friend class NodeDatabase;

public:
  virtual bool DbEqual( const DbNode *n, VarMapping var_map ) const;
  virtual size_t Hash() const;

  virtual RecursiveFunction* ReplaceTerms( AbstractTermList &terms ) const;

  virtual bool HaveEqualSymbol( const Function *o ) const;

public:
  static RecursiveFunction* New( NodeDatabase *ndb, const Symbol *e,
                                 AbstractTermList *terms, RecursiveExpr *expr );
};

////////////////////////////////////////////////////////////////////////////////

class RecursivePredicate : public AtomFormula, public RecursiveObject
{
protected:
  RecursivePredicate( const Symbol *e, int arity,
                      RecursiveExpr *expr  );
  virtual ~RecursivePredicate() {};

  friend class NodeDatabase;

public:
  virtual bool DbEqual( const DbNode *n, VarMapping var_map ) const;
  virtual size_t Hash() const;

  virtual RecursivePredicate* ReplaceTerms( AbstractTermList &terms ) const;

public:
  static RecursivePredicate* New( NodeDatabase *ndb, const Symbol *e,
                                  AbstractTermList *terms, RecursiveExpr *expr );
};

////////////////////////////////////////////////////////////////////////////////

}

#endif
