////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: symboltable.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _SYMBOLTABLE_H_
#define _SYMBOLTABLE_H_

#include "chars/chars.h"

#define SYMBOL_TYPE_NONE                    0
#define SYMBOL_TYPE_VARIABLE                1
#define SYMBOL_TYPE_CONSTANT                2
#define SYMBOL_TYPE_FUNCTION                3
#define SYMBOL_TYPE_PREDICATE               4
#define SYMBOL_TYPE_RULE                    5
#define SYMBOL_TYPE_PROOF                   6
#define SYMBOL_TYPE_JUNCTOR                 7
#define SYMBOL_TYPE_SEQUENTLIST             8

#define SYMBOL_TYPE_USER                    9 // first user value


#define OP_SYMBOL_NONE                      0
#define OP_SYMBOL_AND                       1
#define OP_SYMBOL_OR                        2
#define OP_SYMBOL_IMPL                      3
#define OP_SYMBOL_NEG                       4
#define OP_SYMBOL_ALL                       5
#define OP_SYMBOL_EX                        6
#define OP_SYMBOL_ALL2                      7
#define OP_SYMBOL_EX2                       8


#define OP_SYMBOL_USER                      9 // first user value

namespace ProofLib
{

  class AbstractTermListBase;
////////////////////////////////////////////////////////////////////////////////
// STANDARD SYMBOLS

typedef size_t SymbolType;

////////////////////////////////////////////////////////////////////////////////

class Symbol
{
public:
  virtual ~Symbol() {}

public:
  virtual const USTRING& GetName() const = 0;
  virtual size_t GetArity() const = 0;
  virtual SymbolType GetSymbolType() const = 0;

  virtual bool IsInfix() const { return false; } // default

  virtual bool IsDefined( const AbstractTermListBase *terms = NULL ) const
    { return false; }
  virtual const UCHAR* GetDefinitionId( const AbstractTermListBase *terms = NULL ) const
    { return L""; }

  virtual size_t Hash() const = 0;

public:
  // hash value
  size_t  operator() ( const Symbol &o ) const { return o.Hash(); }

  virtual bool operator== ( const Symbol &o ) const = 0;
};

}

namespace stlhash
{
  template<> struct hash<const ProofLib::Symbol*> {
    size_t operator()(const ProofLib::Symbol* x) const
      { return size_t(x) >> 2; }
  };
  template<> struct hash<ProofLib::Symbol*> {
    size_t operator()(const ProofLib::Symbol* x) const
      { return size_t(x) >> 2; }
  };
}

namespace ProofLib
{

////////////////////////////////////////////////////////////////////////////////
// OPERATOR SYMBOLS

typedef size_t OpSymbolType;

class OperatorSymbol : public Symbol
{
public:
  virtual ~OperatorSymbol() {}

public:
  virtual UCHAR GetDisplayChar() const = 0;

  virtual bool IsAndJunctor() const = 0;
  virtual bool IsOrJunctor() const = 0;
  virtual bool IsImplJunctor() const = 0;
  virtual bool IsNegJunctor() const = 0;

  virtual bool IsAllQuantor() const = 0;
  virtual bool IsExistsQuantor() const = 0;

  virtual bool IsSecondOrderAllQuantor() const = 0;
  virtual bool IsSecondOrderExistsQuantor() const = 0;
};

////////////////////////////////////////////////////////////////////////////////

class SymbolTable
{
public:
  virtual ~SymbolTable() {}

public:
  virtual const Symbol* GetOrAdd( const USTRING &name,
                                  size_t arity, SymbolType type ) = 0;
  virtual const Symbol* Get( const USTRING &name,
                             size_t arity, SymbolType type ) = 0;
  virtual const Symbol* Add( const USTRING &name,
                             size_t arity, SymbolType type ) = 0;
  virtual const OperatorSymbol* GetOperatorSymbol( OpSymbolType type ) = 0;
  virtual size_t GetArity( const USTRING &name, SymbolType type ) const = 0;
};

/*
class OperatorSymbolTable
{
public:
  virtual void Add( OperatorSymbol *sym ) = 0;
  virtual OperatorSymbol *Get( const char* name ) = 0;
};

extern OperatorSymbol *SymbolAndJunctor;
extern OperatorSymbol *SymbolOrJunctor;
extern OperatorSymbol *SymbolImplJunctor;
extern OperatorSymbol *SymbolNegJunctor;
extern OperatorSymbol *SymbolAllQuantor;
extern OperatorSymbol *SymbolExistsQuantor;
*/

}

#endif
