////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: util/exceptions.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _EXCEPTIONS_H_
#define _EXCEPTIONS_H_

#include <ostream>
#include <sstream>

////////////////////////////////////////////////////////////////////////////////

class Exception
{
  Exception *m_before;
public:
  Exception() { m_before = NULL; }
  virtual ~Exception() { if( m_before != NULL ) delete m_before; }

  virtual void PrintAll( std::ostream &out ) const
    { if( m_before != NULL ) { m_before->PrintAll( out ); out << std::endl; }
      Print( out ); }

  virtual void Print( std::ostream &out ) const = 0;

  void AppendTo( Exception *e ) { m_before = e; }
};

////////////////////////////////////////////////////////////////////////////////

class GenericException : public Exception
{
  std::ostringstream     m_stream;

public:
  virtual std::ostringstream& Stream() { return m_stream; }

  virtual void Print( std::ostream &out ) const
  {
    out << m_stream.str();
  }
};

////////////////////////////////////////////////////////////////////////////////

class InterruptException : public Exception
{
public:
  virtual void Print( std::ostream &out ) const { out << "InterruptException"; }
};

////////////////////////////////////////////////////////////////////////////////

class AssertException : public GenericException
{
public:
  AssertException( const char *cond, const char *file, int line )
  {
    Stream() << "Assertation failed: '" << cond << "' at "
             << file << ":" << line;
  }
};

////////////////////////////////////////////////////////////////////////////////

#endif
