////////////////////////////////////////////////////////////////////////////////
//
// Proof Library for C++ -------------------------------------------------------
//
// File: util/libxml2/xmlwriteimpl.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#include "util/myassert.h"
#include "util/fileexceptions.h"
#include "util/xmlwrite.h"

#include <libxml/xmlwriter.h>

///TESTINGONLY
#include <iostream>

////////////////////////////////////////////////////////////////////////////////

class LibXml2XmlWriter : public XmlWriter
{
  xmlTextWriterPtr       m_xml_writer;
  int                    m_compression;

public:
  virtual ~LibXml2XmlWriter() {}

  virtual void Open( const char *sourcename )
  {
    m_xml_writer = xmlNewTextWriterFilename( sourcename, m_compression );

    if( m_xml_writer == NULL )
      throw new FileOpenFailException( sourcename, errno );

    xmlTextWriterSetIndent( m_xml_writer, ( m_compression == 0 ? 1 : 0 ) );
  }

  virtual void Close()
  {
    ASSERT( m_xml_writer != NULL );

    xmlFreeTextWriter( m_xml_writer );
  }

  virtual void SetCompression( int level )
  {
    ASSERT( level >= 0 && level <= 9 );

    m_compression = level;
  }

  virtual void StartDocument( const char* xml_version, const char* enc )
  {
    xmlTextWriterStartDocument( m_xml_writer, xml_version, enc, NULL );
  }

  virtual void EndDocument()
  {
    xmlTextWriterEndDocument( m_xml_writer );
  }

  virtual void WriteDTD( const char* root_node, const char* pub_id,
                         const char* sys_id, const char* subset )
  {
    xmlTextWriterWriteDTD( m_xml_writer, (xmlChar*) root_node,
                           (xmlChar*) pub_id, (xmlChar*) sys_id,
                           (xmlChar*) subset );

    xmlTextWriterEndDTD( m_xml_writer );
  }

  virtual void OpenElement( const char* name )
  {
    xmlTextWriterStartElement( m_xml_writer, (xmlChar*) name );
  }

  virtual void CloseElement()
  {
    xmlTextWriterEndElement( m_xml_writer );
  }

  virtual void WriteAttribute( const char* name, const char* value )
  {
    xmlTextWriterWriteAttribute( m_xml_writer, (xmlChar*) name,
                                 (xmlChar*) value );
  }

  virtual void WriteAttribute( const char* name, const USTRING &value )
  {
    std::string val;

    UTF8FromUString( value, val );

    xmlTextWriterWriteAttribute( m_xml_writer, (xmlChar*) name,
                                 (xmlChar*) val.c_str() );
  }
};

XmlWriter* XmlWriter::New()
{
  return new LibXml2XmlWriter;
}

