////////////////////////////////////////////////////////////////////////////////
//
// ProofLib --------------------------------------------------------------------
//
// File: sloc.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#include <string.h>

#include <util/myassert.h>
#include <util/sloc.h>
#include <malloc.h>

namespace ProofLib
{

////////////////////////////////////////////////////////////////////////////////

SourceLocation::SourceLocation()
{
  m_file = NULL;
  m_line = 0;
  m_charno = 0;
}

SourceLocation::SourceLocation( const SourceLocation &o )
{
  m_file = NULL;
  SetSourceLocation( o );
}

SourceLocation::SourceLocation( const char* file, size_t line, size_t charno )
{
  m_file = NULL;
  SourceLocation::SetSourceLocation( file, line, charno );
}

SourceLocation& SourceLocation::operator=( const SourceLocation &o )
{
  SetSourceLocation( o );
  return *this;
}

////////////////////////////////////////////////////////////////////////////////

SourceLocation::~SourceLocation()
{
  if( m_file != NULL )
    free( (void*) m_file );
}

////////////////////////////////////////////////////////////////////////////////

void SourceLocation::SetSourceLocation( const char* file,
                                        size_t line, size_t charno )
{
  ASSERT( file != NULL );

  if( m_file != NULL )
    free( (void*) m_file );

  m_file = strdup( file );
  m_line = line;
  m_charno = charno;
}

void SourceLocation::SetSourceLocation( size_t line, size_t charno )
{
  m_line = line;
  m_charno = charno;
}

void SourceLocation::SetSourceLocation( size_t charno )
{
  m_charno = charno;
}

void SourceLocation::SetSourceLocation( const SourceLocation &o )
{
  SetSourceLocation( o.GetFile(), o.GetLine(), o.GetChar() );
}

////////////////////////////////////////////////////////////////////////////////

void SLocException::Print( std::ostream &out ) const
{
  if( GetLine() > 0 )
  {
    if( GetFile() != NULL )
      out << GetFile() << ":";
  
    out << GetLine();
  
    if( GetChar() > 0 )
      out << "," << GetChar();
    out << ": ";
  }

  out << "(error) ";
}

////////////////////////////////////////////////////////////////////////////////

}
