////////////////////////////////////////////////////////////////////////////////
//
// ProofLib --------------------------------------------------------------------
//
// File: sloc.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _SLOC_H_
#define _SLOC_H_

#include <util/exceptions.h>

////////////////////////////////////////////////////////////////////////////////

namespace ProofLib
{

////////////////////////////////////////////////////////////////////////////////

class SourceLocation
{
  char *m_file;
  size_t m_line, m_charno;

public:
  SourceLocation();
  SourceLocation( const SourceLocation &o );
  SourceLocation( const char* file, size_t line, size_t charno );
  ~SourceLocation();

  SourceLocation& operator=( const SourceLocation &o );

  void SetSourceLocation( const SourceLocation &o );
  void SetSourceLocation( const char* file,
                          size_t line, size_t charno );
  void SetSourceLocation( size_t line, size_t charno );
  void SetSourceLocation( size_t charno );

  const char* GetFile() const { return m_file; }
  size_t GetLine() const { return m_line; }
  size_t GetChar() const { return m_charno; }
};

////////////////////////////////////////////////////////////////////////////////

class SLocException : public Exception, public SourceLocation
{
public:
  SLocException( const SourceLocation *sloc )
    : SourceLocation( *sloc ) {}

  virtual void Print( std::ostream &out ) const;
};

////////////////////////////////////////////////////////////////////////////////

class SLocCoverException : public SLocException
{
  Exception *sub;

public:
  SLocCoverException( const SourceLocation *sloc, Exception *e )
    : SLocException( sloc ) { sub = e; }

  virtual void Print( std::ostream &out ) const
  {
    SLocException::Print( out );
    if( sub != NULL ) sub->Print( out );
  }
};

////////////////////////////////////////////////////////////////////////////////

class GenericSLocException : public SLocException
{
  std::ostringstream     m_stream;

public:
  GenericSLocException( const SourceLocation *sloc )
    : SLocException( sloc ) {}
  GenericSLocException( const SLocException &e )
    : SLocException( e ) {}

  virtual std::ostringstream& Stream() { return m_stream; }

  virtual void Print( std::ostream &out ) const
  {
    SLocException::Print( out );
    out << m_stream.str();
  }
};

////////////////////////////////////////////////////////////////////////////////

}

#endif
