////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: main.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <queue>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef DEBUG
#ifdef __linux
#include <mcheck.h>
#endif
#endif


#define MAIN_CONF_FILENAME "prooftool.conf.scm"

//#include "import/latexproof.h"
//#include "graphic/proofsheet.h"

#ifdef USE_X11
#include "graphic/x11/x11.h"
#else
#include "graphic/qt3/qt3impl.h"
//#include "graphic/gtkmm/gtkmmimpl.h"
#endif

#include "command/guile/globalcommandregister.h"
#include "command/commandprocessor.h"
#include "command/guile/guilecommandcall.h"
//#include "lisp/lispprocessor.h"
//#include "input/readlineinput.h"
#include "simplesymboltable.h"
#include "libproof/util/xmlread.h"
#include "libproof/util/proofdbdtd.h"
#include "import/opendbxml.h"

#include <libguile.h>

/*
#include <readline/readline.h>
#include <readline/history.h>
*/

#define HISTORY_FILE ".prooftool_hist"

#ifdef DEBUG
using namespace std;
#endif

#define HOME "HOME"
#define PROOFTOOL_HOME "PROOFTOOL_HOME"
const char *home_env = NULL;
const char *prooftool_home_env = NULL;

#ifdef __APPLE__
const char* base_path = NULL;
#endif

bool ExistsFile( const char *filename )
{
  ASSERT( filename != NULL );

  struct stat info;

  return ( stat( filename, &info ) == 0 );
}

#define DIR_SEP '/'
std::string GetRightPathFor( const char *std_location, const char *filename )
{
  size_t len_home = strlen( HOMEDIR );
  ASSERT( strlen(std_location) > len_home+1 );

  std::string rel = &std_location[len_home+1];
#if 0
std::cerr << "GetRightPathFor - rel: " << rel << std::endl;
std::cerr << "GetRightPathFor - HOMEDIR: " << HOMEDIR << std::endl;
#endif

  std::string full = HOMEDIR;
  full += DIR_SEP;
  full += rel;
  full += DIR_SEP;
  full += filename;

  if( ExistsFile( full.c_str() ) )
  {
#if 0
std::cerr << "GetRightPathFor : " << full << std::endl;
#endif
    return full;
  }

  if( home_env != NULL )
  {
#if 0
std::cerr << "GetRightPathFor - home_env: " << home_env << std::endl;
#endif
    full = home_env;
    full += DIR_SEP;
    full += ".prooftool";
    full += DIR_SEP;
    full += rel;
    full += DIR_SEP;
    full += filename;
  
    if( ExistsFile( full.c_str() ) )
    {
#if 0
std::cerr << "GetRightPathFor : " << full << std::endl;
#endif
      return full;
    }
  }

  if( prooftool_home_env != NULL )
  {
#if 0
std::cerr << "GetRightPathFor - prooftool_home_env: " << prooftool_home_env << std::endl;
#endif
    full = prooftool_home_env;
    full += DIR_SEP;
    full += rel;
    full += DIR_SEP;
    full += filename;
  
    if( ExistsFile( full.c_str() ) )
    {
#if 0
std::cerr << "GetRightPathFor : " << full << std::endl;
#endif
      return full;
    }
  }

#ifdef __APPLE__
  if( base_path != NULL )
  {
#if 0
std::cerr << "GetRightPathFor - base_path: " << base_path << std::endl;
#endif
    full = base_path;
    full += DIR_SEP;
    full += rel;
    full += DIR_SEP;
    full += filename;
  
    if( ExistsFile( full.c_str() ) )
    {
#if 0
std::cerr << "GetRightPathFor : " << full << std::endl;
#endif
      return full;
    }
  }
#endif

#if 0
std::cerr << "GetRightPathFor : EMPTY" << std::endl;
#endif
  return "";
}

std::string GetPathForFont( const char* font_filename )
{
  return GetRightPathFor( FONTDIR, font_filename );
}

std::string GetPathForDTD( const char* dtd_filename )
{
  return GetRightPathFor( DTDDIR, dtd_filename );
}

std::string GetPathForConf( const char* conf_filename )
{
  return GetRightPathFor( CONFDIR, conf_filename );
}

class GuileCommandProcessor : public CommandProcessor
{
  TCondition             m_cond_input_ready;
  TMutex                 m_mutex_queue;
  std::queue<GuileCommandCall*> m_queue;
  bool m_catched_scm_err;

  bool m_in_user_input;

public:
  void Start( int argc, char *argv[] )
  {
    scm_boot_guile(argc, argv, &GuileCommandProcessor::inner_main, this);
  }

private:
  static void inner_main(void *closure,int argc, char **argv)
  {
    ((GuileCommandProcessor*) closure)->Run();
  }

  static SCM catcher( void* in, SCM tag, SCM throw_args )
  {
#ifdef DEBUG
    std::cout << "* CATCHER *" << std::endl;
    scm_write( tag, scm_current_output_port() );
    scm_putc( '\n', scm_current_output_port() );
    scm_write( throw_args, scm_current_output_port() );
    scm_putc( '\n', scm_current_output_port() );
#endif
    SCM name = SCM_CAR( throw_args );
    SCM format = SCM_CAR( SCM_CDR( throw_args ) );
    SCM args = SCM_CAR( SCM_CDR( SCM_CDR( throw_args ) ) );

    SCM msg = format;

    if( args != SCM_BOOL_F )
      msg = scm_simple_format( SCM_BOOL_F, format, args );

    SCM str_port = scm_open_output_string();

    if( name != SCM_BOOL_F )
    {
      scm_display( name, str_port );
      scm_putc( ':', str_port );
      scm_putc( ' ', str_port );
    }
    scm_display( msg, str_port );
    //scm_putc( '\n', str_port );

    SCM str = scm_get_output_string( str_port );
    scm_close_output_port( str_port );

    if( GetContext()->GetConsole() != NULL )
      GetContext()->GetConsole()->ShowError( SCM_STRING_CHARS( str ) );

    if( in != NULL )
    {
      GuileCommandProcessor *inst = (GuileCommandProcessor*) in;
      inst->m_catched_scm_err = true;
    }

    return SCM_UNSPECIFIED;
  }
  
public:
  virtual void Boot( int argc, char* argv[] )
  {
    m_in_user_input = false;

    RegisterGlobalCommands();

#if 0
std::cerr << "BOOT" << std::endl;
#endif
    scm_internal_catch(
      SCM_BOOL_T,
      (scm_t_catch_body) scm_c_primitive_load,
      (void*) GetPathForConf(MAIN_CONF_FILENAME).c_str(),
      catcher,
      NULL );
  }

  virtual void Startup( int argc, char* argv[] )
  {
    std::ostringstream init_call;

    init_call << "(startup-prooftool (list ";
    for( int i = 0; i < argc; i++ )
    {
      init_call << '"' << argv[i] << '"';
      if( i + 1 < argc )
        init_call << ' ';
    }
    init_call << "))";

#if 0
std::cerr << "HIER: " << init_call.str() << std::endl;
#endif
    scm_internal_catch(
      SCM_BOOL_T,
      (scm_t_catch_body) scm_c_eval_string,
      (void*) init_call.str().c_str(),
      catcher,
      NULL );
  }

protected:
  virtual int Run()
  {
#if 0
    Lock();

    RegisterGlobalCommands();

#if 0
    scm_write( scm_compile_shell_switches( argc, argv ), scm_current_output_port() );
    scm_putc( '\n', scm_current_output_port() );
#endif

    scm_internal_catch(
      SCM_BOOL_T,
      (scm_t_catch_body) scm_c_primitive_load,
      (void*) "prooftool.conf.scm",
      catcher,
      NULL );

    while(1)
    {
      m_cond_input_ready.WaitOnMutex( *this );

      bool b_queue_empty = true;
      do
      {
        m_mutex_queue.Lock();
        if( m_queue.empty() )
        {
          m_mutex_queue.Unlock();
          break;
        }
        GuileCommandCall *cmd = m_queue.front();
        m_mutex_queue.Unlock();

#ifdef DEBUG
        std::cout << "TO PROCESS: " << cmd << std::endl;
        scm_write( cmd->GetCallSCM(), scm_current_output_port() );
        scm_putc( '\n', scm_current_output_port() );
#endif

        //SCM res = scm_c_eval_string( cmd.c_str() );
        m_catched_scm_err = false;

        GetContext()->GetGraphicSystem()->GetMainInputDispatcher()->
          IndicateProcessorWorking();

        SCM res = scm_internal_catch(
          SCM_BOOL_T,
          (scm_t_catch_body) scm_primitive_eval,
          cmd->GetCallSCM(),
          (scm_t_catch_handler) catcher,
          this );
              //  scm_c_eval_string, cmd.c_str(), catcher, this );

        GetContext()->GetGraphicSystem()->GetMainInputDispatcher()->
          IndicateProcessorWorking( false );

        if( !m_catched_scm_err )
        {
#ifdef DEBUG
          std::cout << "RETURN FROM PROCESS " << cmd << ": ";
          std::cout.flush();
#endif
          cmd->SetResultSCM( res );

          SCM str_port = scm_open_output_string();
          scm_write( res, str_port ); //scm_current_output_port() );
          SCM str = scm_get_output_string( str_port );
          scm_close_output_port( str_port );

          cmd->SetResponse( SCM_STRING_CHARS( str ) );

#ifdef DEBUG
          scm_display( str, scm_current_output_port() );
          scm_putc( '\n', scm_current_output_port() );
#endif
        }

        if( cmd->HaveToWaitForResult() )
        {
          cmd->GetCondition()->Broadcast();
        }
        else
          delete cmd;

        m_mutex_queue.Lock();
        m_queue.pop();
        b_queue_empty = m_queue.empty();
        m_mutex_queue.Unlock();
      }
      while( !b_queue_empty );
    }

    Unlock();

#endif
    return 0;
  }

public:
  virtual void Process( CommandCall *cmd )
  {
    if( cmd == NULL )
      return;

    Process( dynamic_cast<GuileCommandCall*>( cmd ) );
  }

  virtual void Process( GuileCommandCall *cmd )
  {
#if 0
    if( cmd == NULL )
      return;

    TCondition *wait_cond = NULL;

    if( cmd->HaveToWaitForResult() )
    {
      wait_cond = new TCondition;
      cmd->SetCondition( wait_cond );
    }

    m_mutex_queue.Lock();
    m_queue.push( cmd );
    m_mutex_queue.Unlock();

    if( wait_cond != NULL )
    {
      Lock();
      m_cond_input_ready.Broadcast();
      wait_cond->WaitOnMutex( *this );
      delete wait_cond;
      Unlock();
    }
    else
      m_cond_input_ready.Broadcast();
#endif

/**/

#ifdef DEBUG
    std::cout << "TO PROCESS: " << cmd << std::endl;
    scm_write( cmd->GetCallSCM(), scm_current_output_port() );
    scm_putc( '\n', scm_current_output_port() );
#endif

    m_catched_scm_err = false;

//  GetContext()->GetGraphicSystem()->GetMainInputDispatcher()->
//    IndicateProcessorWorking();

    SCM res = scm_internal_catch(
      SCM_BOOL_T,
      (scm_t_catch_body) scm_primitive_eval,
      cmd->GetCallSCM(),
      (scm_t_catch_handler) catcher,
      this );
          //  scm_c_eval_string, cmd.c_str(), catcher, this );

//  GetContext()->GetGraphicSystem()->GetMainInputDispatcher()->
//    IndicateProcessorWorking( false );

    if( !m_catched_scm_err )
    {
#ifdef DEBUG
      std::cout << "RETURN FROM PROCESS " << cmd << ": ";
      std::cout.flush();
#endif
      cmd->SetResultSCM( res );

      SCM str_port = scm_open_output_string();
      scm_write( res, str_port ); //scm_current_output_port() );
      SCM str = scm_get_output_string( str_port );
      scm_close_output_port( str_port );

      cmd->SetResponse( SCM_STRING_CHARS( str ) );

      if( cmd->HaveToShowResultInConsole() )
        GetContext()->GetConsole()->ShowMessage( SCM_STRING_CHARS( str ) );

#ifdef DEBUG
      scm_display( str, scm_current_output_port() );
      scm_putc( '\n', scm_current_output_port() );
#endif
    }
  }

  void BeginUserInput()
  {
    if( m_in_user_input )
    {
      GenericException *e = new GenericException();
      e->Stream() << "Already waiting for user input.";
      throw e;
    }

    m_in_user_input = true;
  }

  void EndUserInput()
  {
    ASSERT( m_in_user_input );
    m_in_user_input = false;
  }

  SCM ReadString( const char *str )
  {
    return scm_internal_catch(
      SCM_BOOL_T,
      (scm_t_catch_body) scm_c_read_string,
      (void*) str,
      (scm_t_catch_handler) catcher,
      NULL );
  }

  virtual void Process( const char *str, bool show_in_console )
  {
    try
    {
      SCM scmd = ReadString( str );

      if( scmd != SCM_UNSPECIFIED ) // parsing failed
      {
        GuileCommandCall call( show_in_console );
        call.SetWaitForResult( true );
        call.SetSCM( scmd );

        Process( &call );
  
        std::cout << call.GetResponse() << std::endl;
      }
    }
    catch( ... )
    {
      std::cerr << "Unknown Exception was thrown." << std::endl;
    }
  }
};

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#if 0
class ReadLineInput : public TThread
{
private:
  static ReadLineInput *m_sigt;
  jmp_buf m_readline_read_mark;
  GuileCommandCall *m_call;

  static void sig_int_handler( int sig )
  {
    if( TThread::CurrentThread() != m_sigt )
    {
      m_sigt->Kill( SIGINT );
      return;
    }

    m_sigt->HandleIntSignal();
  }

  void HandleIntSignal()
  {
    if( m_sigt->m_call != NULL )
    {
      if( HaveToQuit() )
      {
        write_history( HISTORY_FILE );
        errno = 0; // ignore error
        Exit();
      }

      std::cout << "* CANCEL ACTION *" /*<< ": " << sig*/ << std::endl;

      GetContext()->GetGraphicSystem()->GetMainInputDispatcher()->
        AbortDialog();
      GetContext()->GetGraphicSystem()->GetMainInputDispatcher()->
        AbortSelectItem();
    }
    else
    {
      rl_cleanup_after_signal();
      std::cout << std::endl;

      if( HaveToQuit() )
      {
        write_history( HISTORY_FILE );
        errno = 0; // ignore error
        Exit();
      }
      else
        //std::cout << "* LONG JUMP *" << std::endl;
        longjmp( m_sigt->m_readline_read_mark, SIGINT );
    }
  }

public:
  virtual void Quit()
  {
    TThread::Quit();
    Kill( SIGINT );
  }

protected:
  virtual void Init()
  {
    m_sigt = this;
    m_call = NULL;
  }
  
  virtual int Run()
  {
#if 0
    //scm_sigaction( SCM_INUM(SCM_INT_SIGNAL), scm_c_eval_string("(lambda (x) (display x))"), SCM_INUM(6) );
    //scm_sigaction( SIGINT, sig_int_handler, SCM_BOOL_F );
    scm_c_eval_string("(use-modules (ice-9 readline))");
    scm_c_eval_string("(activate-readline)");
    scm_c_eval_string("(set-repl-prompt! \"> \")");
    scm_shell(argc, argv);
#endif
    GuileCommandProcessor *processor =
      dynamic_cast<GuileCommandProcessor*>(GetContext()->GetCommandProcessor());

    ASSERT( processor != NULL );

    using_history();
    read_history( HISTORY_FILE );
    errno = 0; // ignore error

    while(1)
    {
      rl_catch_signals = 0;
      setjmp( m_readline_read_mark );
      signal( SIGINT, sig_int_handler );

      char *ret = readline( "> " );
#ifdef DEBUG
      std::cout << "readline-done" << std::endl;
#endif

      if( ret == NULL )
      {
#ifdef DEBUG
        std::cout << "readline-exit" << std::endl;
#endif
        ret = (char*) malloc( sizeof(char) * ( strlen( QUIT_COMMAND ) + 3 ) );
        sprintf( ret, "(%s)", QUIT_COMMAND );
      }
  
      if( ret[0] == '\0' )
      {
        free( ret );
        continue;
      }
  
      try
      {
        SCM scmd = processor->ReadString( ret );

        if( scmd != SCM_UNSPECIFIED ) // parsing failed
        {
          GuileCommandCall call;
          call.SetWaitForResult( true );
          call.SetSCM( scmd );

          m_call = &call;
  
          processor->Process( &call );
    
          m_call = NULL;

          std::cout << call.GetResponse() << std::endl;

          add_history( ret );
        }
      }
      catch( ... )
      {
        std::cerr << "Unknown Exception was thrown." << std::endl;
      }
  
      free( ret );
    }

  }
};


ReadLineInput* ReadLineInput::m_sigt = NULL;
#endif


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

class TestExecContext : public ExecContext
{
  GraphicSystem            *m_gsys;
  GuileCommandProcessor     m_processor;
  ProofLib::ProofDatabase  *m_pdb;
  GraphicWindow            *m_pwnd;
  CommandConsole           *m_console;

public:
  TestExecContext()
  {
    m_pdb = NULL;
    m_pwnd = NULL;
    m_gsys = NULL;
    m_console = NULL;
  }

  virtual ~TestExecContext()
  {
  }

  virtual void SetGraphicSystem( GraphicSystem *gsys )
  {
    ASSERT( m_gsys == NULL );

    m_gsys = gsys;
  }
  
  virtual GraphicSystem* GetGraphicSystem()
  {
    return m_gsys;
  }
  
  virtual void SetProofDatabase( ProofLib::ProofDatabase *db )
  {
    m_pdb = db;

    if( GetContext()->GetProofWindow() == NULL )
      return;

    GraphicView* v = GetContext()->GetProofWindow()->GetView();

    if( v == NULL )
      return;

    GraphicProofView *pv = dynamic_cast<GraphicProofView*>( v );

    if( pv == NULL || pv->GetProofSheet() == NULL )
      return;

    pv->GetProofSheet()->SetProofDatabase( m_pdb );
  }

  virtual ProofLib::ProofDatabase* GetProofDatabase() const
  {
    return m_pdb;
  }

  virtual ProofLib::NodeDatabase* GetNodeDatabase() const
  {
    return GetProofDatabase()->GetNodeDatabase();
  }
  
  virtual ProofLib::SymbolTable* GetSymbolTable() const
  {
    return GetProofDatabase()->GetSymbolTable();
  }

  virtual void SetProofWindow( GraphicWindow *wnd )
  {
    if( m_pwnd != NULL )
      delete m_pwnd;

    m_pwnd = wnd;
  }

  virtual GraphicWindow* GetProofWindow() const
  {
    return m_pwnd;
  }

  virtual CommandProcessor* GetCommandProcessor()
  {
    return &m_processor;
  }

  virtual CommandConsole* GetConsole() const
  {
    return m_console;
  }

  virtual void SetConsole( CommandConsole* console )
  {
    ASSERT( console != NULL );

    m_console = console;
  }
};

TestExecContext ctxt;

ExecContext *GetContext()
{
  return &ctxt;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

//static ReadLineInput rli;

void Shutdown()
{
  ctxt.GetGraphicSystem()->GetMainInputDispatcher()->Quit();
 // rli.Quit();

  //ctxt.GetGraphicSystem()->GetMainInputDispatcher()->Join();
 // rli.Join();

  exit(0);
}

static void inner_main( void *closure, int argc, char **argv )
{
  ProofLib::ProofDatabase *pdb = NULL;

  try
  {
    GraphicSystem *gs = new QtGraphicSystem( argc, argv );

    ctxt.SetGraphicSystem( gs );

    pdb = new ProofLib::ProofDatabase( new SimpleSymbolTable() );

    ctxt.SetProofDatabase( pdb );

    ProofSheet *sheet = new ProofSheet( ctxt.GetProofDatabase() );
    sheet->SetFontManager( gs->GetFontManager() );

#if 0
    gs->GetMainInputDispatcher()->Lock();
    gs->GetMainInputDispatcher()->Unlock();
    
    rli.Start();

    GuileCommandProcessor *gcp = dynamic_cast<GuileCommandProcessor*>
      ( GetContext()->GetCommandProcessor() );
    ASSERT( gcp != NULL );
    gcp->Start( argc, argv );
#endif
    {
#if 0
    QMainWindow *mw = new QMainWindow;
    mw->setCaption( "Main Window" );
    mw->show();
#endif
    }
    {
/*
    QMainWindow *mw = new QMainWindow;
    QSplitter *split = new QSplitter( mw );
    split->setOrientation( QSplitter::Vertical );

    QTextEdit *log = new QTextEdit( split, "log" );
    log->setReadOnly( true );
    log->setTextFormat( Qt::LogText );

    QVBox *editpane = new QVBox( split );
    QTextEdit *edit = new QTextEdit( editpane, "editor" );
    edit->setTextFormat( Qt::PlainText );
    QPushButton *execbutton = new QPushButton( "Execute", editpane );

    split->setResizeMode( log, QSplitter::Stretch );
    split->setResizeMode( editpane, QSplitter::KeepSize );

    edit->setFocus();
    mw->setCaption( "Console" );
    mw->setCentralWidget( split );
    mw->show();
*/
    }

    GuileCommandProcessor *gcp = dynamic_cast<GuileCommandProcessor*>
      ( GetContext()->GetCommandProcessor() );

    GraphicWindow *wnd = gs->NewProofWindow( sheet );
    GetContext()->SetProofWindow( wnd );

    QtConsole *con = new QtConsole( gcp, dynamic_cast<QtGraphicWindow*>(wnd) );
    GetContext()->SetConsole( con );
    con->Show();

    gcp->Boot( argc, argv );
    gcp->Startup( argc, argv );

    ctxt.GetGraphicSystem()->GetMainInputDispatcher()->Start();
  }
  catch( ... )
  {
    std::cerr << "Unknown Exception was thrown." << std::endl;
  }

  /*
  if( pdb != NULL )
    delete pdb;
    */
}

int main( int argc, char *argv[], char *envp[] )
{
#ifdef DEBUG
#ifdef __linux
// mtrace();
#endif
#endif

  size_t len_prooftool_home = strlen( PROOFTOOL_HOME );
  size_t len_home = strlen( HOME );

  for( size_t i = 0; envp[i] != NULL; i++ )
  {
    if( strncmp( envp[i], PROOFTOOL_HOME, len_prooftool_home ) == 0 )
    {
      if( envp[i][len_prooftool_home] != '=' )
        continue;

      prooftool_home_env = strdup( &envp[i][len_prooftool_home+1] );
    }

    if( strncmp( envp[i], HOME, len_home ) == 0 )
    {
      if( envp[i][len_home] != '=' )
        continue;

      home_env = strdup( &envp[i][len_home+1] );
    }
  }
#ifdef __APPLE__
  for( size_t i = 0; envp[i] != NULL; i++ )
  {
    if( envp[i][0] == '_' && envp[i][1] == '=' )
    {
      base_path = strdup( &envp[i][2] );

      char* h = strrchr( base_path, '/' );
      if( h != NULL ) *h = '\0';
      h = strrchr( base_path, '/' );
      if( h != NULL ) *h = '\0';

      break;
    }
  }
#endif
  XmlReader::Init();

  std::string dtd_file = GetPathForDTD("xml/4.0/proofdatabase.dtd");

  if( !dtd_file.empty() )
    XmlReader::AddCatalogSystemItem( PROOFDBDTD_DTD_PROOFDATABASE,
                                     dtd_file.c_str() );
    
  std::cout.sync_with_stdio( true );

  scm_boot_guile( argc, argv, inner_main, NULL );

  return 0;
}

