;; ZOOM FUNCTIONS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (zoom-setfactor f) 
  (zoom-factor
    f
    (car (get-zoom-mid-point))
    (cdr (get-zoom-mid-point))
  )
)

(define (zoom-in) 
  (zoom-factor
    (* (get-zoom-factor) 1.5) 
    (car (get-zoom-mid-point))
    (cdr (get-zoom-mid-point))
  )
)

(define (zoom-out) 
  (zoom-factor
    (/ (get-zoom-factor) 1.5) 
    (car (get-zoom-mid-point))
    (cdr (get-zoom-mid-point))
  )
)

(define (zoom-normal) 
  (zoom-factor
    1.0
    (car (get-zoom-mid-point))
    (cdr (get-zoom-mid-point))
  )
)

(define (move-up x)
  (zoom-factor
    (get-zoom-factor)
    (car (get-zoom-mid-point))
    (+ (cdr (get-zoom-mid-point)) x)
  )
)

(define (move-left x)
  (zoom-factor
    (get-zoom-factor)
    (- (car (get-zoom-mid-point)) x)
    (cdr (get-zoom-mid-point))
  )
)

;; KEY BINDINGS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define key_shift 1)
(define key_ctrl  2)
(define key_alt   4)
(define key_meta  8)

(bind-key (char->integer #\A) 0 '(lambda () (zoom-all)))
(bind-key (char->integer #\+) 0 '(lambda () (zoom-in)))
(bind-key (char->integer #\-) 0 '(lambda () (zoom-out)))
(bind-key (char->integer #\=) 1 '(lambda () (zoom-normal)))

(bind-key 65362 0 '(lambda () (move-up  30)))
(bind-key 65364 0 '(lambda () (move-up -30)))
(bind-key 65361 0 '(lambda () (move-left  30)))
(bind-key 65363 0 '(lambda () (move-left -30)))

(bind-key (char->integer #\u) 0 '(lambda () (unsplit-proof (select-prooflink))))

;; MENUES ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (make-prooflist-menu-items)
  (map
    (lambda (y) (make-menu-item y (lambda () (show-proof y))))
    (sort (list-proofs) string<?)
  )
)

(define (make-sequentlistlist-menu-items)
  (map
    (lambda (y) (make-menu-item y (lambda () (show-sequentlist y))))
    (sort (list-sequentlists) string<?)
  )
)

(define (make-prooflist-menu)
  (make-menu-item "Show Proof" 
    (primitive-eval (cons (quote make-menu) (make-prooflist-menu-items)))
  )
)

(define (make-sequentlistlist-menu)
  (make-menu-item "Show Sequent Lists" 
    (primitive-eval (cons (quote make-menu) (make-sequentlistlist-menu-items)))
  )
)

(define (list-subst-options)
  '(
      ("at this position" . 'here)
      ("in the sequent" . 'sequent)
      ("in the sequent and above" . 'above)
      ("in whole proof" . 'proof)
  )
)

(define (make-subst-options-menu func)
  (map
    (lambda (y)
      (let ((f (func y)))
        (make-menu-item (car y) (lambda () (primitive-eval f)))
      )
    )
    (list-subst-options)
  )
)

;; open file

(define (open-dlg func msg)
  (let ((filename (dlg-file msg 'open)))
    (cond
      ((string? filename)
        (chdir (dirname filename))
        (show-proof (func filename))
      )
    )
  )
)

;; open file

(define (save-dlg func msg)
  (let ((filename (dlg-file msg 'save)))
    (cond
      ((string? filename)
        (chdir (dirname filename))
        (func filename)
      )
    )
  )
)

(define (save-dlg-opt func opt msg)
  (let ((filename (dlg-file msg 'save)))
    (cond
      ((string? filename)
        (chdir (dirname filename))
        (func opt filename)
      )
    )
  )
)

;; Main menu
(clear-menu)
(add-menu "File"
  '(make-menu
    (make-menu-item "New Database" (lambda () (new-db))
      key_ctrl (char->integer #\N)
    )
    (make-menu-item "Open Database" (lambda ()
        (open-dlg open-db "Select database file")
      ) key_ctrl (char->integer #\O)
    )
    (make-menu-item "Save Database" (lambda ()
        (save-dlg save-db "Choose database file")
      ) key_ctrl (char->integer #\S)
    )
    (make-separator-menu-item)
    (make-menu-item "Import Proof"
      (make-menu
        (make-menu-item "XML"
          (lambda () (open-dlg import-proof-xml "Select XML proof file"))
        )
        (make-menu-item "Otter"
          (lambda () (open-dlg import-otter "Select Otter file"))
        )
      )
    )
    (make-separator-menu-item)
    (make-menu-item "Export Proof"
      (make-menu
        (make-menu-item "XML"
          (primitive-eval (cons (quote make-menu)
            (map
              (lambda (y) (make-menu-item y (lambda ()
                (save-dlg-opt export-proof-xml y "Choose XML file")
              )))
              (sort (list-proofs) string<?)
            )
          ))
        )
        (make-menu-item "LaTeX"
          (primitive-eval (cons (quote make-menu)
            (map
              (lambda (y) (make-menu-item y (lambda ()
                (save-dlg-opt export-proof-latex y "Choose LaTeX file")
              )))
              (sort (list-proofs) string<?)
            )
          ))
        )
      )
    )
    (make-menu-item "Export Sequent List"
      (make-menu
        (make-menu-item "LaTeX"
          (primitive-eval (cons (quote make-menu)
            (map
              (lambda (y) (make-menu-item y (lambda ()
                (save-dlg-opt export-sequentlist-latex y "Choose LaTeX file")
              )))
              (sort (list-sequentlists) string<?)
            )
          ))
        )
      )
    )
    (make-menu-item "Export Database"
      (make-menu
        (make-menu-item "LaTeX" (lambda ()
          (save-dlg export-latex "Choose LaTeX file")
        ))
      )
    )
    (make-separator-menu-item)
    (make-menu-item "Quit" (lambda () (quit))
      key_ctrl (char->integer #\Q))
  )
)
(add-menu "Edit"
  '(make-menu
    (make-menu-item "Split Proof"
      (lambda () (split-proof (dlg-input) (select-sequent)))
    )
    (make-menu-item "Unsplit Proof"
      (lambda () (unsplit-proof (select-prooflink)))
    )
    (make-separator-menu-item)
    (make-menu-item "Substitute Formula"
      (primitive-eval (cons (quote make-menu) (make-subst-options-menu
        (lambda (x)
          (list 'subst-formula '(dlg-input) '(select-formula) (cdr x))
        )
      )))
    )
    (make-menu-item "Unsubstitute Formula"
      (primitive-eval (cons (quote make-menu) (make-subst-options-menu
        (lambda (x)
          (list 'unsubst-formula '(select-formula-var) (cdr x))
        )
      )))
    )
    (make-menu-item "Substitute S-Elements"
      (primitive-eval (cons (quote make-menu) (make-subst-options-menu
        (lambda (x)
          (list 'subst-selements '(dlg-input)
                '(select-selement) '(select-selement) (cdr x))
        )
      )))
    )
    (make-menu-item "Unsubstitute Formula List Variable"
      (primitive-eval (cons (quote make-menu) (make-subst-options-menu
        (lambda (x)
          (list 'unsubst-selements '(select-sformula-list-var) (cdr x))
        )
      )))
    )
  )
)
(add-menu "View"
  '(make-menu
    (make-menu-item "Zoom in" (lambda () (zoom-in))
      key_ctrl (char->integer #\+) )
    (make-menu-item "Zoom out" (lambda () (zoom-out))
      key_ctrl (char->integer #\-) )
    (make-menu-item "Zoom all" (lambda () (zoom-all))
      key_ctrl (char->integer #\9) )
    (make-menu-item "Zoom standard" (lambda () (zoom-normal))
      key_ctrl (char->integer #\0) )
    (make-separator-menu-item)
    (make-prooflist-menu)
    (make-sequentlistlist-menu)
  )
)

;; Contextmenues
(bind-button -1 0 '(lambda (x) 
  (popup-menu
    (cond
      ( (prooflink? x)
        (make-menu
          (make-prooflist-menu)
          (make-separator-menu-item)
          (make-menu-item "Unsplit Proof" (lambda () (unsplit-proof x)))
        )
      )
      ( (sequent? x)
        (make-menu
          (make-prooflist-menu)
          (make-separator-menu-item)
          (make-menu-item "Split Proof" (lambda () (split-proof (dlg-input) x)))
        )
      )
      ( (formula-var-ref? x)
        (make-menu
          (make-prooflist-menu)
          (make-separator-menu-item)
          (make-menu-item "Unsubstitute Formula"
            (primitive-eval (cons (quote make-menu) (make-subst-options-menu
              (lambda (option)
                (list 'unsubst-formula x (cdr option))
              )
            )))
          )
        )
      )
      ( (formula-ref? x)
        (make-menu
          (make-prooflist-menu)
          (make-separator-menu-item)
          (make-menu-item "Substitute Formula"
            (primitive-eval (cons (quote make-menu) (make-subst-options-menu
              (lambda (option)
                (list 'subst-formula '(dlg-input) x (cdr option))
              )
            )))
          )
        )
      )
      ( (selement-ref? (sequent-item-get-parent x))
        (make-menu
          (make-prooflist-menu)
          (make-separator-menu-item)
          (make-menu-item "Unsubstitue S-Elements"
            (primitive-eval (cons (quote make-menu) (make-subst-options-menu
              (lambda (option)
                (list 'unsubst-selements (sequent-item-get-parent x)
                      (cdr option))
              )
            )))
          )
        )
      )
      ( else
        (make-menu
          (make-prooflist-menu)
        )
      )
    )
    3 ;; use same button number!!
  )
))

;; BUTTON BINDINGS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(bind-button 1 0 '(lambda (x) 
;;(if (prooflink? x) (show-proof (get-proofname x)))
  (cond
    ((prooflink? x) (show-proof (get-proofname x)))
;;  ((formula-ref? x) (subst-formula (dlg-input) x 'proof))
  )
))

(bind-button 1 1 '(lambda (x) 
  (if (prooflink? x) (unsplit-proof x))
))

;; STARTUP CODE ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (startup-prooftool args)
  (if (not (null? (cdr args)))
    (show-proof (open-db (car (cdr args))))
  )
)

;; LAZY TESTING ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;(show-proof (import-proof-xml "../examples/xml/n6.xml.gz"))

