////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./simplesymboltable.cpp
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Symboltable for all used symbols in Proofs
////////////////////////////////////////////////////////////////////////////////
//#include <iostream> // TEST
//std::cerr << i->second->GetName().c_str() << " != " << name.c_str() << std::endl;
#include "util/myassert.h"
#include "simplesymboltable.h"
#include <typeinfo>

////////////////////////////////////////////////////////////////////////////////

SimpleOperatorSymbol::SimpleOperatorSymbol()
{
  m_sym = '?';
  m_arity = 0;
}

SimpleOperatorSymbol::SimpleOperatorSymbol( const SimpleOperatorSymbol &o )
{
  m_name = o.m_name;
  m_sym = o.m_sym;
  m_arity = o.m_arity;
}

SimpleOperatorSymbol::SimpleOperatorSymbol( const USTRING &name,
                                            ProofLib::OpSymbolType type,
                                            UCHAR sym, size_t arity )
{
  ASSERT( !name.empty() );
  m_name = name;
  m_sym = sym;
  m_arity = arity;
  m_type = type;
}

bool SimpleOperatorSymbol::operator== ( const Symbol &o_0 ) const
{
  try
  {
    const SimpleOperatorSymbol &o =
      dynamic_cast<const SimpleOperatorSymbol&>( o_0 );
 
    if( m_arity != o.m_arity )
      return false;
  
    if( m_name != o.m_name )
      return false;
  }
  catch( std::bad_cast )
  {
    return false;
  }

  return true;
}

size_t SimpleOperatorSymbol::Hash() const
{
  size_t h = 0;

  h = m_arity;

  if( m_name.empty() )
    return h;

  const UCHAR *p = m_name.c_str();

  size_t h_c = 0;
  while( *p ) h_c = ( h_c << 1 ) ^ *(p++);

  return (h_c << 3) ^ h;
}


#if 0
OperatorSymbol::OperatorSymbol( OperatorSimpleSymbolTable* table,
                                const char *name, UCHAR sym )
{
  ASSERT( table != NULL );
  ASSERT( sym != 0 );
  ASSERT( name != NULL );
  ASSERT( strlen(name) > 0 );

  m_name = name;
  m_sym  = sym;

  table->Add( this );
}

void OperatorSimpleSymbolTable::Add( OperatorSymbol *sym )
{
  ASSERT( (*this)[sym->GetName().c_str()] == NULL );

  (*this)[sym->GetName().c_str()] = sym;
}

OperatorSimpleSymbolTable globalOperatorSimpleSymbolTable;

OperatorSymbol *SymbolAndJunctor
  = new OperatorSymbol( &globalOperatorSimpleSymbolTable, "and", CHAR_AND );
OperatorSymbol *SymbolOrJunctor
  = new OperatorSymbol( &globalOperatorSimpleSymbolTable, "or", CHAR_OR );
OperatorSymbol *SymbolImplJunctor
  = new OperatorSymbol( &globalOperatorSimpleSymbolTable, "impl", CHAR_IMPL );
OperatorSymbol *SymbolNegJunctor
  = new OperatorSymbol( &globalOperatorSimpleSymbolTable, "neg", CHAR_NEG );
OperatorSymbol *SymbolAllQuantor
  = new OperatorSymbol( &globalOperatorSimpleSymbolTable, "all", CHAR_ALL );
OperatorSymbol *SymbolExistsQuantor
  = new OperatorSymbol( &globalOperatorSimpleSymbolTable, "exists", CHAR_EXISTS );

OperatorSimpleSymbolTable *GetGlobalOperatorSimpleSymbolTable()
{
  return &globalOperatorSimpleSymbolTable;
}

////////////////////////////////////////////////////////////////////////////////

static SimpleSymbolTable _glbsymtbl;

SimpleSymbolTable *GetGlobalSimpleSymbolTable()
{
  return &_glbsymtbl;
}
#endif

////////////////////////////////////////////////////////////////////////////////

SimpleSymbol::SimpleSymbol()
{
  m_arity = 0;
  m_type  = SYMBOL_TYPE_NONE;
}

SimpleSymbol::SimpleSymbol( const SimpleSymbol& other )
{
  m_name  = other.m_name;
  m_arity = other.m_arity;
  m_type  = other.m_type;
}

SimpleSymbol::SimpleSymbol( const USTRING& name, size_t arity,
                            ProofLib::SymbolType type )
{
  ASSERT( name.length() > 0 );

  m_name  = name;
  m_arity = arity;
  m_type  = type;
}

size_t SimpleSymbol::Hash() const
{
  size_t h = 0;

  h = m_arity;
  h = (h << 1) ^ m_type;
#if 0
  h = m_type << (sizeof(size_t)-1)*8;
  h += m_arity << (sizeof(size_t)-2)*8;
#endif
#if 0
  h = m_arity;

  while( h & m_type != 0 )
  {
    h = h << 1;
  }
  h += m_type;
#endif

  if( m_name.empty() )
    return h;

  const UCHAR *p = m_name.c_str();

  size_t h_c = 0;
  while( *p ) h_c = ( h_c << 1 ) ^ *(p++);

#if 0
fprintf( stderr, "hash: %s/%d (type=%d): %ld", m_name, m_arity, m_type, h );
#endif
  return (h_c << 3) ^ h;
}

bool SimpleSymbol::operator== ( const ProofLib::Symbol &o_0 ) const
{
  try
  {
    const SimpleSymbol &o = dynamic_cast<const SimpleSymbol&>( o_0 );
 
    if( m_type != o.m_type )
      return false;
  
    if( m_arity != o.m_arity )
      return false;
  
    if( m_name != o.m_name )
      return false;
  }
  catch( std::bad_cast )
  {
    return false;
  }

  return true;
}

bool SimpleSymbol::IsInfix() const
{
  if( GetArity() != 2 )
    return false;

  if( GetSymbolType() == SYMBOL_TYPE_FUNCTION &&
      ( GetName()[0] == '+' ||
        GetName()[0] == '-' ||
        GetName()[0] == '*' ||
        GetName()[0] == '/' ||
        GetName()[0] == CHAR_ISIN ||
        GetName()[0] == CHAR_NOTIN ||
        GetName()[0] == CHAR_NI ||
        GetName()[0] == CHAR_MINUS ||
        GetName()[0] == CHAR_CAP ||
        GetName()[0] == CHAR_CUP ||
        GetName()[0] == CHAR_LE ||
        GetName()[0] == CHAR_GE ||
        GetName()[0] == CHAR_SUB ||
        GetName()[0] == CHAR_SUP ||
        GetName()[0] == CHAR_NSUB ||
        GetName()[0] == CHAR_SUBE ||
        GetName()[0] == CHAR_SUPE ||
        GetName()[0] == CHAR_OPLUS ||
        GetName()[0] == CHAR_OTIMES ) )
    return true;

  if( GetSymbolType() == SYMBOL_TYPE_PREDICATE &&
      ( GetName()[0] == '=' ||
        GetName()[0] == '<' ||
        GetName()[0] == '>' ||
        GetName()[0] == '!' ||
        GetName()[0] == CHAR_ISIN ||
        GetName()[0] == CHAR_NOTIN ||
        GetName()[0] == CHAR_NI ||
        GetName()[0] == CHAR_CAP ||
        GetName()[0] == CHAR_CUP ||
        GetName()[0] == CHAR_LE ||
        GetName()[0] == CHAR_GE ||
        GetName()[0] == CHAR_SUB ||
        GetName()[0] == CHAR_SUP ||
        GetName()[0] == CHAR_NSUB ||
        GetName()[0] == CHAR_SUBE ||
        GetName()[0] == CHAR_SUPE ) )
    return true;

  return false;
}

////////////////////////////////////////////////////////////////////////////////

class SimpleSymbolTableStore
{
public:
  typedef stlhash::hash_map<SimpleSymbol, SimpleSymbol*, SimpleSymbol> Table;
  typedef stlhash::hash_map<USTRING, SimpleOperatorSymbol*> OpTable;
  typedef std::vector<SimpleOperatorSymbol*> OpArray;

  Table table;
  OpTable op_table;
  OpArray op_array;

public:
  SimpleSymbolTableStore() : op_array( OP_SYMBOL_USER ) {}
};

////////////////////////////////////////////////////////////////////////////////

SimpleSymbolTable::SimpleSymbolTable()
{
  m_store = new SimpleSymbolTableStore;


#define ADD_OP_SYMBOL( NAME, TYPE, CHAR, ARITY ) \
{ \
  SimpleOperatorSymbol *o = \
    new SimpleOperatorSymbol( NAME, TYPE, CHAR, ARITY ); \
  m_store->op_table[NAME] = o; \
  m_store->op_array[TYPE] = o; \
}

  ADD_OP_SYMBOL( L"and", OP_SYMBOL_AND, CHAR_AND, 2 );
  ADD_OP_SYMBOL( L"or", OP_SYMBOL_OR, CHAR_OR, 2 );
  ADD_OP_SYMBOL( L"impl", OP_SYMBOL_IMPL, CHAR_IMPL, 2 );
  ADD_OP_SYMBOL( L"neg", OP_SYMBOL_NEG, CHAR_NEG, 1 );
  ADD_OP_SYMBOL( L"all", OP_SYMBOL_ALL, CHAR_ALL, 1 );
  ADD_OP_SYMBOL( L"exists", OP_SYMBOL_EX, CHAR_EXISTS, 1 );
  ADD_OP_SYMBOL( L"all2", OP_SYMBOL_ALL2, CHAR_ALL, 1 );
  ADD_OP_SYMBOL( L"exists2", OP_SYMBOL_EX2, CHAR_EXISTS, 1 );

#undef ADD_OP_SYMBOL
}

SimpleSymbolTable::~SimpleSymbolTable()
{
  delete m_store;
}

ProofLib::Symbol* SimpleSymbolTable::Get( const USTRING &name,
                                          size_t arity,
                                          ProofLib::SymbolType type )
{
  if( type == SYMBOL_TYPE_JUNCTOR )
  {
    SimpleSymbolTableStore::OpTable::const_iterator i =
      m_store->op_table.find( name );

    if( i == m_store->op_table.end() )
      return NULL;

    return i->second;
  }

  SimpleSymbol e( name, arity, type );
  
  SimpleSymbolTableStore::Table::const_iterator i = m_store->table.find( e );

  if( i == m_store->table.end() )
    return NULL;

#if 0
fprintf( stderr, "symtbl: %s/%d (type=%d)", name, arity, type );
fprintf( stderr, " (old)\n" );
#endif

  return i->second;
}

ProofLib::Symbol* SimpleSymbolTable::Add( const USTRING &name,
                                          size_t arity,
                                          ProofLib::SymbolType type )
{
  if( type == SYMBOL_TYPE_JUNCTOR )
    return NULL; // this type we can not add dynamically

  ProofLib::Symbol *e_0 = Get( name, arity, type );

  if( e_0 != NULL )
    return NULL; // exists already

  SimpleSymbol *e = new SimpleSymbol( name, arity, type );
  
  m_store->table[*e] = e;
#if 0
fprintf( stderr, "symtbl: %s/%d (type=%d)", name, arity, type );
fprintf( stderr, " (new)\n" );
#endif
  return e;
}

ProofLib::Symbol* SimpleSymbolTable::GetOrAdd( const USTRING &name,
                                               size_t arity,
                                               ProofLib::SymbolType type )
{
  ProofLib::Symbol *e = Get( name, arity, type );

  if( e != NULL )
    return e;

  return Add( name, arity, type );
#if 0
  if( type == SYMBOL_TYPE_JUNCTOR )
  {
    SimpleSymbolTableStore::OpTable::const_iterator i =
      m_store->op_table.find( name );

    if( i == m_store->op_table.end() )
      return NULL;

    return i->second;
  }

  SimpleSymbol *e = new SimpleSymbol( name, arity, type );
  
  SimpleSymbolTableStore::Table::const_iterator i = m_store->table.find( *e );

  if( i == m_store->table.end() )
  {
    m_store->table[*e] = e;
#if 0
fprintf( stderr, "symtbl: %s/%d (type=%d)", name, arity, type );
fprintf( stderr, " (new)\n" );
#endif
    SimpleSymbol f = *e;
    return e;
  }

  delete e;
#if 0
fprintf( stderr, "symtbl: %s/%d (type=%d)", name, arity, type );
fprintf( stderr, " (old)\n" );
#endif

  return i->second;
#endif
}

template<class T>
static size_t GetArity( T &table, const USTRING &name,
                        ProofLib::SymbolType type )
{
  typename T::const_iterator i = table.begin();
  for( ; i != table.end(); i++ )
  {
    if( i->second->GetName() != name )
      continue;

    if( i->second->GetSymbolType() != type )
      continue;

    return i->second->GetArity();
  }

  ASSERT( false ); // do better
  return 0;
}

size_t SimpleSymbolTable::GetArity( const USTRING &name,
                                    ProofLib::SymbolType type ) const
{
  if( type == SYMBOL_TYPE_JUNCTOR )
    return ::GetArity( m_store->op_table, name, type );
  else
    return ::GetArity( m_store->table, name, type );
}

ProofLib::OperatorSymbol*
SimpleSymbolTable::GetOperatorSymbol( ProofLib::OpSymbolType type )
{
  ASSERT( type < m_store->op_array.size() );
  return m_store->op_array[type];
}
