////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./simplesymboltable.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Symboltable for all used symbols in Proofs
////////////////////////////////////////////////////////////////////////////////
#ifndef _SYMBOLTABLE_XX__H_
#define _SYMBOLTABLE_XX__H_

#include <string>

#include "util/hash.h"
#include "chars/chars.h"
#include "symboltable.h"

////////////////////////////////////////////////////////////////////////////////

#if 0
typedef enum SymbolType
{
  SymTypeVariable,
  SymTypeFunction,
  SymTypePredicate,
  SymTypeFormula,
  SymTypeFormulaList,
  SymTypeProof
};
#endif

class SimpleSymbol : public ProofLib::Symbol
{
private:
  USTRING      m_name;
  size_t       m_arity;
  ProofLib::SymbolType   m_type;

public:
  SimpleSymbol();
  SimpleSymbol( const SimpleSymbol& other );
  SimpleSymbol( const USTRING &name, size_t arity, ProofLib::SymbolType type );
  virtual ~SimpleSymbol() {};

  virtual const USTRING& GetName() const { return m_name; }
  virtual size_t GetArity() const { return m_arity; }
  virtual ProofLib::SymbolType GetSymbolType() const { return m_type; }

  virtual bool IsInfix() const;

  virtual size_t Hash() const;

public:
  virtual bool operator== ( const Symbol &o ) const;
};

////////////////////////////////////////////////////////////////////////////////

class SimpleSymbolTableStore;

class SimpleSymbolTable : public ProofLib::SymbolTable
{
  SimpleSymbolTableStore *m_store;

public:
  SimpleSymbolTable();
  virtual ~SimpleSymbolTable();

  virtual ProofLib::Symbol *Get( const USTRING &name,
                                 size_t arity, ProofLib::SymbolType type );
  virtual ProofLib::Symbol *Add( const USTRING &name,
                                 size_t arity, ProofLib::SymbolType type );
  virtual ProofLib::Symbol *GetOrAdd( const USTRING &name,
                                      size_t arity, ProofLib::SymbolType type );
  virtual ProofLib::OperatorSymbol*
    GetOperatorSymbol( ProofLib::OpSymbolType type );
  virtual size_t GetArity( const USTRING &name,
                           ProofLib::SymbolType type ) const ;
};

////////////////////////////////////////////////////////////////////////////////

//SymbolTable *GetGlobalSymbolTable();

////////////////////////////////////////////////////////////////////////////////

#if 0
class OperatorSymbolTable;
#endif

class SimpleOperatorSymbol : public ProofLib::OperatorSymbol
{
  USTRING                m_name;
  UCHAR                  m_sym;
  size_t                 m_arity;
  ProofLib::OpSymbolType m_type;

public:
  SimpleOperatorSymbol();
  SimpleOperatorSymbol( const SimpleOperatorSymbol &other );
  SimpleOperatorSymbol( const USTRING &name, ProofLib::OpSymbolType type,
                        UCHAR sym, size_t arity );
  virtual ~SimpleOperatorSymbol() {}

  virtual const USTRING& GetName() const { return m_name; }
  virtual size_t GetArity() const { return m_arity; }
  virtual ProofLib::SymbolType GetSymbolType() const
    { return SYMBOL_TYPE_JUNCTOR; }

  virtual size_t Hash() const;

  virtual UCHAR GetDisplayChar() const { return m_sym; }

  virtual bool IsAndJunctor() const { return m_type == OP_SYMBOL_AND; }
  virtual bool IsOrJunctor() const { return m_type == OP_SYMBOL_OR; }
  virtual bool IsImplJunctor() const { return m_type == OP_SYMBOL_IMPL; }
  virtual bool IsNegJunctor() const { return m_type == OP_SYMBOL_NEG; }
  virtual bool IsAllQuantor() const { return m_type == OP_SYMBOL_ALL; }
  virtual bool IsExistsQuantor() const { return m_type == OP_SYMBOL_EX; }
  virtual bool IsSecondOrderAllQuantor() const { return m_type == OP_SYMBOL_ALL2; }
  virtual bool IsSecondOrderExistsQuantor() const { return m_type == OP_SYMBOL_EX2; }

public:
  virtual bool operator== ( const Symbol &o ) const;
};

#if 0
class OperatorSymbolTable : protected
 stlhash::hash_map<const char*,OperatorSymbol*,stlhash::hash<const char*>,eqstr>
{
public:
  void Add( OperatorSymbol *sym );
  OperatorSymbol *Get( const char* name ) { return (*this)[name]; }
};

OperatorSymbolTable *GetGlobalOperatorSymbolTable();

extern OperatorSymbol *SymbolAndJunctor;
extern OperatorSymbol *SymbolOrJunctor;
extern OperatorSymbol *SymbolImplJunctor;
extern OperatorSymbol *SymbolNegJunctor;
extern OperatorSymbol *SymbolAllQuantor;
extern OperatorSymbol *SymbolExistsQuantor;
#endif

////////////////////////////////////////////////////////////////////////////////

#endif
