////////////////////////////////////////////////////////////////////////////////
//
// ProofTool -------------------------------------------------------------------
//
// File: ./util/threads.h
// Copyright 2005-2006 by Hendrik Spohr, Vienna, AUSTRIA
//
////////////////////////////////////////////////////////////////////////////////
#ifndef _THREADS_H_
#define _THREADS_H_

#include "util/myassert.h" // as long we have here unimplemented stuff...

////////////////////////////////////////////////////////////////////////////////

class TMutex //: public ITMutex
{
private:

public:
  TMutex() {};
  virtual ~TMutex() {};

  virtual void Lock() {};

  virtual bool TryLock() { return true; };
  virtual void Unlock() {};
};

class TThread //: public ITThread
{
public:
  TThread() {};
  virtual ~TThread() {};

  void Start() { Run(); };
  void* Join() { return NULL; };
  virtual void Quit() {};
  void Kill( int signal ) {};

public:
  static TThread* CurrentThread() { return NULL; };

protected:
  virtual void Init() = 0;
  virtual int Run() = 0;

  bool HaveToQuit() const { return false; };
  void Exit( void *value_ptr = NULL ) {};
};

class TCondition //: public ITCondition
{
public:
  TCondition() {};
  virtual ~TCondition() {};

  virtual void Signal() {};
  virtual void Broadcast() {};
  virtual void WaitOnMutex( TMutex &m ) {};
  virtual void WaitTimeoutOnMutex( TMutex &m, unsigned long sec ) {};
};

template<class T> class TSyncVar
{
private:
  T                      m_var;
  mutable TMutex         m_mutex;

public:
  void Set( const T &v )
  {
    m_mutex.Lock();
    m_var = v;
    m_mutex.Unlock();
  }  
  const T& Get() const
  {
    m_mutex.Lock();
    const T &v = m_var;
    m_mutex.Unlock();
    return v;
  }  

};
#endif
